#! /bin/sh

# /*
# *********************************************************************************
# *             restore.sh : Robocup 3D Soccer Simulation Team Zigorat            *
# *                                                                               *
# *  Date: 01/24/2009                                                             *
# *  Author: Mahdi Hamdarsi                                                       *
# *  Comments: A not simple script to restore a previously backup up project      *
# *                                                                               *
# *********************************************************************************
# */

PROJECT_DIR="/home/Klaus/Projects/Zigorat"
PROJECT_BASE_DIR="/home/Klaus/Projects"
BACKUP_DIR="${PROJECT_DIR}/.backups"
OLD_BACKUP_DIR="${PROJECT_DIR}/.backups/Old Backups"
TEMP_DIR="/home/Klaus"

if test $# -eq 2; then
  MONTH=$1
  DAY=$2
elif test $# -eq 1; then
  if test $1 == "--help"; then
    echo "Script used to restore a valid backup"
    exit 0
  fi

  echo "Unknown option: $1"
  exit 1
elif test $# -eq 0; then
  echo -n "Enter month and day of the backup (mm dd): "
  read MONTH DAY
else
  echo "Unknown number of options: $#"
  exit 1
fi

if test ! -n ${MONTH}; then
  echo "Could not continue, month not set"
  exit 1
fi

if test ! -n ${DAY}; then
  echo "Could not continue, day not set"
  exit 1
fi

FILE="${BACKUP_DIR}/`date +%Y`-${MONTH}-${DAY}.tar.gz"

if test ! -e "${FILE}"; then
  FILE="${OLD_BACKUP_DIR}/`date +%Y`-${MONTH}-${DAY}.tar.gz"

  if test ! -e "${FILE}"; then
    echo "Backup of ${DAY}/${MONTH}/`date +%Y` was not found"
    exit 1;
  fi
fi

echo "Backup found at: ${FILE}"

# Regular unpacking
COPIED_FILE="${TEMP_DIR}/`date +%Y-${MONTH}-${DAY}`.tar.gz"
cp "${FILE}" "${COPIED_FILE}"
tar -xf ${COPIED_FILE} -C ${TEMP_DIR}
rm -f ${COPIED_FILE}
mv "${TEMP_DIR}/Zigorat-Backup" "${TEMP_DIR}/Zigorat"
mkdir -p "${TEMP_DIR}/Zigorat/.backups"
mkdir -p "${TEMP_DIR}/Zigorat/.backups/Old Backups"
cp ${BACKUP_DIR}/*.gz ${TEMP_DIR}/Zigorat/.backups/
cp "${OLD_BACKUP_DIR}"/*.gz ${TEMP_DIR}/Zigorat/.backups/Old\ Backups/

# The dangerous part
rm -rf ${PROJECT_DIR}
mv ${TEMP_DIR}/Zigorat ${PROJECT_BASE_DIR}

echo "Backup successfully restored"

echo -n "Do you want to compile the project? (yes/no default is yes): "
read answer
if test ${answer} == "no"; then
  exit 0;
fi

cd ${PROJECT_DIR}
cmake .
make -j2

echo "Project successfully compiled"

#END
