/*
 *********************************************************************************
 *          BasicAgent.cpp : Robocup 3D Soccer Simulation Team Zigorat           *
 *                                                                               *
 *  Date: 03/20/2007                                                             *
 *  Author: Mahdi Hamdarsi                                                       *
 *  Comments: This file contains class definition BasicAgent which contains      *
 *            several routines needed for thinkin' module                        *
 *                                                                               *
 *********************************************************************************
 */

/*! \file BasicAgent.cpp
 <pre>
 <b>File:</b>          BasicAgent.cpp
 <b>Project:</b>       Robocup Soccer Simulation Team: Zigorat
 <b>Authors:</b>       Mahdi Hamdarsi
 <b>Created:</b>       03/20/2007
 <b>Last Revision:</b> $ID$
 <b>Contents:</b>      This file contains class definition BasicAgent which contains
 several routines needed for thinkin' module
 <hr size=2>
 <h2><b>Changes</b></h2>
 <b>Date</b>             <b>Author</b>          <b>Comment</b>
 03/20/2007       Mahdi           Initial version created
 </pre>
 */

#include "BasicAgent.h"
#include "Logger.h"

using namespace std;

/*! This is the constructor of BasicAgent, initilizes self
 variables, sends identification signal to the server
 and starts the agent
 */
BasicAgent::BasicAgent() {
	countor = 0;
	rneck = false;
	rneck_f = false;
	sb = false;
	state=1;
	Hstate = nothing;
	Angle_Turn=0;
	this_action = nothing ; 	//abc 14
	l_action = l_nothing;
	m_IsFirst = true ;	//abc 15
	m_Finished = false;
	connection = RoboCupConnection::getSingleton();
	WM = WorldModel::getSingleton();
	ACT = ActHandler::getSingleton();
	AS = AgentSettings::getSingleton();
	Log = Logger::getSingleton();
	fallFront=false;
	fallBack=false;
	fallGhalt=false;
	GOAL=UnknownPosition;
	OBJECT=OBJECT_PLAYER_UNKNOWN;

}

/*! This is the destructor of the basicAgent claass, fres all the instances
 of planners and deletes them all.
 */
BasicAgent::~BasicAgent() {
	finalizeLearning();
}

/*! This method inits agents information by loading agents scene graph
 into WorldModel and then it will send scene command to the simulator */
bool BasicAgent::initAgentScene() {
	// Load agent information like joints and sizes into WorldModel
	// This may take a while, so do it before initializing the simulator
	AgentModelT model = AS->getAgentModel();
	if (!WM->setAgentModel(model))
		return false;

	// Send scene command to init the agent
	// use the scene effector to build the agent and beam to strategic position
	char strScene[128] = "";
	sprintf(strScene, "(scene %s)",
			SoccerTypes::getAgentModelString(model).c_str());
	ACT->sendMessage(strScene);

	bool joint_used[MAX_JOINTS];
	for (unsigned i = 0; i < MAX_JOINTS; i++)
		joint_used[i] = SoccerTypes::isJointBeingUsed((JointT) i);
	ACT->setUsedJoints(joint_used);
	return true;
}

/**
 * This method beams agent to his current strategic position
 * @param angBody Body angle when beamed to the strategic position
 */
void BasicAgent::moveToStrategicPosition(AngDeg angBody) {
	VecPosition posBeam = WM->getStrategicPosition();
	beam(posBeam[0], posBeam[1], angBody);
	//beam(4,4,angBody);
}

/*! This method inits the agent for server by sending his name and number */
void BasicAgent::sendPlayerInformation() {
	static int iFirstCycle = WM->getCurrentCycle();
	if ((WM->getCurrentCycle() - iFirstCycle) == 1) {
		char cmd[128];
		sprintf(cmd, "(init (unum %d)(teamname %s))", WM->getAgentNumber(),
				WM->getTeamName().c_str());
		ACT->sendMessage(cmd);
	} /*else if ((WM->getCurrentCycle() - iFirstCycle) == 10)
		moveToStrategicPosition();*/
}

/**
 * This method just says something to indicate that agent
 has finished execution in the normal way.
 */
void BasicAgent::sayGoodbye() {
	if (WM->getAgentNumber() == 1)
		cout << "Goalie out." << endl;
	else
		cout << "#" << WM->getAgentNumber() << " out." << endl;

	Log->log(6, "Agent execution finished");
}

/*! This method sends a beam command to server: A beam command is a command
 that moves a robot in less than a moment to the desired location (not
 in the gameplay)
 \param dx X-Coordinate of desired move location
 \param dy Y-Coordinate of desired move location
 \param dAng Agent direction when has moved to the deisred location */
void BasicAgent::beam(double dx, double dy, double dAng) {
	if (WM->canAgentBeam()) {
		char str[128];
		sprintf(str, "(beam %f %f %f)", dx, dy, dAng);
		ACT->sendMessage(str);
	}
}

/**
 * This method sends a string to be broadcasted to all teammates
 * @param str String containing the talk message
 */
void BasicAgent::say(const char * str) {
	ACT->putCommandInQueue(SoccerCommand(CMD_SAY, str));
}

/*! This function directly adjustes joints angular speeds by
 the values given
 \param id Joint ID to be updated
 \param dSpeedMotor1 Joints first motors angular speed
 \param dSpeedMotor2 Joints second motors angular speed
 \return bool indicating whether information were accepted or not
 */
bool BasicAgent::adjustJoint(JointT id, double dSpeedMotor1,
		double dSpeedMotor2) {
	if (dSpeedMotor1 == UnknownDoubleValue || id == JID_ILLEGAL)
		return false;

	if (dSpeedMotor2 == UnknownDoubleValue)
		dSpeedMotor2 = 0;

	////////////////////////////////////////////////////German change
	double dsm1 = dSpeedMotor1;
	double dsm2 = dSpeedMotor2;
	ACT->putCommandInQueue(SoccerCommand(CMD_JOINT, id, dsm1,
				dsm2));
//	ACT->putCommandInQueue(SoccerCommand(CMD_JOINT, id, dsm1/2,
//			dsm2/2));
	return true;
}

/*--------------------------------------------------------------------------*/
/*----------------------- Changing of hinge joints -------------------------*/
/*--------------------------------------------------------------------------*/
////////////2
bool BasicAgent::neck(AngDeg valueFront, AngDeg valueOpen, double dSpeed ) {
	//	bool bResult;
	//
	//	if (dSpeed == -1)
	//		dSpeed = AS->getJointGain();
	//
	//	AgentModelT model = WM->getAgentModel();
	//	AngDeg curAngleFront, curAngleOpen;
	//	AngDeg angFront, angOpen;
	//	const TSceneJoint * j1, *j2;
	//
	//	if (model == MODEL_NAO) {
	//		j1 = WM->getJoint(JID_HEAD_2);
	//		j2 = WM->getJoint(JID_HEAD_1);
	//		if (!j1 || !j2) {
	//			Log->log(7, "(BasicAgent::neck) Nao head joint is not found");
	//			return false;
	//		}
	//		curAngleFront = j1->getCurrentAngle();
	//		curAngleOpen = j2->getCurrentAngle();
	//	} else {
	//		Log->log(7, "(BasicAgent::neck) Agent model is not supported");
	//		return false;
	//	}
	//
	//	// First Front/Back
	//	angFront = dSpeed * (valueFront - curAngleFront);
	//	if (fabs(curAngleFront - valueFront) > AS->getMaxJointError()) {
	//		bResult = false;
	//	} else {
	//		bResult = true;
	//	}
	//
	//	// Then Open/Close
	//	angOpen = (valueOpen - curAngleOpen) * dSpeed / 100;
	//	if (fabs(curAngleOpen - valueOpen) > AS->getMaxJointError()) {
	//		bResult &= false;
	//	} else {
	//		bResult &= true;
	//	}
	//
	//	if (model == MODEL_NAO) {
	//		adjustJoint(JID_HEAD_2, angFront);
	//		adjustJoint(JID_HEAD_1, angOpen);
	//	}
	//
	//	return bResult;

	bool bResult;

	if (dSpeed == -1)
		dSpeed = AS->getJointGain();

	AgentModelT model = WM->getAgentModel();
	AngDeg curAngleFront, curAngleOpen;
	AngDeg angFront, angOpen;
	const TSceneJoint * j1, *j2;

	if (model == MODEL_NAO) {
		j1 = WM->getJoint(JID_HEAD_2);
		j2 = WM->getJoint(JID_HEAD_1);
		if (!j1 || !j2) {
			Log->log(7, "(BasicAgent::hip) Nao hip joint is not found");
			return false;
		}
		curAngleFront = j1->getCurrentAngle();
		curAngleOpen = j2->getCurrentAngle();
		cout << "Current angle: , value: , max error: " << curAngleFront << ' '
				<< valueFront << ' ' << AS->getMaxJointError() << endl;
	} else {
		Log->log(7, "(BasicAgent::hip) Agent model is not supported");
		return false;
	}

	// First Front/Back
	angFront = (valueFront - curAngleFront) * dSpeed / 100;
	if (fabs(curAngleFront - valueFront) > AS->getMaxJointError()) {
		bResult = false;
	} else {
		bResult = true;
	}

	// Then Open/Close
	angOpen = (valueOpen - curAngleOpen) * dSpeed / 100;
	if (fabs(curAngleOpen - valueOpen) > AS->getMaxJointError()) {
		bResult &= false;
	} else {
		bResult &= true;
	}

	if (model == MODEL_NAO) {
		adjustJoint(JID_HEAD_2, angFront);
		adjustJoint(JID_HEAD_1, angOpen);
	}
	return bResult;

}
/*! This method changes the angle of First joint of leg (Hip), if
 currently  value  is near the  current angle of  knee then no
 changes are made.
 \param side which leg to change angle
 \param value new angle to change to
 \param dSpeed speed to set for action */
bool BasicAgent::rotateLeg(SideT side, AngDeg value, double dSpeed) {
	if (dSpeed == -1)
		dSpeed = AS->getJointGain();

	JointT joint = (side == SIDE_LEFT) ? JID_LLEG_1 : JID_RLEG_1;
	const TSceneJoint* j = WM->getJoint(joint);
	if (!j) {
		Log->log(7, "(BasicAgent::rotateLeg) Leg rotation joint does not exist");
		return false;
	}

	AngDeg curAngle = j->getCurrentAngle();

	adjustJoint(joint, (value - curAngle) * dSpeed / 100);
	if (fabs(curAngle - value) > AS->getMaxJointError()) {
		return false;
	} else {
		return true;
	}
}

/*! This method changes the angle of First joint of arm(Shoulder), if
 currently value is near the current angle of knee then no changes
 are made.
 \param side which leg to change angle
 \param value new angle to change to
 \param dSpeed speed to set for action */
bool BasicAgent::rotateArm(SideT side, AngDeg value, double dSpeed) {
	if (dSpeed == -1)
		dSpeed = AS->getJointGain();

	JointT joint = (side == SIDE_LEFT) ? JID_LARM_3 : JID_RARM_3;
	const TSceneJoint* j = WM->getJoint(joint);
	if (!j) {
		Log->log(7, "(BasicAgent::rotateArm) Arm rotation joint does not exist");
		return false;
	}

	AngDeg curAngle = j->getCurrentAngle();

	adjustJoint(joint, (value - curAngle) * dSpeed / 100);
	if (fabs(curAngle - value) > AS->getMaxJointError()) {
		return false;
	} else {
		return true;
	}
}

/*! This method changes the angle of knee, if currently value is near
 the current angle of knee then no changes are made
 \param side which leg to change angle
 \param value new angle to change to
 \param dSpeed speed to set for action */
bool BasicAgent::knee(SideT side, AngDeg value, double dSpeed) {
	if (dSpeed == -1)
		dSpeed = AS->getJointGain();

	JointT joint = (side == SIDE_LEFT) ? JID_LLEG_4 : JID_RLEG_4;
	const TSceneJoint* j = WM->getJoint(joint);
	if (!j) {
		Log->log(7, "(BasicAgent::knee) Knee joint does not exist");
		return false;
	}

	AngDeg curAngle = j->getCurrentAngle();

	cout << "Current j4 value: " << curAngle << ' ' << value << ' ' << dSpeed
			<< endl;
	adjustJoint(joint, (value - curAngle) * dSpeed / 100);
	if (fabs(curAngle - value) > AS->getMaxJointError()) {
		return false;
	} else {
		return true;
	}
}

/*! This method changes the angle of knee, if currently value is near
 the current angle of knee then no changes are made
 \param side which leg to change angle
 \param value new angle to change to
 \param dSpeed speed to set for action */
bool BasicAgent::elbow(SideT side, AngDeg value, double dSpeed) {
	if (dSpeed == -1)
		dSpeed = AS->getJointGain();

	JointT joint = (side == SIDE_LEFT) ? JID_LARM_4 : JID_RARM_4;
	const TSceneJoint* j = WM->getJoint(joint);
	if (!j) {
		Log->log(7, "(BasicAgent::elbow) Elbow joint does not exist");
		return false;
	}

	AngDeg curAngle = j->getCurrentAngle();

	adjustJoint(joint, (value - curAngle) * dSpeed / 100);
	if (fabs(curAngle - value) > AS->getMaxJointError()) {
		return false;
	} else {
		return true;
	}
}

/*--------------------------------------------------------------------------*/
/*--------------------- Changing of universal joints -----------------------*/
/*--------------------------------------------------------------------------*/

/*! This method changes the angle of shoulder, if currently value is near
 the current angle of shoulder then no changes are made
 \param side which leg to change angle
 \param valueFront New angle ( for moving the should towards front and back )
 \param valueOpen New angle ( for Opening and closing the arms )
 \param dSpeed speed to set for action */
bool BasicAgent::shoulder(SideT side, AngDeg valueFront, AngDeg valueOpen,
		double dSpeed) {
	bool bResult;

	if (dSpeed == -1)
		dSpeed = AS->getJointGain();

	AgentModelT model = WM->getAgentModel();
	AngDeg curAngleFront, curAngleOpen;
	AngDeg angFront, angOpen;
	const TSceneJoint * j1, *j2;

	if (model == MODEL_NAO) {
		j1 = WM->getJoint(side == SIDE_LEFT ? JID_LARM_1 : JID_RARM_1);
		j2 = WM->getJoint(side == SIDE_LEFT ? JID_LARM_2 : JID_RARM_2);
		if (!j1 || !j2) {
			Log->log(7,
					"(BasicAgent::shoulder) Nao shoulder joint is not found");
			return false;
		}
		curAngleFront = j1->getCurrentAngle();
		curAngleOpen = j2->getCurrentAngle();
		cout << "Current (r/l)arm1 value: " << curAngleFront << ' ' << valueFront << ' ' << dSpeed
				<< endl;
	} else {
		Log->log(7, "(BasicAgent::shoulder) Agent model is not supported");
		return false;
	}

	// First Front/Back
	angFront = (valueFront - curAngleFront) * dSpeed / 100;
	if (fabs(curAngleFront - valueFront) > AS->getMaxJointError()) {
		bResult = false;
	} else {
		bResult = true;
	}

	// Then Open/Close
	angOpen = (valueOpen - curAngleOpen) * dSpeed / 100;
	if (fabs(curAngleOpen - valueOpen) > AS->getMaxJointError()) {
		bResult &= false;
	} else {
		bResult &= true;
	}

	if (model == MODEL_NAO) {
		adjustJoint(side == SIDE_LEFT ? JID_LARM_1 : JID_RARM_1, angFront);
		adjustJoint(side == SIDE_LEFT ? JID_LARM_2 : JID_RARM_2, angOpen);
	}

	return bResult;
}

/*! This method changes the angle of hip, if currently value is near
 the current angle of hip then no changes are made
 \param side which leg to change angle
 \param valueFront New angle ( for moving the should towards front and back )
 \param valueOpen New angle ( for Opening and closing the arms )
 \param dSpeed speed to set for action */
bool BasicAgent::hip(SideT side, AngDeg valueFront, AngDeg valueOpen,
		double dSpeed) {
	bool bResult;

	if (dSpeed == -1)
		dSpeed = AS->getJointGain();

	AgentModelT model = WM->getAgentModel();
	AngDeg curAngleFront, curAngleOpen;
	AngDeg angFront, angOpen;
	const TSceneJoint * j1, *j2;

	if (model == MODEL_NAO) {
		j1 = WM->getJoint(side == SIDE_LEFT ? JID_LLEG_3 : JID_RLEG_3);
		j2 = WM->getJoint(side == SIDE_LEFT ? JID_LLEG_2 : JID_RLEG_2);
		if (!j1 || !j2) {
			Log->log(7, "(BasicAgent::hip) Nao hip joint is not found");
			return false;
		}
		curAngleFront = j1->getCurrentAngle();
		curAngleOpen = j2->getCurrentAngle();
		cout << "Current angle: , value: , max error: " << curAngleFront << ' '
				<< valueFront << ' ' << AS->getMaxJointError() << endl;
	} else {
		Log->log(7, "(BasicAgent::hip) Agent model is not supported");
		return false;
	}

	// First Front/Back
	angFront = (valueFront - curAngleFront) * dSpeed / 100;
	if (fabs(curAngleFront - valueFront) > AS->getMaxJointError()) {
		bResult = false;
	} else {
		bResult = true;
	}

	// Then Open/Close
	angOpen = (valueOpen - curAngleOpen) * dSpeed / 100;
	if (fabs(curAngleOpen - valueOpen) > AS->getMaxJointError()) {
		bResult &= false;
	} else {
		bResult &= true;
	}

	if (model == MODEL_NAO) {
		adjustJoint(side == SIDE_LEFT ? JID_LLEG_3 : JID_RLEG_3, angFront);
		adjustJoint(side == SIDE_LEFT ? JID_LLEG_2 : JID_RLEG_2, angOpen);
	}
	return bResult;
}

/*! This method changes the angle of shoulder, if currently value is near
 the current angle of hip then no changes are made
 \param side which leg to change angle
 \param valueFront new angle to change to
 \param valueOpen new angle to change to
 \param dSpeed speed to set for action */
bool BasicAgent::ankle(SideT side, AngDeg valueFront, AngDeg valueOpen,
		double dSpeed) {
	bool bResult;

	if (dSpeed == -1)
		dSpeed = AS->getJointGain();

	AgentModelT model = WM->getAgentModel();
	AngDeg curAngleFront, curAngleOpen;
	AngDeg angFront, angOpen;
	const TSceneJoint *j1, *j2;

	if (model == MODEL_NAO) {
		j1 = WM->getJoint(side == SIDE_LEFT ? JID_LLEG_5 : JID_RLEG_5);
		j2 = WM->getJoint(side == SIDE_LEFT ? JID_LLEG_6 : JID_RLEG_6);
		if (!j1 || !j2) {
			Log->log(7, "(BasicAgent::ankle) Nao ankle joint is not found");
			return false;
		}
		curAngleFront = j1->getCurrentAngle();
		curAngleOpen = j2->getCurrentAngle();
	} else {
		Log->log(7, "(BasicAgent::ankle) Agent model is not supported");
		return false;
	}

	// First Front/Back
	angFront = (valueFront - curAngleFront) * dSpeed / 100;
	if (fabs(curAngleFront - valueFront) > AS->getMaxJointError()) {
		bResult = false;
	} else {
		bResult = true;
	}

	// Then Open/Close
	angOpen = (valueOpen - curAngleOpen) * dSpeed / 100;
	if (fabs(curAngleOpen - valueOpen) > AS->getMaxJointError()) {
		bResult &= false;
	} else {
		bResult &= true;
	}

	if (model == MODEL_NAO) {
		adjustJoint(side == SIDE_LEFT ? JID_LLEG_5 : JID_RLEG_5, angFront);
		adjustJoint(side == SIDE_LEFT ? JID_LLEG_6 : JID_RLEG_6, angOpen);
	}

	return bResult;
}
bool BasicAgent::hip1(SideT side, AngDeg value, double dSpeed) {
	if (dSpeed == -1)
		dSpeed = AS->getJointGain();

	JointT joint = (side == SIDE_LEFT) ? JID_LLEG_1 : JID_RLEG_1;
	const TSceneJoint* j = WM->getJoint(joint);
	if (!j) {
		Log->log(7, "(BasicAgent::hip1) hip1 joint does not exist");
		return false;
	}

	AngDeg curAngle = j->getCurrentAngle();

	adjustJoint(joint, dSpeed * (value - curAngle));
	if (fabs(curAngle - value) > AS->getMaxJointError()) {
		return false;
	} else {
		return true;
	}
}

/*--------------------------------------------------------------------------*/
/*--------------------------  Heuristic Procedures  ------------------------*/
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/*--------------------------  Heuristic Procedures  ------------------------*/
/*--------------------------------------------------------------------------*/
// Remained from Gcyrus

/********************************
 *   behind		    .			*
 * 								*
 * 			 pos				*
 * 								*
 *		.			.			*
 *								*
 *						goal	*
 ********************************/

VecPosition BasicAgent::Behind_ofPos_l(VecPosition pos , VecPosition goal)
{

	AngDeg Direction_PosAndGoal = VecPosition::normalizeAngle((goal - pos).getDirection());
	VecPosition Behind_of_Pos = VecPosition ( pos.getX() - 0.2 * cosDeg(Direction_PosAndGoal) ,
			pos.getY() - 0.2 * sinDeg(Direction_PosAndGoal));
	return Behind_of_Pos;
}

/********************************
 *   	.		    .			*
 * 								*
 * 			 pos				*
 * 								*
 *		.	      behind		*
 *								*
 *						goal	*
 ********************************/

VecPosition BasicAgent::Behind_ofPos_r(VecPosition pos , VecPosition Behind_of_Pos , AngDeg slope , double distancce)
{
	VecPosition behind;
	double x,y;
	x = (distancce / sqrt(1 + slope*slope)) + pos.getX();
	y = slope * (x - pos.getX()) + pos.getY();
	cout<<"  XX = "<<x<<"  YY = "<<y
			<<" sqrt(1 + slope*slope) = "<<sqrt(1 + slope*slope)
			<<"  distancce / sqrt(1 + slope*slope) = "<<distancce / sqrt(1 + slope*slope)
			<<endl;
	if(x != Behind_of_Pos.getX() && y != Behind_of_Pos.getY())
		behind = VecPosition(x,y);
	else
	{
		x = (-1 * distancce) / sqrt(1 + slope*slope) + pos.getX();
		y = slope * (x - pos.getX()) + pos.getY();
		behind = VecPosition(x,y);
		cout<<"  XxX = "<<x<<"  YyY = "<<y<<endl;

	}
	return behind;
}

/********************************
 *   	.		    behind		*
 * 								*
 * 			 pos				*
 * 								*
 *		.	      	.			*
 *								*
 *						goal	*
 ********************************/

VecPosition BasicAgent::Behind_ofPos_u(VecPosition pos , AngDeg slope , double distancce)
{
	VecPosition behind;
	double x,y;
	AngDeg slope_  = -1 / slope;
	x = distancce / sqrt(1 + slope_*slope_) + pos.getX();
	y = slope_ * (x - pos.getX()) + pos.getY();
	behind = VecPosition(x,y);
	return behind;
}

/********************************
 *   	.		    .			*
 * 								*
 * 			 pos				*
 * 								*
 *		behind	   	.			*
 *								*
 *						goal	*
 ********************************/

VecPosition BasicAgent::Behind_ofPos_d(VecPosition pos , AngDeg slope , double distancce)
{
	VecPosition behind;
	double x,y;
	AngDeg slope_ = -1 / slope;
	x = (-1 * distancce) / sqrt(1 + slope_*slope_) + pos.getX();
	y = slope_ * (x - pos.getX()) + pos.getY();
	behind = VecPosition(x,y);
	return behind;
}

bool BasicAgent::Behind (VecPosition goal , double error)
{
	VecPosition zero = VecPosition(0,0,0);
//	VecPosition Ball = WM->getBallPos();
	VecPosition Ball = VecPosition(5,2);
//	VecPosition agent = WM->getAgentPosition();
	VecPosition agent = VecPosition(3,1);
	double DistanceToBall =  agent.getDistanceTo2(Ball);
	AngDeg Direction_PosAndGoal = VecPosition::normalizeAngle((goal - Ball).getDirection());
	VecPosition Behind_l = Behind_ofPos_l(Ball , goal);
	double distance = (Ball - Behind_l).getMagnitude2();
	VecPosition Behind_r = Behind_ofPos_r(Ball , Behind_l ,Direction_PosAndGoal , distance);
	VecPosition Behind_u = Behind_ofPos_u(Ball , Direction_PosAndGoal , distance);
	VecPosition Behind_d = Behind_ofPos_d(Ball , Direction_PosAndGoal , distance);
	AngDeg ang = (Behind_ofPos_l(Ball , goal) - Ball).getDirection();
	Matrix m_t = Matrix::createMatrixFromVecPosition(
													VecPosition(1,0,0),
													VecPosition(-1*Ball.getX(),1-Ball.getY(),0),
													VecPosition(0,0,0)
													);
	Matrix m_r = Matrix::createRotationMatrix(-1 * ang);
	Matrix m_agent = Matrix::createMatrixFromVecPosition(agent,zero,zero);
	Matrix m = m_t * m_agent;
	VecPosition agent_New = VecPosition(
										(m_r * m).getElement(1,1),
										(m_r * m).getElement(2,1),
										(m_r * m).getElement(3,1)
										);
	cout<<
			"\nBall = "<<Ball<<"\tagent = "<<agent
			<<"\tDistanceToBall = "<<DistanceToBall
			<<"\tDirection_PosAndGoal = "<<Direction_PosAndGoal
			<<"\tBehind_l = "<<Behind_l
			<<"\tdistance = "<<distance
			<<"\tBehind_r = "<<Behind_r
			<<"\tBehind_u = "<<Behind_u
			<<"\tBehind_d = "<<Behind_d
			<<"\tang = "<<ang
			<<"\tm_t = "<<m_t.str()
			<<"\tm_r = "<<m_r.str()
			<<"\tm = "<<m.str()
			<<"\tm_agent = "<<m_agent.str()
			<<"\tm_t*m_agent = "<<(m_t*m_agent).str()
			<<"\t(m_t * m_agent).getElement(1,1) = "<<(m_t * m_agent).getElement(1,1)
			<<"\t(m_t * m_agent).getElement(2,1) = "<<(m_t * m_agent).getElement(2,1)
			<<"\t(m_t * m_agent).getElement(3,1) = "<<(m_t * m_agent).getElement(3,1)
			<<"\tagent_New = "<<agent_New
			<<endl;



	if(DistanceToBall > error && agent_New.getX() > 0 )
	{
//		counter ++;
		if(agent_New.getX()>0)
		{
			if(agent_New.getY()>0)
			{
				GP = Behind_u;
				this_action = goingToPos_c;
			}
			else
			{
				GP = Behind_d;
				this_action = goingToPos_c;
			}
		}
		else
		{
			GP = Behind_l;
			this_action = goingToPos_c;
		}
		return false;
	}
	else
		return true;
}



/*--------------------------------------------------------------------------*/
/*--------------------------  Turning Procedures  --------------------------*/
/*--------------------------------------------------------------------------*/


//////////////////////////////abc 13
bool BasicAgent::CheckLocalize ()
{
	static bool b1 = true;
	if(!WM->get_r_localize ())
	{
		cout<<"\n CheckLocalize "<<endl;
		Rneck(b1);
		if(WM->get_r_localize())
		{
			b1 = false;
			return true;
		}
		else
		{
			b1=true;
			return false;
		}

	}
	return true;
}
///////////////////////////////
bool BasicAgent::IsFallen()
{
	if(fabs(WM->fall[0])>9.8 || fabs(WM->fall[1])>9.8 ||fabs(WM->fall[2])>9.8 )
		return false;
	else
	{
	if(WM->fall[2]<1||l_action==l_standing_b||l_action==l_standing_f)
	{
		if(WM->fall[1]<-7.5)
		{
			cout<<"\nFALL FRONT "<<endl;
			fallFront= true;
			return true;
		}
		else if(WM->fall[1]>7.5)
		{
			cout<<"\nFALL BACK "<<endl;
			fallBack=true;
			return true;
		}
		else if(WM->fall[0]<-7.5||WM->fall[0]>7.5)
		{
			cout<<"\n FALL BAGHAL "<<endl;
			fallGhalt=true;
			return true;
		}

	}
	}
	return false;
}
////////////////////////////////////////////SKILL////////////////////////////////////////////////
////////////////////////////////////////LOWLEVELACTION////////////////////////////////////////////
bool BasicAgent::Rneck(bool start) {
	bool a;
	static int state=1;
	if (start) {

		//state 1
		if(state ==1 ){
			a=neck( 	0,	 90,5 );
			if (a )
			{
				//				l_action = l_Rneck;
				cout<<"rneck 1"<<endl;
				state ++;
			}
			return false;
		}
		else if(state == 2){
			a=neck(   0,   -90,5 );
			if (a )
			{
				//				l_action = l_actDone;
				cout<<"rneck 2"<<endl;
				state=1;
				rneck = true;
				return true;
			}
			return false;
		}
	}
	return false;
}
//////////////////
bool BasicAgent::Rneck_f(bool start) {
	bool a;
	static int state=1;
	if (start) {
		//state 1
		if(state ==1 ){
			a=neck(    -25, 90   , 10 );
			if (a )
			{
				cout<<"rneck_f 1"<<endl;

				//				l_action = l_RneckF;
				state ++;
			}
			return false;
		}
		else if(state ==2 ){
			a=neck(    -25,  0   , 10);
			if (a )
			{
				cout<<"rneck_f 2"<<endl;
				//				l_action = l_RneckF;
				state ++;
			}
			return false;
		}
		else if(state ==3 ){
			a=neck(    -25, 90   , 10 );
			if (a )
			{
				cout<<"rneck_f 3"<<endl;
				//				l_action = l_RneckF;
				state ++;
			}
			return false;
		}

		if(state ==4){
			a=neck(    0,   0, 10 );
			if (a )
			{
				cout<<"rneck_f 4"<<endl;
				//				l_action=l_RneckF;
				state=1;
				rneck_f = true;
				return true;
			}
			return false;
		}
	}
	return false;
}

///////////////////////////////////
bool BasicAgent::StandBack(bool  start) {
	static int t1 =0;
	static int t2 =0;
	cout<<"state 0 of standing_b "<<state<<endl;
	bool a, b, c, d, e, f, g, h, i, j , k , l, m , n ;
	if (start) {
		//state 1
		if (state == 1) {
			a = shoulder(SIDE_LEFT, -120, 0,40);		b = shoulder(SIDE_RIGHT, -120, 0, 40);
			c = hip		(SIDE_LEFT, 0, 0, 40);		d = hip		(SIDE_RIGHT, 0, 0, 40);
			e = ankle	(SIDE_LEFT, 0, 0, 40);		f = ankle	(SIDE_RIGHT, 0, 0, 40);
			g = knee	(SIDE_LEFT, 0, 40);			h = knee	(SIDE_RIGHT, 0, 40);
			i = elbow	(SIDE_LEFT, -90, 40);			j = elbow	(SIDE_RIGHT, 90, 40);
			k =rotateArm(SIDE_LEFT, 90, 40);			l =rotateArm(SIDE_RIGHT, -90, 40);
			m =rotateLeg(SIDE_LEFT, 0, 40);			n =rotateLeg(SIDE_RIGHT, 0, 40);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				t2++;
				cout<<"t2ttttttttttttttttt="<<t2<<endl;
				l_action = l_standing_b;
				if(t2==20){
					t2=0;
					state ++;
					cout<<"state 1:true of standing_b "<<state<<endl;
				}
			}
			cout<<"state 1:false of standing_b "<<state<<endl;
		}
		//state 2
		else if (state == 2) {
			a = shoulder(SIDE_LEFT, -40, 0, 10);		b = shoulder(SIDE_RIGHT, -40, 0, 10);
			c = hip		(SIDE_LEFT, 100, 0, 10);		d = hip		(SIDE_RIGHT,100, 0, 10);
			e = ankle	(SIDE_LEFT, -45, 0, 10);		f = ankle	(SIDE_RIGHT,-45, 0, 10);
			g = knee	(SIDE_LEFT, -80, 10);			h = knee	(SIDE_RIGHT,-80, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT,0, 10);
			m =rotateLeg(SIDE_LEFT, -70, 10);			n =rotateLeg(SIDE_RIGHT, -70, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				l_action = l_standing_b;
				state ++;
				cout<<"state 2:true of standing_b "<<state<<endl;
			}
			cout<<"state 2:false of standing_b "<<state<<endl;
		}
		//state 3
		else if (state == 3) {
			a = shoulder(SIDE_LEFT,-60, 0, 10);		b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip		(SIDE_LEFT, 70, 10, 10);		d = hip		(SIDE_RIGHT,70, -10, 10);
			e = ankle	(SIDE_LEFT, 45, -15, 10);		f = ankle	(SIDE_RIGHT,45, 15, 10);
			g = knee	(SIDE_LEFT, -115, 10);			h = knee	(SIDE_RIGHT,-115, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT,0, 10);
			m =rotateLeg(SIDE_LEFT, -35, 10);			n =rotateLeg(SIDE_RIGHT, -35, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				l_action = l_standing_b;
				state ++;
				cout<<"state 3:true of standing_b "<<state<<endl;
			}
			cout<<"state 3:false of standing_b "<<state<<endl;

		}
		//state 4
		else if (state == 4) {
			a = shoulder(SIDE_LEFT,-60,0, 10);		b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip		(SIDE_LEFT, 70, 10, 10);		d = hip		(SIDE_RIGHT,70, -10, 10);
			e = ankle	(SIDE_LEFT, 35, -15, 10);		f = ankle	(SIDE_RIGHT,35, 15, 10);
			g = knee	(SIDE_LEFT,-80, 10);			h = knee	(SIDE_RIGHT,-80, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT,0, 10);			l =rotateArm(SIDE_RIGHT,0, 10);
			m =rotateLeg(SIDE_LEFT, -15, 10);			n =rotateLeg(SIDE_RIGHT, -15, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				l_action = l_standing_b;
				state ++;
				cout<<"state 4:true of standing_b "<<state<<endl;
			}
			cout<<"state 4:false of standing_b "<<state<<endl;

		}
		//state 5
		else if (state == 5) {
			a = shoulder(SIDE_LEFT, -60, 0, 10);		b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip		(SIDE_LEFT, 70, 0, 10);		d = hip		(SIDE_RIGHT, 70, 0, 10);
			e = ankle	(SIDE_LEFT, 35, 0, 10);		f = ankle	(SIDE_RIGHT, 35, 0, 10);
			g = knee	(SIDE_LEFT, -80, 10);			h = knee	(SIDE_RIGHT, -80, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT, 0, 10);
			m =rotateLeg(SIDE_LEFT, -15, 10);			n =rotateLeg(SIDE_RIGHT, -15, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				t1++;
				l_action = l_standing_b;
				if(t1==2){
					state ++;
					cout<<"state 5:true of standing_b "<<state<<endl;
				}
			}
			cout<<"state 5:false of standing_b "<<state<<endl;

		}
		//state 6
		else if (state == 6) {
			a = shoulder(SIDE_LEFT, -60, 0, 10);
			b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip(SIDE_LEFT, 35, 2, 10);
			d = hip(SIDE_RIGHT, 35, -2, 10);
			e = ankle(SIDE_LEFT, 55, -2, 10);
			f = ankle(SIDE_RIGHT, 55, 2, 10);
			g = knee(SIDE_LEFT, -90, 10);
			h = knee(SIDE_RIGHT, -90, 10);
			i = elbow(SIDE_LEFT, 0, 10);
			j = elbow(SIDE_RIGHT, 0, 10);
			if (a && b && c && d && e && f && g && h && i && j )
			{
				l_action=l_actDone;
				state =1;
				sb=true;
				fallBack=false;
				cout<<"state 10:true of standing_b "<<state<<endl;
				t1 = 0;
				//				t2 = 0;
				return true;
			}

		}
		cout<<"state 10:false of standing_b "<<state<<endl;

	}
	return false;


}
////////////////////////////////
bool BasicAgent::StandFront(bool  start) {
	static int t1 =0;
	static int t2 =0;
	static int t3 =0;

	bool a, b, c, d, e, f, g, h, i, j , k , l, m , n ;
	if (start) {

		//state 1
		if (state == 1) {
			a = shoulder(SIDE_LEFT, 0, 0, 10);		b = shoulder(SIDE_RIGHT, 0, 0, 10);
			c = hip		(SIDE_LEFT, 0, 0, 10);		d = hip		(SIDE_RIGHT, 0, 0, 10);
			e = ankle	(SIDE_LEFT, 0, 0, 10);		f = ankle	(SIDE_RIGHT, 0, 0, 10);
			g = knee	(SIDE_LEFT, 0, 10);			h = knee	(SIDE_RIGHT, 0, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT, 0, 10);
			m =rotateLeg(SIDE_LEFT, 0, 10);			n =rotateLeg(SIDE_RIGHT, 0, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{


				l_action = l_standing_f;

				state ++;
				cout<<"state 1:true of standing_f "<<state<<endl;
			}

			cout<<"state 1:false of standing_f "<<state<<endl;

		}
		//state 2
		else if (state == 2) {
			a = shoulder(SIDE_LEFT,-20, 0, 40);		b = shoulder(SIDE_RIGHT,-20, 0,40);
			c = hip		(SIDE_LEFT,90,10,40);		d = hip		(SIDE_RIGHT,90,-10, 40);
			e = ankle	(SIDE_LEFT,15,-30,40);		f = ankle	(SIDE_RIGHT,15,30, 40);
			g = knee	(SIDE_LEFT,-125,40);		h = knee	(SIDE_RIGHT,-125, 40);
			i = elbow	(SIDE_LEFT, 0, 40);			j = elbow	(SIDE_RIGHT, 0, 40);
			k =rotateArm(SIDE_LEFT, 0,40);			l =rotateArm(SIDE_RIGHT, 0, 40);
			m =rotateLeg(SIDE_LEFT,-80, 40);		n =rotateLeg(SIDE_RIGHT,-80, 40);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				t3++;
				l_action = l_standing_f;
				if(t3==10)
				{
					t3=0;
					state ++;
					cout<<"state 2:true of standing_f "<<state<<endl;
				}
			}
			cout<<"state 2:false of standing_f "<<state<<endl;
		}
		//state 3
		else if (state == 3) {
			a = shoulder(SIDE_LEFT,-25, 0, 10);		b = shoulder(SIDE_RIGHT,-25, 0, 10);
			c = hip		(SIDE_LEFT,70,10, 10);		d = hip		(SIDE_RIGHT,70,-10, 10);
			e = ankle	(SIDE_LEFT,45,-15, 10);		f = ankle	(SIDE_RIGHT,45,15, 10);
			g = knee	(SIDE_LEFT,-115, 10);		h = knee	(SIDE_RIGHT,-115, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT, 0, 10);
			m =rotateLeg(SIDE_LEFT,-35, 10);		n =rotateLeg(SIDE_RIGHT,-35, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				//		t1++;
				l_action = l_standing_f;
				//		if(t1==10){
				state ++;
				t1=0;
				cout<<"state 3:true of standing_f "<<state<<endl;
				//		}
			}
			cout<<"state 3:false of standing_f "<<state<<endl;
		}
		//state 8
		else if (state == 4) {
			a = shoulder(SIDE_LEFT, 0, 0, 10);		b = shoulder(SIDE_RIGHT, 0, 0, 10);
			c = hip		(SIDE_LEFT,70,10, 10);		d = hip		(SIDE_RIGHT,70,-10, 10);
			e = ankle	(SIDE_LEFT,35,-15, 10);		f = ankle	(SIDE_RIGHT,35,15, 10);
			g = knee	(SIDE_LEFT,-80, 10);		h = knee	(SIDE_RIGHT,-80, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT, 0, 10);
			m =rotateLeg(SIDE_LEFT,-15, 10);		n =rotateLeg(SIDE_RIGHT,-15, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				l_action = l_standing_f;
				state ++;
				cout<<"state 4:true of standing_f "<<state<<endl;
			}
			cout<<"state 4:false of standing_f "<<state<<endl;
		}
		//state 9
		else if (state == 5) {
			a = shoulder(SIDE_LEFT, 0, 0, 10);		b = shoulder(SIDE_RIGHT, 0, 0, 10);
			c = hip		(SIDE_LEFT,70, 0, 10);		d = hip		(SIDE_RIGHT,70, 0, 10);
			e = ankle	(SIDE_LEFT,35, 0, 10);		f = ankle	(SIDE_RIGHT,35, 0, 10);
			g = knee	(SIDE_LEFT,-80, 10);		h = knee	(SIDE_RIGHT,-80, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT, 0, 10);
			m =rotateLeg(SIDE_LEFT,-15, 10);		n =rotateLeg(SIDE_RIGHT,-15, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				//		t2++;
				l_action = l_standing_f;
				//	if(t2==20){
				t2=0;
				state ++;
				cout<<"state 5:true of standing_f "<<state<<endl;
				//	}
			}
			cout<<"state 5:false of standing_f "<<state<<endl;
		}
		//state 10
		else if (state == 6) {
			a = shoulder(SIDE_LEFT, -60, 0, 10);
			b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip(SIDE_LEFT, 35, 2, 10);
			d = hip(SIDE_RIGHT, 35, -2, 10);
			e = ankle(SIDE_LEFT, 55, -2, 10);
			f = ankle(SIDE_RIGHT, 55, 2, 10);
			g = knee(SIDE_LEFT, -90, 10);
			h = knee(SIDE_RIGHT, -90, 10);
			i = elbow(SIDE_LEFT, 0, 10);
			j = elbow(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT, 0, 10);
			m =rotateLeg(SIDE_LEFT, 0, 10);			n =rotateLeg(SIDE_RIGHT, 0, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				cout<< "this_action="<<this_action<<endl;
				cout<< "last_actionInStand="<<last_action<<endl;
				l_action=l_actDone;
				sb=true;
				state =1;
				fallFront=false;
				cout<<"state 6:true of standing_f "<<state<<endl;
				//				t1 = 0;
				//				t2 = 0;
				//				t3=0;
				return true;
			}
			cout<<"state 6:false of standing_f "<<state<<endl;
		}

	}
	return false;
}
/////////////////////////////////////

bool BasicAgent::Ghalt(bool  start) {
	bool a, b, c, d, e, f, g, h, i, j , k , l, m , n ;
	if (start) {
		//state 1
		if (state == 1) {
			a = shoulder(SIDE_LEFT,0,0, 10);		b = shoulder(SIDE_RIGHT, 0, 0, 10);
			c = hip		(SIDE_LEFT, 0, 0, 10);		d = hip		(SIDE_RIGHT,0, 0, 10);
			e = ankle	(SIDE_LEFT, 0, 0, 10);		f = ankle	(SIDE_RIGHT,0, 0, 10);
			g = knee	(SIDE_LEFT,0, 10);			h = knee	(SIDE_RIGHT,0, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT,0, 10);			l =rotateArm(SIDE_RIGHT,0, 10);
			m =rotateLeg(SIDE_LEFT, -90, 10);			n =rotateLeg(SIDE_RIGHT, -90, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				cout<< "this_action="<<this_action<<endl;
				cout<< "last_actionInStand="<<last_action<<endl;
				l_action=l_actDone;
				sb=true;
				state =1;
				fallGhalt=false;
				cout<<"state 1:true of ghalting"<<state<<endl;
				return true;
			}
			cout<<"state 1:false of ghalting"<<state<<endl;
		}

	}
	return false;
}

//////////////////////////////////////////
bool BasicAgent::walk(bool start) {
	bool a, b, c, d, e, f, g, h, i, j,  l, m;
	cout<<"state 0 of walking "<<state<<endl;
	if (start) {
		//state 1
		if (state == 1) {
			a = shoulder(SIDE_LEFT, -60, 0, 30);
			b = shoulder(SIDE_RIGHT, -60, 0, 30);
			c = hip(SIDE_LEFT, 45, 2, 30);
			d = hip(SIDE_RIGHT, 65, -2, 30);
			e = ankle(SIDE_LEFT, 50, -2, 30);
			f = ankle(SIDE_RIGHT, 30, 2, 30);
			g = knee(SIDE_LEFT, -85, 30);
			h = knee(SIDE_RIGHT, -85, 30);
			i = elbow(SIDE_LEFT, -50, 30);
			j = elbow(SIDE_RIGHT,50, 30);
			l = rotateArm(SIDE_LEFT, -80, 30);
			m = rotateArm(SIDE_RIGHT, 80, 30);
			if (a && b && c && d && e && f && g && h && i && j && l && m)
			{
				//				this_action = walking;
				l_action=l_walking;
				cout<<"state 1:true of walking "<<state<<endl;
				state++;
			}
			cout<<"state 1:false of walking "<<state<<endl;
			return false;
		}
		//state 2

		else if (state == 2) {
			a = shoulder(SIDE_LEFT, -60, 0, 30);
			b = shoulder(SIDE_RIGHT, -60, 0, 30);
			c = hip(SIDE_LEFT, 65, 2, 30);
			d = hip(SIDE_RIGHT, 55, -2, 30);
			e = ankle(SIDE_LEFT, 45, -2, 30);
			f = ankle(SIDE_RIGHT, 40, 0, 30);
			g = knee(SIDE_LEFT, -100, 30);
			h = knee(SIDE_RIGHT, -85, 30);
			i = elbow(SIDE_LEFT, -50, 30);
			j = elbow(SIDE_RIGHT, 50, 30);
			l = rotateArm(SIDE_LEFT, -80, 30);
			m = rotateArm(SIDE_RIGHT, 80, 30);
			if (a && b && c && d && e && f && g && h && i && j && l && m)
			{
				//				this_action = walking;
				l_action=l_walking;
				cout<<"state 2:true of walking "<<state<<endl;
				state++;
			}
			cout<<"state 2:false of walking "<<state<<endl;
		}

		//state 2
		else if (state == 3) {
			a = shoulder(SIDE_LEFT, -60, 0, 30);
			b = shoulder(SIDE_RIGHT, -60, 0, 30);
			c = hip(SIDE_LEFT, 65, 2, 30);
			d = hip(SIDE_RIGHT, 45, -2, 30);
			e = ankle(SIDE_LEFT, 30, -2, 30);
			f = ankle(SIDE_RIGHT, 50, 2, 30);
			g = knee(SIDE_LEFT, -85, 30);
			h = knee(SIDE_RIGHT, -85, 30);
			i = elbow(SIDE_LEFT, -50, 30);
			j = elbow(SIDE_RIGHT, 50, 30);
			l = rotateArm(SIDE_LEFT, -80, 30);
			m = rotateArm(SIDE_RIGHT, 80, 30);
			if (a && b && c && d && e && f && g && h && i && j && l && m)
			{
				//				this_action = walking;
				l_action=l_walking;
				cout<<"state 3:true of walking "<<state<<endl;
				state++;
			}
			cout<<"state 3:false of walking "<<state<<endl;
		}
		//state 2
		else if (state == 4) {
			a = shoulder(SIDE_LEFT, -60, 0, 30);
			b = shoulder(SIDE_RIGHT, -60, 0, 30);
			c = hip(SIDE_LEFT, 55, 2, 30);
			d = hip(SIDE_RIGHT, 65, -2, 30);
			e = ankle(SIDE_LEFT,40, 0, 30);
			f = ankle(SIDE_RIGHT, 45, 2, 30);
			g = knee(SIDE_LEFT, -85, 30);
			h = knee(SIDE_RIGHT, -100, 30);
			i = elbow(SIDE_LEFT, -50, 30);
			j = elbow(SIDE_RIGHT, 50, 30);
			l = rotateArm(SIDE_LEFT, -80, 30);
			m = rotateArm(SIDE_RIGHT, 80, 30);
			if (a && b && c && d && e && f && g && h && i && j && l && m)
			{
				//				this_action = actIsDone;
				l_action=l_actDone;
				cout<<"state 4:true of walking "<<state<<endl;
				state = 1;
				return true;
			}
			cout<<"state 4:false of walking "<<state<<endl;
		}
	}
	return false;
}
bool BasicAgent::littlewalk(bool start) {
	cout<<"state 0 of littlewalking "<<state<<endl;
	bool a, b, c, d, e, f, g, h, i, j, l, m, n, o;
	if (start) {
		//state 1
		if (state == 1) {
			a = shoulder(SIDE_LEFT, -60, 0, 25);
			b = shoulder(SIDE_RIGHT, -60, 0, 25);
			c = hip(SIDE_LEFT, 60, 0, 25);
			d = hip(SIDE_RIGHT, 60, 0,25);
			e = ankle(SIDE_LEFT, 50, 0, 25);
			f = ankle(SIDE_RIGHT, 35, 0, 25);
			g = knee(SIDE_LEFT, -95, 25);
			h = knee(SIDE_RIGHT, -80, 25);
			i = elbow(SIDE_LEFT, -50, 25);
			j = elbow(SIDE_RIGHT, 50, 25);
			l = rotateArm(SIDE_LEFT, -80, 25);
			m = rotateArm(SIDE_RIGHT, 80, 25);
			n = rotateLeg(SIDE_LEFT, 0, 25);
			o = rotateLeg(SIDE_RIGHT, 0, 25);
			if (a && b && c && d && e && f && g && h && i && j && l && m && n
					&& o) {
				//				this_action = littlewalking;
				cout<<"state 1:true of littlewalking "<<state<<endl;
				l_action=l_littelwalking;
				state++;
			}
			cout<<"state 1:false of littlewalking "<<state<<endl;
		}
		//state 2
		else if (state == 2) {
			a = shoulder(SIDE_LEFT, -60, 0, 25);
			b = shoulder(SIDE_RIGHT, -60, 0, 25);
			c = hip(SIDE_LEFT, 60, 0, 25);
			d = hip(SIDE_RIGHT, 60, 0,25);
			e = ankle(SIDE_LEFT, 35, 0, 25);
			f = ankle(SIDE_RIGHT, 50, 0, 25);
			g = knee(SIDE_LEFT, -80, 25);
			h = knee(SIDE_RIGHT, -95, 25);
			i = elbow(SIDE_LEFT, -50, 25);
			j = elbow(SIDE_RIGHT, 50, 25);
			l = rotateArm(SIDE_LEFT, -80, 25);
			m = rotateArm(SIDE_RIGHT, 80, 25);
			n = rotateLeg(SIDE_LEFT, 0, 25);
			o = rotateLeg(SIDE_RIGHT, 0, 25);
			if (a && b && c && d && e && f && g && h && i && j && l && m && n
					&& o)

			{
				cout<<"state 2:true of littlewalking "<<state<<endl;
				//				this_action = actIsDone;
				l_action=l_actDone;
				state = 1;
				return true;
			}
			cout<<"state 2:false of littlewalking "<<state<<endl;
		}
	}
	return false;
}
//////////////////////////////////
bool BasicAgent::walkback(bool  start)
{
	cout<<"state 0 of backwalking "<<state<<endl;
	bool a, b, c, d, e, f, g, h, i, j ,l,m;
	if (start) {
		//state 1
		if(state ==1 ){
			a = shoulder(SIDE_LEFT, -60, 0, 25);
			b = shoulder(SIDE_RIGHT, -60, 0, 25);
			c = hip(SIDE_LEFT, 45, 2, 25);
			d = hip(SIDE_RIGHT, 55, -2, 25);
			e = ankle(SIDE_LEFT, 40, 0, 25);
			f = ankle(SIDE_RIGHT, 45, 2, 25);
			g = knee(SIDE_LEFT, -85, 25);
			h = knee(SIDE_RIGHT, -100, 25);
			i = elbow(SIDE_LEFT, -40, 25);
			j = elbow(SIDE_RIGHT, 40, 25);
			l = rotateArm(SIDE_LEFT, -80, 25);
			m = rotateArm(SIDE_RIGHT, 80, 25);
			if (a && b && c && d && e && f && g && h && i && j&& l && m )
			{
				//				this_action=walkingback;
				cout<<"state 1:true of backwalking "<<state<<endl;
				l_action=l_walkingback;
				state ++;
			}
			cout<<"state 1:false of backwalking "<<state<<endl;
		}
		//state 2
		else if(state == 2){
			a = shoulder(SIDE_LEFT, -60, 0, 25);
			b = shoulder(SIDE_RIGHT, -60, 0, 25);
			c = hip(SIDE_LEFT, 55, 2, 25);
			d = hip(SIDE_RIGHT, 35, -2, 25);
			e = ankle(SIDE_LEFT, 30, -2, 25);
			f = ankle(SIDE_RIGHT, 50, 2, 25);
			g = knee(SIDE_LEFT, -85, 25);
			h = knee(SIDE_RIGHT, -85, 25);
			i = elbow(SIDE_LEFT, -40, 25);
			j = elbow(SIDE_RIGHT, 40, 25);
			l = rotateArm(SIDE_LEFT, -80, 25);
			m = rotateArm(SIDE_RIGHT, 80, 25);
			if (a && b && c && d && e && f && g && h && i && j&& l && m )
			{
				//				this_action=walkingback;
				cout<<"state 2:true of backwalking "<<state<<endl;
				l_action=l_walkingback;
				state ++;
			}
			cout<<"state 2:false of backwalking "<<state<<endl;
		}
		//state 3
		else if(state == 3){
			a = shoulder(SIDE_LEFT, -60, 0, 25);
			b = shoulder(SIDE_RIGHT, -60, 0, 25);
			c = hip(SIDE_LEFT, 55, 2, 25);
			d = hip(SIDE_RIGHT, 45, -2, 25);
			e = ankle(SIDE_LEFT, 45, -2, 25);
			f = ankle(SIDE_RIGHT, 40, 0, 25);
			g = knee(SIDE_LEFT, -100, 25);
			h = knee(SIDE_RIGHT, -85, 25);
			i = elbow(SIDE_LEFT, -40, 25);
			j = elbow(SIDE_RIGHT, 40, 25);
			l = rotateArm(SIDE_LEFT, -80, 25);
			m = rotateArm(SIDE_RIGHT, 80, 25);
			if (a && b && c && d && e && f && g && h && i && j&& l && m )
			{
				//				this_action=walkingback;
				cout<<"state 3:true of backwalking "<<state<<endl;
				l_action=l_walkingback;
				state ++;
			}
			cout<<"state 3:false of backwalking "<<state<<endl;
		}
		//state 4
		else if(state == 4){
			a = shoulder(SIDE_LEFT, -60, 0, 25);
			b = shoulder(SIDE_RIGHT, -60, 0, 25);
			c = hip(SIDE_LEFT, 35, 2, 25);
			d = hip(SIDE_RIGHT, 55, -2, 25);
			e = ankle(SIDE_LEFT, 50, -2, 25);
			f = ankle(SIDE_RIGHT, 30, 2, 25);
			g = knee(SIDE_LEFT, -85, 25);
			h = knee(SIDE_RIGHT, -85, 25);
			i = elbow(SIDE_LEFT, -40, 25);
			j = elbow(SIDE_RIGHT, 40, 25);
			l = rotateArm(SIDE_LEFT, -80, 25);
			m = rotateArm(SIDE_RIGHT, 80, 25);
			if (a && b && c && d && e && f && g && h && i && j&& l && m )
			{
				//				this_action=actIsDone;
				cout<<"state 4:true of backwalking "<<state<<endl;
				l_action=l_actDone;
				state =1;
				return true;
			}
			cout<<"state 4:false of backwalking "<<state<<endl;
		}

	}
	return false;
}
/////////////////////////////////////////
bool BasicAgent::reset() {
	bool a, b, c, d, e, f, g, h, i, j;
	a = shoulder(SIDE_LEFT, -60, 0, 10);
	b = shoulder(SIDE_RIGHT, -60, 0, 10);
	c = hip(SIDE_LEFT, 35, 2, 10);
	d = hip(SIDE_RIGHT, 35, -2, 10);
	e = ankle(SIDE_LEFT, 55, -2, 10);
	f = ankle(SIDE_RIGHT, 55, 2, 10);

//	e = ankle(SIDE_LEFT, -55, -40, 10);
//		f = ankle(SIDE_RIGHT, -55, -40, 10);

	g = knee(SIDE_LEFT, -90, 10);
	h = knee(SIDE_RIGHT, -90, 10);
	i = elbow(SIDE_LEFT, 0, 10);
	j = elbow(SIDE_RIGHT, 0, 10);
	if (a && b && c && d && e && f && g && h && i && j){
		cout<<"resetisfinish."<<state <<endl;
		l_action = l_actDone;
		return true;
	}
	return false;
}
//////////////////////////////////////////
bool BasicAgent::turn_l(bool start) {
	bool a, b, c, d, e, f, g, h, i, j, l, m, n, o;
	l_action=l_turning_l;
	cout<<"state 0 of turning_l "<<state<<endl;
	if (start) {
		//state 1
		if (state == 1) {
			a = shoulder(SIDE_LEFT, -90, 0, 10);
			b = shoulder(SIDE_RIGHT, -90, 0, 10);
			c = hip(SIDE_LEFT, 50, 2, 10);
			d = hip(SIDE_RIGHT, 50, -2, 10);
			e = ankle(SIDE_LEFT, 45, -2, 10);
			f = ankle(SIDE_RIGHT, 45, 2, 10);
			g = knee(SIDE_LEFT, -85, 10);
			h = knee(SIDE_RIGHT, -85, 10);
			i = elbow(SIDE_LEFT, -40, 25);
			j = elbow(SIDE_RIGHT, 40, 25);
			l = rotateArm(SIDE_LEFT, -80, 25);
			m = rotateArm(SIDE_RIGHT, 80, 25);
			n = rotateLeg(SIDE_LEFT, 0, 10);
			o = rotateLeg(SIDE_RIGHT, 0, 10);
			if (a && b && c && d && e && f && g && h && i && j && l && m && n
					&& o)
			{
				//this_action = turning_l;
				cout<<"state 1:true of turning_l "<<state<<endl;
				l_action=l_turning_l;
				state++;
			}
			cout<<"state 1:false of turning_l "<<state<<endl;
		}
		//state 2
		else if (state == 2) {
			a = shoulder(SIDE_LEFT, -90, 0, 10);
			b = shoulder(SIDE_RIGHT, -90, 0, 10);
			c = hip(SIDE_LEFT, 55, 2, 10);
			d = hip(SIDE_RIGHT, 45, -2, 10);
			e = ankle(SIDE_LEFT, 45, -2, 10);
			f = ankle(SIDE_RIGHT, 40, 2, 10);
			g = knee(SIDE_LEFT, -100, 10);
			h = knee(SIDE_RIGHT, -85, 10);
			i = elbow(SIDE_LEFT, -40, 25);
			j = elbow(SIDE_RIGHT, 40, 25);
			l = rotateArm(SIDE_LEFT, -80, 25);
			m = rotateArm(SIDE_RIGHT, 80, 25);
			n = rotateLeg(SIDE_LEFT, -15, 10);
			o = rotateLeg(SIDE_RIGHT, -15, 10);
			if (a && b && c && d && e && f && g && h && i && j && l && m && n
					&& o)
			{
				//this_action = actIsDone;
				cout<<"state 2:true of turning_l "<<state<<endl;
				l_action=l_actDone;
				state = 1;
				return true;
			}
			cout<<"state 2:false of turning_l "<<state<<endl;
		}
	}
	return false;
}
///////////////////////////////////
bool BasicAgent::turn_r(bool start) {
	bool a, b, c, d, e, f, g, h, i, j, l, m, n, o;
	l_action=l_turning_r;
	cout<<"state 0 of turning_r "<<state<<endl;
	if (start) {
		//state 1
		if (state == 1) {
			a = shoulder(SIDE_LEFT, -90, 0, 10);
			b = shoulder(SIDE_RIGHT, -90, 0, 10);
			c = hip(SIDE_LEFT,50, 2, 10);
			d = hip(SIDE_RIGHT, 50, -2, 10);
			e = ankle(SIDE_LEFT, 45, -2, 10);
			f = ankle(SIDE_RIGHT, 45, 2, 10);
			g = knee(SIDE_LEFT, -85, 10);
			h = knee(SIDE_RIGHT, -85, 10);
			i = elbow(SIDE_LEFT, -40, 25);
			j = elbow(SIDE_RIGHT, 40, 25);
			l = rotateArm(SIDE_LEFT, -80, 25);
			m = rotateArm(SIDE_RIGHT, 80, 25);
			n = rotateLeg(SIDE_LEFT, 0, 10);
			o = rotateLeg(SIDE_RIGHT, 0, 10);
			if (a && b && c && d && e && f && g && h && i && j && l && m && n
					&& o)
			{
				//this_action = turning_r;
				cout<<"state 1:true of turning_r "<<state<<endl;
				l_action=l_turning_r;
				state++;
			}
			cout<<"state 1:false of turning_r "<<state<<endl;
		}
		//state 2
		else if (state == 2) {
			a = shoulder(SIDE_LEFT, -90, 0, 10);
			b = shoulder(SIDE_RIGHT, -90, 0, 10);
			c = hip(SIDE_LEFT, 45, 2, 10);
			d = hip(SIDE_RIGHT, 55, -2, 10);
			e = ankle(SIDE_LEFT, 40, -2, 10);
			f = ankle(SIDE_RIGHT, 45, 2, 10);
			g = knee(SIDE_LEFT, -85, 10);
			h = knee(SIDE_RIGHT, -100, 10);
			i = elbow(SIDE_LEFT, -40, 25);
			j = elbow(SIDE_RIGHT, 40, 25);
			l = rotateArm(SIDE_LEFT, -80, 25);
			m = rotateArm(SIDE_RIGHT, 80, 25);
			n = rotateLeg(SIDE_LEFT, -15, 10);
			o = rotateLeg(SIDE_RIGHT, -15, 10);
			if (a && b && c && d && e && f && g && h && i && j && l && m && n
					&& o) {
				//this_action = actIsDone;
				cout<<"state 2:true of turning_r "<<state<<endl;
				l_action=l_actDone;
				state = 1;
				return true;
			}
			cout<<"state 2:false of turning_r "<<state<<endl;
		}
	}
	return false;
}
///////////////////////////////
bool BasicAgent::move_r(bool start) {
	l_action = l_moving_r;
	bool a, b, c, d, e, f, g, h, i, j,  l, m;
	cout<<"state 0 of moving_r "<<state<<endl;
	if (start) {
		//state 1
		if (state == 1) {
			a = shoulder(SIDE_LEFT, -60, 0, 10);
			b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip(SIDE_LEFT, 55, 0, 10);
			d = hip(SIDE_RIGHT, 55, 0, 10);
			e = ankle(SIDE_LEFT, 45, 0, 10);
			f = ankle(SIDE_RIGHT, 45, 0, 10);
			g = knee(SIDE_LEFT, -90, 10);
			h = knee(SIDE_RIGHT, -90, 10);
			i = elbow(SIDE_LEFT, -40, 10);
			j = elbow(SIDE_RIGHT, 40, 10);
			l = rotateArm(SIDE_LEFT, -80, 10);
			m = rotateArm(SIDE_RIGHT, 80, 10);
			if (a && b && c && d && e && f && g && h && i && j && l && m)
			{
				//this_action=moving_r;
				cout<<"state 1 : true of moving_r "<<state<<endl;
				l_action=l_moving_r;
				state++;
			}
			cout<<"state 1 : false of moving_r "<<state<<endl;
		}
		//state 2
		else if (state == 2) {
			a = shoulder(SIDE_LEFT, -60, 0, 10);
			b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip(SIDE_LEFT, 55,30, 10);
			d = hip(SIDE_RIGHT, 55, 0, 10);
			e = ankle(SIDE_LEFT, 45, 0, 10);
			f = ankle(SIDE_RIGHT, 45, 0, 10);
			g = knee(SIDE_LEFT, -90, 10);
			h = knee(SIDE_RIGHT, -90, 10);
			i = elbow(SIDE_LEFT, -40, 10);
			j = elbow(SIDE_RIGHT, 40, 10);
			l = rotateArm(SIDE_LEFT, -80, 10);
			m = rotateArm(SIDE_RIGHT, 80, 10);
			if (a && b && c && d && e && f && g && h && i && j && l && m)
			{
				//this_action=actIsDone;
				l_action=l_actDone;
				state=1;
				cout<<"state 1 : true of moving_r "<<state<<endl;
				return true;
			}
			cout<<"state 2 : false of moving_r "<<state<<endl;
		}
	}
	return false;
}
/////////////////////////////////////
bool BasicAgent::move_l(bool start) {
	bool a, b, c, d, e, f, g, h, i, j,  l, m;
	l_action = l_moving_l;
	cout<<"state 0 of moving_l "<<state<<endl;
	if (start) {
		//state 1
		if (state == 1) {
			a = shoulder(SIDE_LEFT, -60, 0, 10);
			b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip(SIDE_LEFT, 55, 0, 10);
			d = hip(SIDE_RIGHT, 55, 0, 10);
			e = ankle(SIDE_LEFT, 45, 0, 10);
			f = ankle(SIDE_RIGHT, 45, 0, 10);
			g = knee(SIDE_LEFT, -90, 10);
			h = knee(SIDE_RIGHT, -90, 10);
			i = elbow(SIDE_LEFT, -40, 10);
			j = elbow(SIDE_RIGHT, 40, 10);
			l = rotateArm(SIDE_LEFT, -80, 10);
			m = rotateArm(SIDE_RIGHT, 80, 10);
			if (a && b && c && d && e && f && g && h && i && j && l && m)
			{
				//this_action=moving_l;
				l_action=l_moving_l;
				state++;
				cout<<"state 1 : true of moving_l "<<state<<endl;
			}
			cout<<"state 1 : false of moving_l "<<state<<endl;
		}
		//state 2
		else if (state == 2) {
			a = shoulder(SIDE_LEFT, -60, 0, 10);
			b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip(SIDE_LEFT, 55,  0, 10);
			d = hip(SIDE_RIGHT, 55, -30, 10);
			e = ankle(SIDE_LEFT, 45, 0, 10);
			f = ankle(SIDE_RIGHT, 45, 0, 10);
			g = knee(SIDE_LEFT, -90, 10);
			h = knee(SIDE_RIGHT, -90, 10);
			i = elbow(SIDE_LEFT, -40, 10);
			j = elbow(SIDE_RIGHT, 40, 10);
			l = rotateArm(SIDE_LEFT, -80, 10);
			m = rotateArm(SIDE_RIGHT, 80, 10);
			if (a && b && c && d && e && f && g && h && i && j && l && m)
			{
				//this_action=actIsDone;
				l_action=l_actDone;
				state=1;
				cout<<"state 2 : true of moving_l "<<state<<endl;
				return true;
			}
			cout<<"state 2 : false of moving_l "<<state<<endl;
		}
	}
	return false;
}
///////////////////////////////////

bool BasicAgent::kick_r(bool  start) {

	static int t1,t2,t3;
	bool a, b, c, d, e, f, g, h, i, j ,l,m;
	if (start) {
		//state 1
		if(state ==1 ){
			a=shoulder( SIDE_LEFT,   -75,    0, 10 );	  b=shoulder( SIDE_RIGHT,    -75,    0,10 );
			c=hip     ( SIDE_LEFT,    25, -15 , 10 );	  d=hip     ( SIDE_RIGHT,     25,  -15, 10 );
			e=ankle   ( SIDE_LEFT,     30,  15 ,10 );     f=ankle   ( SIDE_RIGHT,    35,   15, 10);
			g=knee    ( SIDE_LEFT,    -50,       10 );     h=knee    ( SIDE_RIGHT,    -55,       10 );
			i=elbow   ( SIDE_LEFT,     -30,      10);	  j=elbow   ( SIDE_RIGHT,     30,      10 );
			l=rotateArm   ( SIDE_LEFT,     -90,       10);	 m=rotateArm  ( SIDE_RIGHT,     90,       10 );
			if (a && b && c && d && e && f && g && h && i && j&& l && m )
			{
				//		t1++;
				//	this_action = kicking_r;
				l_action=l_kicking_r;
				//		if(t1==20)
				state ++;
			}
		}
		//state 2

		else if(state == 2){
			a=shoulder( SIDE_LEFT,   -75,    0, 10 );	  b=shoulder( SIDE_RIGHT,    -75,    0,10 );
			c=hip     ( SIDE_LEFT,    25, -15 , 10 );	  d=hip     ( SIDE_RIGHT,     15,  -15, 10 );////15
			e=ankle   ( SIDE_LEFT,     30,  15 ,10 );     f=ankle   ( SIDE_RIGHT,    55,   15, 10);
			g=knee    ( SIDE_LEFT,    -50,       10 );     h=knee    ( SIDE_RIGHT,    -95,       10 );
			i=elbow   ( SIDE_LEFT,     -30,      10);	  j=elbow   ( SIDE_RIGHT,     30,      10 );
			l=rotateArm   ( SIDE_LEFT,     -90,       10);	 m=rotateArm  ( SIDE_RIGHT,     90,       10 );
			if (a && b && c && d && e && f && g && h && i && j&& l && m )
			{
				//	t1++;

				//this_action = kicking_r;
				l_action=l_kicking_r;
				//if(t1==100)
				state ++;
			}
		}
		//state 3
		else if(state == 3){
			a=shoulder( SIDE_LEFT,   -75,    0, 35 );	  b=shoulder( SIDE_RIGHT,    -75,    0,35 );
			c=hip     ( SIDE_LEFT,    25, -15 , 35 );	  d=hip     ( SIDE_RIGHT,    65,  -15, 35 );//65
			e=ankle   ( SIDE_LEFT,     30,  15 ,35 );     f=ankle   ( SIDE_RIGHT,   -30,   15, 35);//-35
			g=knee    ( SIDE_LEFT,    -50,       35 );     h=knee    ( SIDE_RIGHT,    -40,       35 );//-40
			i=elbow   ( SIDE_LEFT,     -30,      35);	  j=elbow   ( SIDE_RIGHT,     30,      35 );//30
			l=rotateArm   ( SIDE_LEFT,     -90,       35);	 m=rotateArm  ( SIDE_RIGHT,     90,      35 );
			if (a && b && c && d && e && f && g && h && i && j&& l && m )

			{
				//t2++;
				//this_action = kicking_r;
				l_action=l_kicking_r;
				//if(t2==100)
				state ++;
			}
		}
		//state 2
		else if(state == 4){
			a=shoulder( SIDE_LEFT,   -75,    0, 10 );	  b=shoulder( SIDE_RIGHT,    -75,    0,10 );
			c=hip     ( SIDE_LEFT,    25, -15 , 10 );	  d=hip     ( SIDE_RIGHT,     50,  -15, 10 );
			e=ankle   ( SIDE_LEFT,     30,  15 ,10 );     f=ankle   ( SIDE_RIGHT,    35,   15, 10);
			g=knee    ( SIDE_LEFT,    -50,       10 );     h=knee    ( SIDE_RIGHT,    -80,       10 );
			i=elbow   ( SIDE_LEFT,     -30,      10);	  j=elbow   ( SIDE_RIGHT,     30,      10 );
			l=rotateArm   ( SIDE_LEFT,     -90,       10);	 m=rotateArm  ( SIDE_RIGHT,     90,       10 );
			if (a && b && c && d && e && f && g && h && i && j&& l && m )
			{
				t1=0;
				t2=0;
				//this_action = actIsDone;
				l_action=l_actDone;
				state=1;
				return true;
			}
		}

	}
	return false;
}
/////////////////////////////////
bool BasicAgent::kick_l(bool start) {

	static int t1,t2;
	bool a, b, c, d, e, f, g, h, i, j ,l,m;
	if (start) {
		//state 1
		if(state ==1 ){
			a=shoulder( SIDE_RIGHT,   -75,    0, 10 );	  b=shoulder( SIDE_LEFT,    -75,    0,10 );
			c=hip     ( SIDE_RIGHT,    25, 15 , 10 );	  d=hip     ( SIDE_LEFT,     25,  15, 10 );
			e=ankle   ( SIDE_RIGHT,     30,  -15 ,10 );     f=ankle   ( SIDE_LEFT,    35,   -15, 10);
			g=knee    ( SIDE_RIGHT,    -50,       10 );     h=knee    ( SIDE_LEFT,    -55,       10 );
			i=elbow   ( SIDE_RIGHT,     30,      10);	  j=elbow   ( SIDE_LEFT,     -30,      10 );
			l=rotateArm   ( SIDE_RIGHT,     90,       10);	 m=rotateArm  ( SIDE_LEFT,     -90,       10 );
			if (a && b && c && d && e && f && g && h && i && j&& l && m )
			{
				t1++;
				//	this_action = kicking_r;
				l_action=l_kicking_l;
				//	if(t1==20)
				state ++;
			}
		}
		//state 2

		else if(state == 2){
			a=shoulder( SIDE_RIGHT,   -75,    0, 10 );	  b=shoulder( SIDE_LEFT,    -75,    0,10 );
			c=hip     ( SIDE_RIGHT,    25, 15 , 10 );	  d=hip     ( SIDE_LEFT,     15,  15, 10 );////15
			e=ankle   ( SIDE_RIGHT,     30,  -15 ,10 );     f=ankle   ( SIDE_LEFT,    55,   -15, 10);
			g=knee    ( SIDE_RIGHT,    -50,       10 );     h=knee    ( SIDE_LEFT,    -95,       10 );
			i=elbow   ( SIDE_RIGHT,     30,      10);	  j=elbow   ( SIDE_LEFT,     -30,      10 );
			l=rotateArm   ( SIDE_RIGHT,     90,       10);	 m=rotateArm  ( SIDE_LEFT,     -90,       10 );
			if (a && b && c && d && e && f && g && h && i && j&& l && m )
			{
				//	t1++;

				//this_action = kicking_r;
				l_action=l_kicking_l;
				//if(t1==100)
				state ++;
			}
		}
		//state 3
		else if(state == 3){
			a=shoulder( SIDE_RIGHT,   -75,    0, 35 );	  b=shoulder( SIDE_LEFT,    -75,    0,35 );
			c=hip     ( SIDE_RIGHT,    25, 15 , 35 );	  d=hip     ( SIDE_LEFT,    65,  15, 35 );//65
			e=ankle   ( SIDE_RIGHT,     30,  -15 ,35 );     f=ankle   ( SIDE_LEFT,    -30,   -15, 35);//-35
			g=knee    ( SIDE_RIGHT,    -50,       35 );     h=knee    ( SIDE_LEFT,    -40,       35 );//-40
			i=elbow   ( SIDE_RIGHT,     30,      35);	  j=elbow   (SIDE_LEFT,     -30,      35 );//30
			l=rotateArm   ( SIDE_RIGHT,     90,       35);	 m=rotateArm  ( SIDE_LEFT,     -90,      35 );
			if (a && b && c && d && e && f && g && h && i && j&& l && m )

			{
				//t2++;
				//this_action = kicking_r;
				l_action=l_kicking_l;
				//if(t2==100)
				state ++;
			}
		}
		//state 2
		else if(state == 4){
			a=shoulder( SIDE_RIGHT,   -75,    0, 10 );	  b=shoulder(SIDE_LEFT,    -75,    0,10 );
			c=hip     ( SIDE_RIGHT,    25, 15 , 10 );	  d=hip     ( SIDE_LEFT,     50,  15, 10 );
			e=ankle   ( SIDE_RIGHT,     30,  -15 ,10 );     f=ankle   ( SIDE_LEFT  ,  35,   -15, 10);
			g=knee    ( SIDE_RIGHT,    -50,       10 );     h=knee    (SIDE_LEFT,    -80,       10 );
			i=elbow   ( SIDE_RIGHT,     30,      10);	  j=elbow   ( SIDE_LEFT,     -30,      10 );
			l=rotateArm   ( SIDE_RIGHT,     90,       10);	 m=rotateArm  ( SIDE_LEFT,     -90,       10 );
			if (a && b && c && d && e && f && g && h && i && j&& l && m )
			{
				t1=0;
				t2=0;
				//this_action = actIsDone;
				l_action=l_actDone;
				state=1;
				return true;
			}
		}

	}
	return false;
}
////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////MIDLEVELACTION///////////////////////////////////////

bool BasicAgent::StandUp()
{
	this_action=standingUp;
	if(fallBack && l_action != l_standing_f &&  l_action != l_ghalt )
		l_action = l_standing_b;
	else if(fallFront && l_action != l_standing_b &&  l_action != l_ghalt )
		l_action = l_standing_f;
	else if(fallGhalt && l_action != l_standing_f &&  l_action != l_standing_b )
		l_action = l_ghalt;
	if(Do_LAct())
	{
		this_action=actIsDone;
		return true;
	}
	return false;
}
/**
 * This method is used to turn the agent to thee specified position
 * @param posTurn Position for the agent to turn to
 * @return bool When the agent has finished turning
 */
bool BasicAgent::turnToPoint(VecPosition posTurn,double e,bool start) {
	if(start){
		AngDeg angToTurn = VecPosition::normalizeAngle((posTurn - WM->getAgentPosition()).getDirection());
		return turnToAngle(angToTurn,e);
	}
	return false;
}

bool BasicAgent::turnToAngle(AngDeg angTurn,double e) {
	AngDeg Angle=VecPosition::normalizeAngle(angTurn - WM->get_Theta());
	cout<<"\nAnglem="<<Angle<<"\nangTurn : "<< angTurn <<"\n WM->get_Theta() : "<< WM->get_Theta()<<endl;
	bool b=turnToRelativeAngle(Angle,e);
	cout<<"bturn"<<b<<endl;
	return b;
}
/////////////////////////////////////////////
bool BasicAgent::turnToRelativeAngle(AngDeg angTurn,double e) {
	cout<<"ghesmate 0 = "<<angTurn<<endl;
	if(fabs(angTurn) <= e)
	{
		cout<<"ghesmate 1 = "<<angTurn<<endl;
		return true;
	}
	else if(angTurn<-1*e)
	{
		cout<<"ghesmate 2:1 = "<<angTurn<<endl;
		if(turn_r(true))
		{
			cout<<"turn is ok."<<endl;
			cout<<"ghesmate 2 = "<<angTurn<<endl;
		}
		else
			return false;
	}
	else if(angTurn>e)
	{
		cout<<"ghesmate 3:1 = "<<angTurn<<endl;
		if(turn_l(true))
		{
			cout<<"Turn is ok."<<endl;
			cout<<"ghesmate 3 = "<<angTurn<<endl;
		}
		else
			return false ;
	}
	return false;
}
//////////////////////////////////////////////////
bool BasicAgent::shoot()
{
	this_action=shooting;
	AngDeg Angle=WM->getRelativeAngle(OBJECT_BALL)+WM->getJointInstance(JID_HEAD_1)->getCurrentAngle()+2;
	VecPosition Relativepos=WM->Ball_R;
	cout<<"R_pos1"<<Relativepos<<endl;
	Relativepos=VecPosition::getVecPositionFromPolar(Relativepos[0],Relativepos[1],Relativepos[2]+WM->getJointInstance(JID_HEAD_2)->getCurrentAngle());
	//	if(Hstate!=sidewalking && ((Angle<60 && Angle>44)||(Angle>-60 && Angle <-44)) && Do_LAct())
	//		Hstate=sidewalking;
	//	else if (Hstate!=kicking && (fabs(Angle)<=44) && Do_LAct())
	//		Hstate=kicking;
	cout<<"hstate"<<Hstate;
	cout<<"R_Theta="<<Angle<<"R_pos2"<<Relativepos<<endl;
	if((GOAL-WM->getBallPos()).getDirection()-WM->get_Theta()>30&& Do_LAct())
	{
		if(Hstate!=turning_toAngle)
			reset();
		Hstate=turning_toAngle;
	}
	if(fabs(Angle)>33 && Do_LAct())
	{
		if(Hstate!=sidewalking)
			reset();
		Hstate=sidewalking;
	}
	else if(Relativepos.getX()>0.25 && Do_LAct())
	{
		if(Hstate!=littlewalking)
			reset();
		Hstate=littlewalking;
	}
	else if(Relativepos.getX()<0.08 && Do_LAct())
	{
		if(Hstate!=walkingback)
			reset();
		Hstate=walkingback;
	}
	else if ( fabs(Angle)<=33 && Do_LAct())
	{
		if(Hstate!=kicking)
			reset();
		Hstate=kicking;
	}

	cout<<"shootang="<<Angle<<endl;
	switch(Hstate)
	{
	case turning_toAngle:
	{
		turnToAngle((GOAL-WM->getBallPos()).getDirection(),15);

		break;
	}
	case walkingback:
	{
		l_action=l_walkingback;
		break;
	}
	case littlewalking:
	{
		l_action=l_littelwalking;
		break;
	}

	case sidewalking:
	{
		if( Angle<-22)
		{
			cout << "shoot: 3 kick check" << endl;
			l_action=l_moving_r;
		}
		else if( Angle>22)
		{
			cout << "shoot: 4 kick check" << endl;
			l_action=l_moving_l;
		}
		break;
	}
	case kicking:
	{
		if(Angle<0)
		{
			cout << "shoot: 1 kick check" << endl;
			l_action = l_kicking_r;

		}
		else
		{
			cout << "shoot: 2 kick check" << endl;
			l_action = l_kicking_l;
		}
		if(Do_LAct())
		{
			GOAL=UnknownPosition;
			OBJECT=OBJECT_PLAYER_UNKNOWN;
			this_action=actIsDone;
			return true;
		}
		break;
	}
	}
	return false;
}
bool BasicAgent::Check_kickingto(VecPosition goal)
{
	AngDeg Direction_PosAndGoal = VecPosition::normalizeAngle((goal - WM->getBallPos()).getDirection());
	AngDeg Angle_Theta=WM->getRelativeAngle(OBJECT_BALL)+WM->getJointInstance(JID_HEAD_1)->getCurrentAngle();
	AngDeg Angle_Phi=WM->getRelativePosition(OBJECT_BALL).getPhi()+WM->getJointInstance(JID_HEAD_2)->getCurrentAngle();
	if(Hstate!=turning_toAngle && fabs(Direction_PosAndGoal-WM->get_Theta())>30 && Do_LAct())
	{
		cout<<"\nchecking0:1  "<<Hstate<<endl;
		Hstate=turning_toAngle;
	}
	else if (Hstate!= walking && fabs(Angle_Phi)>50 && Do_LAct())
	{
		cout<<"\nchecking0:2"<<Hstate<<endl;
		Hstate=walking;
	}
	else if (Hstate!=sidewalking  && fabs(Angle_Theta)>40 && Do_LAct())
	{
		cout<<"\nchecking0:3 "<<Hstate<<endl;
		Hstate=sidewalking;
	}
	if(Do_LAct())
	{
		switch (Hstate) {
		case turning_toAngle:
		{
			cout<<"\nchecking1 \t"<<Direction_PosAndGoal-WM->get_Theta()<<"\ttheta="<<WM->get_Theta()<<endl;
			if(turnToAngle(Direction_PosAndGoal,5))
			{
				cout<<"ttturn is finish."<<endl;
				Hstate=walking;
			}
			break;
		}
		case walking:
		{
			cout<<"\nchecking2   "<<Angle_Phi<<endl;
			if(fabs(Angle_Phi)<30)
				Hstate=sidewalking;
			else if(Angle_Phi>0)
			{
				cout<<"\nchecking2.1 "<<endl;
				l_action=l_littelwalking;
			}
			//					else if (fabs(Angle_Phi))
			//					{
			//						cout<<"\nchecking2.2 "<<endl;
			//						l_action=l_walkingback;
			//					}
			break;
		}
		case sidewalking:
		{
			cout<<"\nchecking3   "<< Angle_Theta<<endl;
			if( fabs(Angle_Theta)<40)              //fabs(Angle)<170 && fabs(Angle)>140)
				Hstate=actIsDone;
			else if(Angle_Theta<0)
			{
				cout<<"\nchecking3.1   "<<Angle_Theta<<endl;
				l_action=l_moving_r;
			}
			else if(Angle_Theta>0)
			{
				cout<<"\nchecking3.2   "<<Angle_Theta<<endl;
				l_action=l_moving_l;
			}

			break;
		}
		case actIsDone:
		{
			cout<<"\nchecking5 "<<endl;
			return true;
			break;
		}
		default:
		{
			cout<<"\nchecking6 "<<endl;
			Hstate=turning_toAngle;
			break;
		}
		}		//END OF SWITCH
	}//END OF IF
	return false;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
bool BasicAgent::gotopos(VecPosition p,double e)
{

	p=GP ;
	static int gotopos_state =1;
	static bool iswalking=true;
	static bool isturning=false;
	VecPosition Agent = WM->getAgentPosition();
	double distance= Agent.getDistanceTo2(p);
	double dis=distance*100;
	dis=ceil(dis)*0.01;
	AngDeg Angle =VecPosition::normalizeAngle((p-Agent).getDirection()-WM->get_Theta());
	Angle=ceil(Angle);

	cout<<"\nAgent"<<Agent<<endl<<"\nang=="<<Angle<<endl<<"\nDISTAnc"<<distance<<endl<<"dis=="<<dis<<endl;
	cout<<"isturning"<<isturning<<"\t iswalking"<<iswalking<<endl;
	if((dis<0.3&& iswalking) ||  (!iswalking && dis < 2))
	{
		iswalking=false;
		isturning=false;
		if(dis>e)
		{
			if(littlewalk(true))
			{
				cout<<"move3"<<endl;
				this_action = goingToPos;
				gotopos_state =1;
				iswalking=true;
				isturning=false;
				return true;
			}

		}
		else
		{
			cout<<"eis = "<<e<<endl;
			this_action = actIsDone;
			gotopos_state =1;
			iswalking=true;
			isturning=false;
			return true ;
		}
	}
	else if (!iswalking && !isturning)
		iswalking=true;

	switch(gotopos_state)
	{
	case 0:
	{
		bool h=turnToPoint(p,10,isturning);
		if(h)
		{	cout<<"GB : case 0 "<<endl;
		gotopos_state=1;
		}
		break;
	}
	case 1:
	{
		cout<<"GB : case 1 -1 "<<endl;
		if(fabs(Angle)>27.55 )
		{
			cout<<"GB : case 1 -2 "<<endl;
			if(iswalking)
			{
				if(l_action=l_reseting)
				{
					cout<<"GB : case 1 -4 "<<endl;
					iswalking=false;
					isturning=true;
				}
			}
			gotopos_state=0;
			break;
		}

		else
		{
			cout<<"GB : case 1 -5 "<<endl;
			cout<<"reset 2 "<<endl;
			if(isturning)
				if(l_action=l_reseting)
				{
					cout<<"GB : case 1 -7 "<<endl;
					isturning=false;
					iswalking=true;
				}
			if (!walk(iswalking))
			{
				cout<<"GB : case 1 -6 "<<endl;
				this_action = goingToPos;
				return false;
			}

			break;
		}
	}
	}
	this_action = goingToPos;
	return false;
}
bool BasicAgent::gotoposRelative(double e)
{

	static int 	gotopos_state=1;
	static bool iswalking=true;
	static bool isturning=false;
	VecPosition p=WM->getRelativePosition(OBJECT_BALL);
	VecPosition p2=WM->getBallPos();
	cout<<"p_RP=="<<p<<"  p2 = "<<p2<<endl;;
	cout<<"\nJOINTHEAd=="<<WM->getJointInstance(JID_HEAD_1)->getCurrentAngle();
	AngDeg Angle=p.getTheta()+WM->getJointInstance(JID_HEAD_1)->getCurrentAngle();
	//	AngDeg Angle = VecPosition::normalizeAngle((p2-WM->getAgentPosition()).getTheta());
	double distanc=p.getMagnitude2()*100;
	distanc=ceil(distanc)*0.01;
	double distanc2=(p2-WM->getAgentPosition()).getMagnitude2()*100;
	distanc2=ceil(distanc2)*0.01;
	cout<<"p.theta"<<p.getTheta()<<"\tp2.gettheta == "<<p2.getTheta()
				<<"\tpt.theta jadid == "<<(VecPosition::normalizeAngle((p2-WM->getAgentPosition()).getTheta())
				)-WM->getJointInstance(JID_HEAD_1)->getCurrentAngle()
				<<"\tAnggg=="<<Angle<<"\tdis=="<<distanc
				<<"\tdis2=="<<distanc2<<endl;
	cout<<"isturn"<<isturning<<"\tiswalk"<<iswalking<<endl;
	//	if((distanc<0.3 && iswalking) ||  (!iswalking && distanc < 2))
	//	{

	iswalking=false;
	isturning=false;

	if(distanc>e)
	{
		cout<<"OK"<<endl;
		if(littlewalk(true))
		{
			cout<<"move3"<<endl;
			this_action = goingToPos_R;
			gotopos_state=1;
			iswalking=true;
			isturning=false;
			return true;
		}
	}
	else
	{
		cout<<"eis = "<<e<<endl;
		this_action = actIsDone;
		gotopos_state=1;
		iswalking=true;
		isturning=false;
		return true ;
	}
	//	}
	//	else


	if (!iswalking && !isturning)
		iswalking=true;


	switch(gotopos_state){
	case 0:
	{
		bool h= turnToRelativeAngle(Angle,10);
		if(h)
		{
			cout<<"GB : case 0 "<<endl;
			gotopos_state=1;
		}
		break;
	}
	case 1:
	{
		cout<<"GB : case 1 -1 "<<endl;
		if(fabs(Angle)>25.55 &&( l_action==l_actDone ||  l_action==l_nothing))
		{
			cout<<"GB : case 1 -2 "<<endl;
			if(iswalking)
			{
				if(reset()&& reset())
				{
					cout<<"GB : case 1 -4 "<<endl;
					iswalking=false;
					isturning=true;
				}
			}
			gotopos_state=0;
			break;
		}

		else
		{
			cout<<"GB : case 1 -5 "<<endl;
			cout<<"reset 2 "<<endl;
			if(isturning)
				if(reset()&& reset())
				{
					cout<<"GB : case 1 -7 "<<endl;
					isturning=false;
					iswalking=true;
				}
			if (!walk(iswalking))
			{
				cout<<"GB : case 1 -6 "<<endl;
				this_action = goingToPos_R;
				return false;
			}
			break;
		}
	}
	}
	this_action = goingToPos_R;
	return false;
}

///////////////////////////////////////////////////////////////////////
//bool BasicAgent::gotopos_c(VecPosition p,double e)
//{
////	//	this_action=goingToPos_c;
////	VecPosition Agent=WM->getAgentPosition();
////	AngDeg ang1=VecPosition::normalizeAngle(WM->get_Theta()-(Agent-p).getDirection());
////	AngDeg ang2=VecPosition::normalizeAngle((p-Agent).getDirection()-WM->get_Theta());
////	cout<<"goingposHstate="<<WM->get_Theta()<<" ang1: "<<ang1<<" ang2: "<<ang2;
////	double distance= Agent.getDistanceTo2(p);
////	distance=ceil(distance*100)*0.01;
////	//	if(Hstate==turning_toAngle && Do_LAct())
////	//	{
////	//		cout<<"\ngoingpos0:0  "<<Hstate<<endl;
////	//		turnToRelativeAngle(Angle_Turn,15);
////	//	}
////	if(distance<e)
////	{
////		cout<<"\ngoingpos0:5  "<<Hstate<<endl;
////		Hstate=actIsDone;
////		return true;
////	}
////	//	else if(Hstate!=walkingback && ang1<40 && Do_LAct())
////	//	{
////	//		cout<<"\ngoingpos0:1  "<<Hstate<<endl;
////	//		Hstate=walkingback;
////	//	}
////	//	else if(Hstate!=walking && Do_LAct())
////	//	{
////	//		cout<<"\ngoingpos0:3 "<<Hstate<<endl;
////	//		Hstate=walking;
////	//	}
////	if(Do_LAct())
////	{
////		switch (Hstate) {
////		case walkingback:
////		{
////
////			cout<<"\ngoingpos1   "<<ang1<<endl;
////			if(fabs(ang1>20))
////			{
////
////				Angle_Turn=ang1;
////				Hstate=turning_toAngle;
////				cout<<"\ngoingpos1:1 "<<endl;
////			}
////
////			else
////			{
////				l_action=l_walkingback;
////				cout<<"\ngoingpos1:2 "<<endl;
////			}
////			break;
////		}
////		case turning_toAngle:
////		{
////			cout<<"\ngoingpos2 "<<endl;
////			if(turnToAngle(Angle_Turn,15))
////			{
////				cout<<"ttturn is finish."<<endl;
////				if(VecPosition::normalizeAngle(fabs(WM->get_Theta()-ang1))<15)
////				{
////					if(reset())
////					{
////						Hstate=walkingback;
////						cout<<"\ngoingpos2:1 "<<endl;
////					}
////				}
////				else
////				{
////					if(reset())
////					{
////						Hstate=walking;
////						cout<<"\ngoingpos2:2 "<<endl;
////					}
////				}
////			}
////			break;
////		}
////		case walking:
////		{
////			cout<<"\ngoingpos3 "<<ang2<<endl;
////			if(distance < e	)
////			{
////				if(reset())
////					Hstate=actIsDone;
////			}
////			else if(VecPosition::normalizeAngle(fabs(ang2-WM->get_Theta() > 20)))
////			{
////				Angle_Turn=ang2;
////				if(reset())
////				{
////					Hstate=turning_toAngle;
////					cout<<"\ngoingpos3:1 "<<endl;
////				}
////			}
////			else
////			{
////				l_action=l_walking;
////				cout<<"\ngoingpos3:2 "<<endl;
////			}
////			break;
////		}
////
////		case actIsDone:
////		{
////			cout<<"\ngoingtopos5 "<<endl;
////			return true;
////			break;
////		}
////		default:
////		{
////			cout<<"\ngoingpos6 "<<endl;
////			Hstate=turning_toAngle;
////			break;
////		}
////		}		//END OF SWITCH
////	}			//END OF IF
////	return false;
//
//		this_action=goingToPos_c;
//		VecPosition Agent=WM->getAgentPosition();
//		AngDeg ang1=VecPosition::normalizeAngle(WM->get_Theta()-(Agent-p).getDirection());
//		AngDeg ang2=VecPosition::normalizeAngle((p-Agent).getDirection()-WM->get_Theta());
//		cout<<"goingposHstate="<<WM->get_Theta()<<" ang1: "<<ang1<<" ang2: "<<ang2<<"AngTurn="<<Angle_Turn<<endl;
//		double distance= Agent.getDistanceTo2(p);
//		distance=ceil(distance*100)*0.01;
//	//	if(Hstate==turning_toAngle && Do_LAct())
//	//	{
//	//		cout<<"\ngoingpos0:0  "<<Hstate<<endl;
//	//		turnToRelativeAngle(Angle_Turn,15);
//	//	}
//		 if(distance<e)
//		{
//			cout<<"\ngoingpos0:5  "<<Hstate<<endl;
//			Hstate=actIsDone;
//			return true;
//		}
//	//	else if(Hstate!=walkingback && ang1<40 && Do_LAct())
//	//	{
//	//		cout<<"\ngoingpos0:1  "<<Hstate<<endl;
//	//		Hstate=walkingback;
//	//	}
//	//	else if(Hstate!=walking && Do_LAct())
//	//	{
//	//		cout<<"\ngoingpos0:3 "<<Hstate<<endl;
//		 //		Hstate=walking;
//		 //	}
//		 if(Do_LAct())
//		 {
//			 cout<<"goingpos===="<<endl;
//			 switch (Hstate) {
//			 case turning_toAngle:
//			 {
//				 if(ang1>50)
//					 Angle_Turn=ang1;
//				 else
//					 Angle_Turn=ang2;
//				 cout<<"\ngoingpos2 "<<endl;
//				 if(turnToRelativeAngle(Angle_Turn,15))
//				 {
//					 cout<<"ttturn is finish."<<endl;
//					 if(fabs(ang1)<40)
//					 {
//						 if(reset())
//						 {
//							 Hstate=walkingback;
//							 cout<<"\ngoingpos2:1 "<<endl;
//						 }
//					 }
//					 else
//					 {
//						 if(reset())
//						 {
//							 Hstate=walking;
//							 cout<<"\ngoingpos2:2 "<<endl;
//						 }
//					 }
//				 }
//				 break;
//			 }
//			 case walkingback:
//			 {
//				 cout<<"\ngoingpos1   "<<ang1<<endl;
//				 if(fabs(ang1>30))
//				 {
//					 Angle_Turn=ang1;
//					 Hstate=turning_toAngle;
//					 cout<<"\ngoingpos1:1 "<<endl;
//				 }
//				 else
//				 {
//					 l_action=l_walkingback;
//					 cout<<"\ngoingpos1:2 "<<endl;
//				 }
//				 break;
//			 }
//			 case walking:
//			 {
//				 cout<<"\ngoingpos3 "<<ang2<<endl;
//				 if(fabs(ang2) > 30)
//				 {
//					 Angle_Turn=ang2;
//					 Hstate=turning_toAngle;
//					 cout<<"\ngoingpos3:1 "<<endl;
//				 }
//				 else
//				 {
//					 l_action=l_walking;
//					 cout<<"\ngoingpos3:2 "<<endl;
//				 }
//				 break;
//			 }
//
//			 case actIsDone:
//			 {
//				 cout<<"\ngoingtopos5 "<<endl;
//				 return true;
//				 break;
//			 }
//			 default:
//			 {
//				 cout<<"\ngoingpos6 "<<endl;
//				 Hstate=turning_toAngle;
//				 break;
//			 }
//			 }		//END OF SWITCH
//		 }			//END OF IF
//		 return false;
//
//}
bool BasicAgent:: gotopos_c(VecPosition p, double e)
{
	this_action=goingToPos_c;
	VecPosition Agent = WM->getAgentPosition();
	double distance= Agent.getDistanceTo2(p);
	distance=ceil(distance*100)*0.01;
	AngDeg Angle =VecPosition::normalizeAngle((p-Agent).getDirection()-WM->get_Theta());
	Angle=ceil(Angle);
	cout<<"GP="<<GP<<"\tAgent=="<<Agent<<"\tdis="<<distance<<"\tAngle="<<Angle<<"\tHstate="<<Hstate<<endl;
	if(distance<e)
		Hstate=actIsDone;
	else if( fabs(Angle)>30 && Do_LAct())
	{
		if(Hstate!=turning_toAngle)
			reset();
		Hstate=turning_toAngle;
	}
	else if (Hstate!=walking && distance>e && Do_LAct())
		Hstate=walking;

	if(Do_LAct())
	{
		switch (Hstate) {
		case turning_toAngle:
		{
			Angle_Turn=Angle;
			cout<<"\ngoingpos2 "<<endl;
			if(turnToRelativeAngle(Angle_Turn,15))
			{
				cout<<"ttturn is finish."<<endl;

				if(reset()&& reset()&& reset()&& reset())
				{
					Hstate=walking;
					cout<<"\ngoingpos2:1 "<<endl;
				}
			}


			break;
		}

		case walking:
		{
			cout<<"\ngoingpos3 "<<endl;

			l_action=l_walking;

			break;
		}

		case actIsDone:
		{
			cout<<"\ngoingtopos5 "<<endl;
			this_action=actIsDone;
			return true;
			break;
		}
		default:
		{
			cout<<"\ngoingpos6 "<<endl;
			Hstate=turning_toAngle;
			break;
		}
		}		//END OF SWITCH
	}
	return false;
}
//////////////////////////////////////////////////////////////////////
bool BasicAgent::goBehindeTheBall_forKick( double distance , double error_move , AngDeg error_angle)
{
	static int a=0;	//in first
	VecPosition pos = WM->getBallPos();
	cout<<"Ball"<<pos<<endl;
	VecPosition goal = GOAL;
	AngDeg Direction_PosAndGoal = ( goal - pos ).getDirection();
	Line line = Line::makeLineFromTwoPoints(pos , goal);
	//if the agent was yonder land - , + change
	VecPosition Behind_of_Pos = VecPosition ( pos.getX() - distance * cosDeg(Direction_PosAndGoal) ,
			pos.getY() - distance * sinDeg(Direction_PosAndGoal));
	cout<<"Behind_of_Pos"<<Behind_of_Pos<<endl;

	cout<<"GB: in first "<<endl;
	this_action = goingBehind_Kick;

	if(this_action == goingBehind_Kick )
	{
		cout<<"\ngobehind1 "<<endl;

		if(a == 0)
		{
			cout<<"\ngobehind2 "<<endl;
			if(!gotopos(Behind_of_Pos,error_move))
			{
				cout<<"\ngobehind3 "<<endl;
				this_action = goingBehind_Kick;
				return false;
			}
			else
			{
				cout<<"\ngobehind5 "<<endl;
				this_action = goingBehind_Kick;
				a ++;
				return false;
			}
		}
		else if(a == 1)
		{
			if(!reset())
			{
				cout<<"\ngobehind6 "<<endl;
				this_action = goingBehind_Kick;
				return false;
			}
			else
			{
				cout<<"\ngobehind7 "<<endl;
				this_action = goingBehind_Kick;
				if(Rneck_f(true)){
					reset();
					a ++;}

				return false;
			}
		}
		else if(a == 2)
		{
			cout<<"\ngobehind8 "<<endl;
			if(!turnToAngle(Direction_PosAndGoal , error_angle))
			{
				cout<<"\ngobehind9 "<<endl;
				this_action = goingBehind_Kick;
				return false;
			}
			else
			{
				cout<<"\ngobehind10 "<<endl;
				this_action = actIsDone;
				a = 0;
				return true;
			}
		}
	}
	return false;
}

bool BasicAgent::goBehindeTheBall_forKick_Relative(double distance )
{
	VecPosition goal=GOAL;
	VecPosition Ball;
	VecPosition p=WM->getRelativePosition(OBJECT_BALL);
	cout<<"p_RP=="<<p;
	double distanc=p.getMagnitude2()*100;
	distanc=ceil(distanc)*0.01;

	cout<<"\ngobehind2 "<<endl;


	if(!gotoposRelative(distance))
	{
		cout<<"\ngobehind3 "<<endl;
		this_action = goingBehind_Kick_R;
		return false;
	}
	else
	{

		if(!goBehindeTheBall_forKick(0.1,0.1,5))
		{
			this_action=goingBehind_Kick_R;
			return false;
		}
		else
		{
			cout<<"\ngobehind7 "<<endl;
			this_action=actIsDone;
			return true;
		}
	}

	return false;
}
//*************************************************************************	/////
bool BasicAgent::SearchBall()
{
	static bool b=true;
	cout<<"searchBall0"<<endl;
	//	if(WM->getTime()-WM->getTimeLastSeen(OBJECT_BALL)>0.1){
	if(!WM->seeBall()){
		cout<<"searchBall1"<<endl;
		Rneck(b);
	}
	else
	{
		b=false;
		return true;
	}
	return false;
}

//*************************************************************************	/////

void BasicAgent::Say_Ball()
{
	cout<<"ssssssay"<<endl;
	char str[20];
	memset(str,'\0',20);
	int  j=0;
	int max_x=0;
	int max_y=0;
	double x,y;
	//	if(!CheckLocalize())
	//		CheckLocalize();
	//	else
	//	{
	VecPosition Ball=WM->getGlobalPosition(OBJECT_BALL);
	x=Ball.getX()*100;
	x=ceil(x)*0.01;
	if(x<0)
		max_x=6;
	else
		max_x=5;
	y=Ball.getY()*100;
	y=ceil(y)*0.01;
	if(y<0)
		max_y=6;
	else
		max_y=5;
	str[j++]='b';
	//			str[j++]='(';
	j+=snprintf(str+j, max_x+1, "%f", x);
	j=j-4;
	//					cout<<"sttrjx"<<j<<endl;
	str[j++] =',';
	j+=snprintf(str+j, max_y, "%f", y);
	j=j-4;
	str[j]='e';
	cout<<"ssssssay=="<<str<<endl;
	say(str);
	//	}
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*--------------------------  Learning Approcahes  -------------------------*/
/*--------------------------------------------------------------------------*/

/**
 * This method creates, loads and updates learning information and
 * starts the learning processes for agent
 */
bool BasicAgent::initializeLearning() {
	return true;
}

/**
 * This method saves all agent experiences and clears up variables on memory
 */
void BasicAgent::finalizeLearning() {
}

/**
 * This method uses our Genetic Algorithm systems to learn agent to walk
 */
void BasicAgent::walkStraight() {
}

/**
 * This method is used to turn the agent to the specified angle
 * @param angTurn Angle for the agent to turn towards
 * @return bool When the agent has finished turning
 */

//*****************

bool BasicAgent::IsShootable()
{
	VecPosition vec=WM->getBallPos()-WM->getAgentPosition() ;
	if(CloseToBall(0.3))
		return true;

	return false;
}


//bool BasicAgent::IsPasable()
//{
//
//}

//***********************************

bool BasicAgent::CloseToBall(double e)
{
	VecPosition Agent,Ball;
	double dist;
	Agent = WM->getAgentPosition();
	Ball  =WM->getRelativePosition(OBJECT_BALL);
	Ball.setVecPosition(Ball[0],Ball[1],Ball[2],CARTESIAN);
	VecPosition Ball1=VecPosition::getVecPositionFromPolar(Ball[0],Ball[1],Ball[2]);
	dist=Ball1.getMagnitude2();
	if(Ball==NULL){
		Ball=WM->getBallPos();
		dist=Agent.getDistanceTo2(Ball);
	}
	cout<<"dist1"<<dist<<endl;
	if(dist < e)
		return true;
	else
		return false;
}
//***********************************

ObjectT
BasicAgent::getNear_Number(VecPosition My)
{
	double AgentDist,OpponentDist;
	static int iNumber,INUM,MinDist;
	INUM=0;
	MinDist=20;
	for(iNumber=2;iNumber<7;iNumber++)
	{
		if(WM->getAgentNumber()==iNumber)
			continue;
		ObjectT obj = SoccerTypes::getTeammateObjectFromIndex (iNumber-1);
		VecPosition u=WM->getGlobalPosition(obj);
		AgentDist=u.getDistanceTo2(My);
		cout<<"DUP"<<u<<" NUM == "<<iNumber<<"  AGENTDIST == "<<AgentDist<<endl;
		if(AgentDist>1000)
			continue;

		else if(AgentDist<1.5)
		{
			cout<<"LOW"<<endl;
			INUM=iNumber;
			break;
		}
		if(MinDist-AgentDist>0.25)
		{
			MinDist=AgentDist;
			INUM=iNumber;
		}
	}

	cout<<"INUM"<<INUM<<endl;
	if(INUM==0)
	{
		cout<<"NUMBER21"<<endl;
		iNumber=1;
		INUM=0;
		return  OBJECT_TEAMMATE_UNKNOWN;
	}
	if(INUM==2)
	{
		iNumber=1;
		INUM=0;
		cout<<"NUMBER2"<<endl;
		return OBJECT_TEAMMATE_2;
	}
	else if(INUM==3)
	{
		iNumber=1;
		INUM=0;
		cout<<"NUMBER3"<<endl;
		return OBJECT_TEAMMATE_3;
	}
	else if(INUM==4)
	{
		iNumber=1;
		INUM=0;
		cout<<"NUMBER4"<<endl;
		return OBJECT_TEAMMATE_4;
	}
	else if(INUM==5)
	{
		iNumber=1;
		INUM=0;
		cout<<"NUMBER5"<<endl;
		return OBJECT_TEAMMATE_5;
	}
	else if(INUM==6)
	{
		iNumber=0;
		INUM=0;
		cout<<"NUMBER6"<<endl;
		return OBJECT_TEAMMATE_6;
	}
}

//***********************************


bool BasicAgent::DoAct()
{
	cout<<"\n this_action in DoAct is : "<<this_action<<" and last_action in DoAct is : "<<last_action<<endl;
	if( last_action != this_action || this_action == actIsDone )
		Hstate = nothing;

	if(this_action == actIsDone)
		return true;
	else
	{
		if(this_action==nothing)
			return reset();
//		else if(this_action == check_kick)
//			return Check_kickingto(GOAL);
		else if(this_action == standingUp)
			return StandUp();
		else if(this_action == shooting)
			return shoot();
		else if(this_action == goingToPos_R)
			return gotoposRelative(ERROR_MOVE);
		else if(this_action == goingToPos)
			return gotopos(GP , ERROR_MOVE);
		else if(this_action == goingToPos_c)
			return gotopos_c(GP , ERROR_MOVE);
	}
	return false;
}
bool BasicAgent::Do_LAct()
{
	cout<<"\n l_action in DoAct is : "<<l_action<<" and last_l_action in DoAct is : "<<last_l_action<<endl;
	if( last_l_action != l_action || l_action == l_actDone )
		state = 1;
	if(l_action == l_actDone)
		return true;
	else
	{
		if(l_action==l_nothing)
			return reset();
		else if(l_action==l_reseting)
			return reset();
		else if(l_action == l_kicking_l)
			return kick_l(true);
		else if(l_action == l_kicking_r)
			return kick_r(true);
		else if(l_action == l_littelwalking)
			return littlewalk(true);
		else if(l_action == l_moving_l)
			return move_l(true);
		else if(l_action == l_moving_r)
			return move_r(true);
		else if(l_action == l_turning_l)
			return turn_l(true);
		else if(l_action == l_turning_r)
			return turn_r(true);
		else if(l_action == l_walking)
			return walk(true);
		else if(l_action == l_walkingback)
			return walkback(true);
		else if(l_action == l_standing_b)
			return StandBack(true);
		else if(l_action == l_standing_f)
			return StandFront(true);
		else if(l_action == l_ghalt)
			return Ghalt(true);
	}
}

void BasicAgent::Say_Pass(ObjectT obj )
{

	if(obj == OBJECT_TEAMMATE_2)
		say("@");
	else if(obj == OBJECT_TEAMMATE_3)
		say("#");
	else if(obj == OBJECT_TEAMMATE_4)
		say("$");
	else if(obj == OBJECT_TEAMMATE_5)
		say("%");
	else if(obj == OBJECT_TEAMMATE_6)
		say("^");
}


