/*
 *********************************************************************************
 *          BasicAgent.cpp : Robocup 3D Soccer Simulation Team Zigorat           *
 *                                                                               *
 *  Date: 03/20/2007                                                             *
 *  Author: Mahdi Hamdarsi                                                       *
 *  Comments: This file contains class definition BasicAgent which contains      *
 *            several routines needed for thinkin' module                        *
 *                                                                               *
 *********************************************************************************
 */

/*! \file BasicAgent.cpp
 <pre>
 <b>File:</b>          BasicAgent.cpp
 <b>Project:</b>       Robocup Soccer Simulation Team: Zigorat
 <b>Authors:</b>       Mahdi Hamdarsi
 <b>Created:</b>       03/20/2007
 <b>Last Revision:</b> $ID$
 <b>Contents:</b>      This file contains class definition BasicAgent which contains
 several routines needed for thinkin' module
 <hr size=2>
 <h2><b>Changes</b></h2>
 <b>Date</b>             <b>Author</b>          <b>Comment</b>
 03/20/2007       Mahdi           Initial version created
 </pre>
 */

#include "BasicAgent.h"
#include "Logger.h"

using namespace std;
ofstream out01("LOG/Head_Yaw.txt"		,ios::app),
			 out02("LOG/Head_Pitch.txt"		,ios::app),
			 out03("LOG/LShoulder_Pitch.txt",ios::app),
			 out04("LOG/LShoulder_Roll.txt"	,ios::app),
			 out05("LOG/LElbow_Roll.txt"	,ios::app),
			 out06("LOG/LElbow_Pitch.txt"	,ios::app),
			 out07("LOG/RShoulder_Pitch.txt",ios::app),
			 out08("LOG/RShoulder_Roll.txt"	,ios::app),
			 out09("LOG/RElbow_Roll.txt"	,ios::app),
			 out10("LOG/RElbow_Pitch.txt"	,ios::app),
			 out11("LOG/LHip_YawPitch.txt"	,ios::app),
			 out12("LOG/LHip_Pitch.txt"		,ios::app),
			 out13("LOG/LHip_Roll.txt"		,ios::app),
			 out14("LOG/LKnee_Pitch.txt"	,ios::app),
			 out15("LOG/LAnkle_Picth.txt"	,ios::app),
			 out16("LOG/LAnkle_Roll.txt"	,ios::app),
			 out17("LOG/RHip_YawPitch.txt"	,ios::app),
			 out18("LOG/RHip_Pitch.txt"		,ios::app),
			 out19("LOG/RHip_Roll.txt"		,ios::app),
			 out20("LOG/RKnee_Pitch.txt"	,ios::app),
			 out21("LOG/RAnkle_Picth.txt"	,ios::app),
	         out22("LOG/RAnkle_Roll.txt"	,ios::app),
			out23("LOG/Action.txt"	,ios::app);

/*! This is the constructor of BasicAgent, initilizes self
 variables, sends identification signal to the server
 and starts the agent
 */
BasicAgent::BasicAgent() {
	m_IsFirst = true ;	//abc 15
	m_Finished = false;
	connection = RoboCupConnection::getSingleton();
	WM = WorldModel::getSingleton();
	ACT = ActHandler::getSingleton();
	AS = AgentSettings::getSingleton();
	Log = Logger::getSingleton();
	fallFront=false;
	fallBack=false;
	fallGhalt=false;

	neck[0] .initial(JID_HEAD_1);
	neck[1] .initial(JID_HEAD_2);
	arm[0][0].initial(JID_LARM_1);
	arm[1][0].initial(JID_LARM_2);
	arm[2][0].initial(JID_LARM_3);
	arm[3][0].initial(JID_LARM_4);
	arm[0][1].initial(JID_RARM_1);
	arm[1][1].initial(JID_RARM_2);
	arm[2][1].initial(JID_RARM_3);
	arm[3][1].initial(JID_RARM_4);
	leg[0][0].initial(JID_LLEG_1);
	leg[1][0].initial(JID_LLEG_3);
	leg[2][0].initial(JID_LLEG_2);
	leg[3][0].initial(JID_LLEG_4);
	leg[4][0].initial(JID_LLEG_5);
	leg[5][0].initial(JID_LLEG_6);
	leg[0][1].initial(JID_RLEG_1);
	leg[1][1].initial(JID_RLEG_3);
	leg[2][1].initial(JID_RLEG_2);
	leg[3][1].initial(JID_RLEG_4);
	leg[4][1].initial(JID_RLEG_5);
	leg[5][1].initial(JID_RLEG_6);
	heap=VecPosition(0,0,0.2),ankleLeft=VecPosition(0,0,0),ankleRight=VecPosition(0,0,0);
	walkState = 1;shootState=1;walkPriod=0;kickState=1,TurnState=1;StandState=1;
	GotoPos_State =0;GP_gain=0;gain=0;
	Behind_state = 0;
	checkState=1;
	side_kick = SIDE_RIGHT;
	player = WM->getAgentPosition();
	toop = WM->getBallPos();
	player_theta = WM->get_Theta();
//	Agent = WM->getAgentPosition();
//	Ball = WM->getBallPos();
	isFirst = true; isFirst_kick = true; isFirst_shoot = true; isFirst_theta = true;
	state=1;//neck
	this_action = nothing ; 	//abc 14
    KState = 0;TState = 0;
	Speed=0.1;delta=0.1;walkSpeed=0.1;deltaSpeed=0.1;Angle=0;
	Weight=0;Lenght=0;
	Turn=1;
	ang=0;
	b11 = true;
	result=false;
	rotate=0;
	Gain=0;
}

/*! This is the destructor of the basicAgent claass, fres all the instances
 of planners and deletes them all.
 */
BasicAgent::~BasicAgent() {
	finalizeLearning();
}
//bool	BasicAgent::Head_Yaw				(double value,double gain,double error)
//{
//	bool result;
//
//	return neck[0].setAngle(value,gain,error);
//}
//bool	BasicAgent::Head_Pitch				(double value,double gain,double error)
//{
//	bool result;
//
//	return neck[1].setAngle(value,gain,error);
//}
//bool 	BasicAgent::LShoulder_Pitch			(double value,double gain,double error)
//{
//	bool result;
//
//	return arm[0][0].setAngle(value,gain,error);
//}
//bool 	BasicAgent::LShoulder_Roll			(double value,double gain,double error)
//{
//	bool result;
//
//	return arm[1][0].setAngle(value,gain,error);
//}
//bool 	BasicAgent::LElbow_Roll				(double value,double gain,double error)
//{
//	bool result;
//	return arm[2][0].setAngle(value,gain,error);
//}
//bool 	BasicAgent::LElbow_Pitch			(double value,double gain,double error)
//{
//	bool result;
//	return arm[3][0].setAngle(value,gain,error);
//}
//bool 	BasicAgent::RShoulder_Pitch			(double value,double gain,double error)
//{
//	bool result;
//	return arm[0][1].setAngle(value,gain,error);
//}
//bool 	BasicAgent::RShoulder_Roll			(double value,double gain,double error)
//{
//	bool result;
//	return arm[1][1].setAngle(value,gain,error);
//}
//bool 	BasicAgent::RElbow_Roll				(double value,double gain,double error)
//{
//	bool result;
//	return arm[2][1].setAngle(value,gain,error);
//}
//bool 	BasicAgent::RElbow_Pitch			(double value,double gain,double error)
//{
//	bool result;
//	return arm[3][1].setAngle(value,gain,error);
//}
//bool 	BasicAgent::LHip_YawPitch			(double value,double gain,double error)
//{
//	bool result;
//	return leg[0][0].setAngle(value,gain,error);
//}
//bool 	BasicAgent::LHip_Pitch				(double value,double gain,double error)
//{
//	bool result;
//	return leg[1][0].setAngle(value,gain,error);
//}
//bool 	BasicAgent::LHip_Roll				(double value,double gain,double error)
//{
//	bool result;
//	return leg[2][0].setAngle(value,gain,error);
//}
//bool 	BasicAgent::LKnee_Pitch				(double value,double gain,double error)
//{
//	bool result;
//	return leg[3][0].setAngle(value,gain,error);
//}
//bool 	BasicAgent::LAnkle_Picth			(double value,double gain,double error)
//{
//	bool result;
//	return leg[4][0].setAngle(value,gain,error);
//}
//bool 	BasicAgent::LAnkle_Roll				(double value,double gain,double error)
//{
//	bool result;
//	return leg[5][0].setAngle(value,gain,error);
//}
//bool 	BasicAgent::RHip_YawPitch			(double value,double gain,double error)
//{
//	bool result;
//	return leg[0][1].setAngle(value,gain,error);
//}
//bool 	BasicAgent::RHip_Pitch				(double value,double gain,double error)
//{
//	bool result;
//	return leg[1][1].setAngle(value,gain,error);
//}
//bool 	BasicAgent::RHip_Roll				(double value,double gain,double error)
//{
//	bool result;
//	return leg[2][1].setAngle(value,gain,error);
//}
//bool 	BasicAgent::RKnee_Pitch				(double value,double gain,double error)
//{
//	bool result;
//	return leg[3][1].setAngle(value,gain,error);
//}
//bool 	BasicAgent::RAnkle_Picth			(double value,double gain,double error)
//{
//	bool result;
//	return leg[4][1].setAngle(value,gain,error);
//}
//bool 	BasicAgent::RAnkle_Roll				(double value,double gain,double error)
//{
//	bool result;
//	return leg[5][1].setAngle(value,gain,error);
//}
bool	BasicAgent::Head_Yaw				(double value,double gain,double error)
{
//	return neck[0].setAngle(value,gain,error);
	bool result = neck[0].setAngle(value,gain,error);
	out01<<WM->getCurrentCycle()<<"   "<<neck[0].getCurrentAngle()<<"   "<<value<<"   "<<neck[0].getSentAngle()<<"   "<<neck[0].getLast_ang()<<"   "<<value-neck[0].getCurrentAngle()<<endl;
	return result;
}
bool	BasicAgent::Head_Pitch				(double value,double gain,double error)
{
//	return neck[1].setAngle(value,gain,error);
	bool result = neck[1].setAngle(value,gain,error);
	out02<<WM->getCurrentCycle()<<"   "<<neck[1].getCurrentAngle()<<"   "<<value<<"   "<<neck[1].getSentAngle()<<"   "<<neck[1].getLast_ang()<<"   "<<value-neck[1].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::LShoulder_Pitch			(double value,double gain,double error)
{
//	return arm[0][0].setAngle(value,gain,error);
	bool result = arm[0][0].setAngle(value,gain,error);
	out03<<WM->getCurrentCycle()<<"   "<<arm[0][0].getCurrentAngle()<<"   "<<value<<"   "<<arm[0][0].getSentAngle()<<"   "<<arm[0][0].getLast_ang()<<"   "<<value-arm[0][0].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::LShoulder_Roll			(double value,double gain,double error)
{
//	return arm[1][0].setAngle(value,gain,error);
	bool result = arm[1][0].setAngle(value,gain,error);
	out04<<WM->getCurrentCycle()<<"   "<<arm[1][0].getCurrentAngle()<<"   "<<value<<"   "<<arm[1][0].getSentAngle()<<"   "<<arm[1][0].getLast_ang()<<"   "<<value-arm[1][0].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::LElbow_Roll				(double value,double gain,double error)
{
//	return arm[2][0].setAngle(value,gain,error);
	bool result = arm[2][0].setAngle(value,gain,error);
	out05<<WM->getCurrentCycle()<<"   "<<arm[2][0].getCurrentAngle()<<"   "<<value<<"   "<<arm[2][0].getSentAngle()<<"   "<<arm[2][0].getLast_ang()<<"   "<<value-arm[2][0].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::LElbow_Pitch			(double value,double gain,double error)
{
//	return arm[3][0].setAngle(value,gain,error);
	bool result = arm[3][0].setAngle(value,gain,error);
	out06<<WM->getCurrentCycle()<<"   "<<arm[3][0].getCurrentAngle()<<"   "<<value<<"   "<<arm[3][0].getSentAngle()<<"   "<<arm[3][0].getLast_ang()<<"   "<<value-arm[3][0].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::RShoulder_Pitch			(double value,double gain,double error)
{
//	return arm[0][1].setAngle(value,gain,error);
	bool result = arm[0][1].setAngle(value,gain,error);
	out07<<WM->getCurrentCycle()<<"   "<<arm[0][1].getCurrentAngle()<<"   "<<value<<"   "<<arm[0][1].getSentAngle()<<"   "<<arm[0][1].getLast_ang()<<"   "<<value-arm[0][1].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::RShoulder_Roll			(double value,double gain,double error)
{
//	return arm[1][1].setAngle(value,gain,error);
	bool result = arm[1][1].setAngle(value,gain,error);
	out08<<WM->getCurrentCycle()<<"   "<<arm[1][1].getCurrentAngle()<<"   "<<value<<"   "<<arm[1][1].getSentAngle()<<"   "<<arm[1][1].getLast_ang()<<"   "<<value-arm[1][1].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::RElbow_Roll				(double value,double gain,double error)
{
//	return arm[2][1].setAngle(value,gain,error);
	bool result = arm[2][1].setAngle(value,gain,error);
	out09<<WM->getCurrentCycle()<<"   "<<arm[2][1].getCurrentAngle()<<"   "<<value<<"   "<<arm[2][1].getSentAngle()<<"   "<<arm[2][1].getLast_ang()<<"   "<<value-arm[2][1].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::RElbow_Pitch			(double value,double gain,double error)
{
//	return arm[3][1].setAngle(value,gain,error);
	bool result = arm[3][1].setAngle(value,gain,error);
	out10<<WM->getCurrentCycle()<<"   "<<arm[3][1].getCurrentAngle()<<"   "<<value<<"   "<<arm[3][1].getSentAngle()<<"   "<<arm[3][1].getLast_ang()<<"   "<<value-arm[3][1].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::LHip_YawPitch			(double value,double gain,double error)
{
//	return leg[0][0].setAngle(value,gain,error);
	bool result = leg[0][0].setAngle(value,gain,error);
	out11<<WM->getCurrentCycle()<<"   "<<leg[0][0].getCurrentAngle()<<"   "<<value<<"   "<<leg[0][0].getSentAngle()<<"   "<<leg[0][0].getLast_ang()<<"   "<<value-leg[0][0].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::LHip_Pitch				(double value,double gain,double error)
{
//	return leg[1][0].setAngle(value,gain,error);
	bool result = leg[1][0].setAngle(value,gain,error);
	out12<<WM->getCurrentCycle()<<"   "<<leg[1][0].getCurrentAngle()<<"   "<<value<<"   "<<leg[1][0].getSentAngle()<<"   "<<leg[1][0].getLast_ang()<<"   "<<value-leg[1][0].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::LHip_Roll				(double value,double gain,double error)
{
//	return leg[2][0].setAngle(value,gain,error);
	bool result = leg[2][0].setAngle(value,gain,error);
	out13<<WM->getCurrentCycle()<<"   "<<leg[2][0].getCurrentAngle()<<"   "<<value<<"   "<<leg[2][0].getSentAngle()<<"   "<<leg[2][0].getLast_ang()<<"   "<<value-leg[2][0].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::LKnee_Pitch				(double value,double gain,double error)
{
//	return leg[3][0].setAngle(value,gain,error);
	bool result = leg[3][0].setAngle(value,gain,error);
	out14<<WM->getCurrentCycle()<<"   "<<leg[3][0].getCurrentAngle()<<"   "<<value<<"   "<<leg[3][0].getSentAngle()<<"   "<<leg[3][0].getLast_ang()<<"   "<<value-leg[3][0].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::LAnkle_Picth			(double value,double gain,double error)
{
//	return leg[4][0].setAngle(value,gain,error);
	bool result = leg[4][0].setAngle(value,gain,error);
	out15<<WM->getCurrentCycle()<<"   "<<leg[4][0].getCurrentAngle()<<"   "<<value<<"   "<<leg[4][0].getSentAngle()<<"   "<<leg[4][0].getLast_ang()<<"   "<<value-leg[4][0].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::LAnkle_Roll				(double value,double gain,double error)
{
//	return leg[5][0].setAngle(value,gain,error);
	bool result = leg[5][0].setAngle(value,gain,error);
	out16<<WM->getCurrentCycle()<<"   "<<leg[5][0].getCurrentAngle()<<"   "<<value<<"   "<<leg[5][0].getSentAngle()<<"   "<<leg[5][0].getLast_ang()<<"   "<<value-leg[5][0].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::RHip_YawPitch			(double value,double gain,double error)
{
//	return leg[0][1].setAngle(value,gain,error);
	bool result = leg[0][1].setAngle(value,gain,error);
	out17<<WM->getCurrentCycle()<<"   "<<leg[0][1].getCurrentAngle()<<"   "<<value<<"   "<<leg[0][1].getSentAngle()<<"   "<<leg[0][1].getLast_ang()<<"   "<<value-leg[0][1].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::RHip_Pitch				(double value,double gain,double error)
{
//	return leg[1][1].setAngle(value,gain,error);
	bool result = leg[1][1].setAngle(value,gain,error);
	out18<<WM->getCurrentCycle()<<"   "<<leg[1][1].getCurrentAngle()<<"   "<<value<<"   "<<leg[1][1].getSentAngle()<<"   "<<leg[1][1].getLast_ang()<<"   "<<value-leg[1][1].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::RHip_Roll				(double value,double gain,double error)
{
//	return leg[2][1].setAngle(value,gain,error);
	bool result = leg[2][1].setAngle(value,gain,error);
	out19<<WM->getCurrentCycle()<<"   "<<leg[2][1].getCurrentAngle()<<"   "<<value<<"   "<<leg[2][1].getSentAngle()<<"   "<<leg[2][1].getLast_ang()<<"   "<<value-leg[2][1].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::RKnee_Pitch				(double value,double gain,double error)
{
//	return leg[3][1].setAngle(value,gain,error);
	bool result = leg[3][1].setAngle(value,gain,error);
	out20<<WM->getCurrentCycle()<<"   "<<leg[3][1].getCurrentAngle()<<"   "<<value<<"   "<<leg[3][1].getSentAngle()<<"   "<<leg[3][1].getLast_ang()<<"   "<<value-leg[3][1].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::RAnkle_Picth			(double value,double gain,double error)
{
//	return leg[4][1].setAngle(value,gain,error);
	bool result = leg[4][1].setAngle(value,gain,error);
	out21<<WM->getCurrentCycle()<<"   "<<leg[4][1].getCurrentAngle()<<"   "<<value<<"   "<<leg[4][1].getSentAngle()<<"   "<<leg[4][1].getLast_ang()<<"   "<<value-leg[4][1].getCurrentAngle()<<endl;
	return result;
}
bool 	BasicAgent::RAnkle_Roll				(double value,double gain,double error)
{
//	return leg[5][1].setAngle(value,gain,error);
	bool result = leg[5][1].setAngle(value,gain,error);
	out22<<WM->getCurrentCycle()<<"   "<<leg[5][1].getCurrentAngle()<<"   "<<value<<"   "<<leg[5][1].getSentAngle()<<"   "<<leg[5][1].getLast_ang()<<"   "<<value-leg[5][1].getCurrentAngle()<<endl;
	return result;
}




bool BasicAgent::rotateLeg(SideT side, AngDeg value, double dSpeed , double error)
{
	if(side == SIDE_LEFT)
	{
		return LHip_YawPitch(value,dSpeed,error);
	}
	else
	{
		return RHip_YawPitch(value,dSpeed,error);
	}
}

/*! This method changes the angle of First joint of arm(Shoulder), if
 currently value is near the current angle of knee then no changes
 are made.
 \param side which leg to change angle
 \param value new angle to change to
 \param dSpeed speed to set for action */
bool BasicAgent::rotateArm(SideT side, AngDeg value, double dSpeed , double error)
{
	if(side == SIDE_LEFT)
	{
		return LElbow_Roll(value,dSpeed,error);
	}
	else
	{
		return RElbow_Roll(value,dSpeed,error);
	}
}

/*! This method changes the angle of knee, if currently value is near
 the current angle of knee then no changes are made
 \param side which leg to change angle
 \param value new angle to change to
 \param dSpeed speed to set for action */
bool BasicAgent::knee(SideT side, AngDeg value, double dSpeed , double error)
{
	if(side == SIDE_LEFT)
	{
		return LKnee_Pitch(value,dSpeed,error);
	}
	else
	{
		return RKnee_Pitch(value,dSpeed,error);
	}
}
/*! This method changes the angle of knee, if currently value is near
 the current angle of knee then no changes are made
 \param side which leg to change angle
 \param value new angle to change to
 \param dSpeed speed to set for action */
bool BasicAgent::elbow(SideT side, AngDeg value, double dSpeed , double error)
{
	if(side == SIDE_LEFT)
	{
		return LElbow_Pitch(value,dSpeed,error);
	}
	else
	{
		return RElbow_Pitch(value,dSpeed,error);
	}
}
/*--------------------------------------------------------------------------*/
/*--------------------- Changing of universal joints -----------------------*/
/*--------------------------------------------------------------------------*/

/*! This method changes the angle of shoulder, if currently value is near
 the current angle of shoulder then no changes are made
 \param side which leg to change angle
 \param valueFront New angle ( for moving the should towards front and back )
 \param valueOpen New angle ( for Opening and closing the arms )
 \param dSpeed speed to set for action */
bool BasicAgent::shoulder(SideT side, AngDeg valueFront, AngDeg valueOpen, double dSpeed , double error)
{
	if(side == SIDE_LEFT)
	{
		bool bResult = true;
		bResult &= LShoulder_Pitch(valueFront,dSpeed,error);
		bResult &= LShoulder_Roll(valueOpen,dSpeed,error);
		return bResult;
	}
	else
	{
		bool bResult = true;
		bResult &= RShoulder_Pitch(valueFront,dSpeed,error);
		bResult &= RShoulder_Roll(valueOpen,dSpeed,error);
		return bResult;
	}
}

/*! This method changes the angle of hip, if currently value is near
 the current angle of hip then no changes are made
 \param side which leg to change angle
 \param valueFront New angle ( for moving the should towards front and back )
 \param valueOpen New angle ( for Opening and closing the arms )
 \param dSpeed speed to set for action */
bool BasicAgent::hip(SideT side, AngDeg valueFront, AngDeg valueOpen, double dSpeed , double error)
{
	if(side == SIDE_LEFT)
	{
		bool bResult=true;
		bResult &= LHip_Pitch(valueFront,dSpeed,error);
		bResult &= LHip_Roll(valueOpen,dSpeed,error);
		return bResult;
	}
	else
	{
		bool bResult=true;
		bResult &= RHip_Pitch(valueFront,dSpeed,error);
		bResult &= RHip_Roll(valueOpen,dSpeed,error);
		return bResult;
	}
}

/*! This method changes the angle of shoulder, if currently value is near
 the current angle of hip then no changes are made
 \param side which leg to change angle
 \param valueFront new angle to change to
 \param valueOpen new angle to change to
 \param dSpeed speed to set for action */
bool BasicAgent::ankle(SideT side, AngDeg valueFront, AngDeg valueOpen, double dSpeed , double error)
{
	if(side == SIDE_LEFT)
	{
		bool bResult=true;
		bResult &= LAnkle_Picth(valueFront,dSpeed,error);
		bResult &= LAnkle_Roll(valueOpen,dSpeed,error);
		return bResult;
	}
	else
	{
		bool bResult=true;
		bResult &= RAnkle_Picth(valueFront,dSpeed,error);
		bResult &= RAnkle_Roll(valueOpen,dSpeed,error);
		return bResult;
	}

}
/*! This method inits agents information by loading agents scene graph
 into WorldModel and then it will send scene command to the simulator */
bool BasicAgent::initAgentScene() {
	// Load agent information like joints and sizes into WorldModel
	// This may take a while, so do it before initializing the simulator
	AgentModelT model = AS->getAgentModel();
	if (!WM->setAgentModel(model))
		return false;

	// Send scene command to init the agent
	// use the scene effector to build the agent and beam to strategic position
	char strScene[128] = "";
	sprintf(strScene, "(scene %s)",
			SoccerTypes::getAgentModelString(model).c_str());
	ACT->sendMessage(strScene);

	bool joint_used[MAX_JOINTS];
	for (unsigned i = 0; i < MAX_JOINTS; i++)
		joint_used[i] = SoccerTypes::isJointBeingUsed((JointT) i);
	ACT->setUsedJoints(joint_used);
	return true;
}

/**
 * This method beams agent to his current strategic position
 * @param angBody Body angle when beamed to the strategic position
 */
void BasicAgent::moveToStrategicPosition(AngDeg angBody) {
	VecPosition posBeam = WM->getStrategicPosition();
	beam(posBeam[0], posBeam[1], angBody);
	//beam(4,4,angBody);
}

/*! This method inits the agent for server by sending his name and number */
void BasicAgent::sendPlayerInformation() {
	static int iFirstCycle = WM->getCurrentCycle();
	if ((WM->getCurrentCycle() - iFirstCycle) == 1) {
		char cmd[128];
		sprintf(cmd, "(init (unum %d)(teamname %s))", WM->getAgentNumber(),
				WM->getTeamName().c_str());
		ACT->sendMessage(cmd);
	} /*else if ((WM->getCurrentCycle() - iFirstCycle) == 10)
		moveToStrategicPosition();*/
}

/**
 * This method just says something to indicate that agent
 has finished execution in the normal way.
 */
void BasicAgent::sayGoodbye() {
	if (WM->getAgentNumber() == 1)
		cout << "Goalie out." << endl;
	else
		cout << "#" << WM->getAgentNumber() << " out." << endl;

	Log->log(6, "Agent execution finished");
}

/*! This method sends a beam command to server: A beam command is a command
 that moves a robot in less than a moment to the desired location (not
 in the gameplay)
 \param dx X-Coordinate of desired move location
 \param dy Y-Coordinate of desired move location
 \param dAng Agent direction when has moved to the deisred location */
void BasicAgent::beam(double dx, double dy, double dAng) {
	if (WM->canAgentBeam()) {
		char str[128];
		sprintf(str, "(beam %f %f %f)", dx, dy, dAng);
		ACT->sendMessage(str);
	}
}

/**
 * This method sends a string to be broadcasted to all teammates
 * @param str String containing the talk message
 */
void BasicAgent::say(const char * str) {
	ACT->putCommandInQueue(SoccerCommand(CMD_SAY, str));
}

/*! This function directly adjustes joints angular speeds by
 the values given
 \param id Joint ID to be updated
 \param dSpeedMotor1 Joints first motors angular speed
 \param dSpeedMotor2 Joints second motors angular speed
 \return bool indicating whether information were accepted or not
 */
bool BasicAgent::adjustJoint(JointT id, double dSpeedMotor1,
		double dSpeedMotor2) {
	if (dSpeedMotor1 == UnknownDoubleValue || id == JID_ILLEGAL)
		return false;

	if (dSpeedMotor2 == UnknownDoubleValue)
		dSpeedMotor2 = 0;

	////////////////////////////////////////////////////German change
	double dsm1 = dSpeedMotor1;
	double dsm2 = dSpeedMotor2;
	ACT->putCommandInQueue(SoccerCommand(CMD_JOINT, id, dsm1,
			dsm2));
	//	ACT->putCommandInQueue(SoccerCommand(CMD_JOINT, id, dsm1/2,
	//			dsm2/2));
	return true;
}

/*--------------------------------------------------------------------------*/
/*----------------------- Changing of hinge joints -------------------------*/


/*--------------------------------------------------------------------------*/
/*--------------------------  Heuristic Procedures  ------------------------*/
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/*--------------------------  Heuristic Procedures  ------------------------*/
/*--------------------------------------------------------------------------*/
// Remained from Gcyrus

/********************************
 *   behind		    .			*
 * 								*
 * 			 pos				*
 * 								*
 *		.			.			*
 *								*
 *						goal	*
 ********************************/

VecPosition BasicAgent::Behind_ofPos_l(VecPosition pos , VecPosition goal)
{

	AngDeg Direction_PosAndGoal = VecPosition::normalizeAngle((goal - pos).getDirection());
	VecPosition Behind_of_Pos = VecPosition ( pos.getX() - 0.2 * cosDeg(Direction_PosAndGoal) ,
			pos.getY() - 0.2 * sinDeg(Direction_PosAndGoal));
	return Behind_of_Pos;
}

/********************************
 *   	.		    .			*
 * 								*
 * 			 pos				*
 * 								*
 *		.	      behind		*
 *								*
 *						goal	*
 ********************************/

VecPosition BasicAgent::Behind_ofPos_r(VecPosition pos , VecPosition Behind_of_Pos , AngDeg slope , double distancce)
{
	VecPosition behind;
	double x,y;
	x = (distancce / sqrt(1 + slope*slope)) + pos.getX();
	y = slope * (x - pos.getX()) + pos.getY();
	cout<<"  XX = "<<x<<"  YY = "<<y
			<<" sqrt(1 + slope*slope) = "<<sqrt(1 + slope*slope)
			<<"  distancce / sqrt(1 + slope*slope) = "<<distancce / sqrt(1 + slope*slope)
			<<endl;
	if(x != Behind_of_Pos.getX() && y != Behind_of_Pos.getY())
		behind = VecPosition(x,y);
	else
	{
		x = (-1 * distancce) / sqrt(1 + slope*slope) + pos.getX();
		y = slope * (x - pos.getX()) + pos.getY();
		behind = VecPosition(x,y);
		cout<<"  XxX = "<<x<<"  YyY = "<<y<<endl;

	}
	return behind;
}

/********************************
 *   	.		    behind		*
 * 								*
 * 			 pos				*
 * 								*
 *		.	      	.			*
 *								*
 *						goal	*
 ********************************/

VecPosition BasicAgent::Behind_ofPos_u(VecPosition pos , AngDeg slope , double distancce)
{
	VecPosition behind;
	double x,y;
	AngDeg slope_  = -1 / slope;
	x = distancce / sqrt(1 + slope_*slope_) + pos.getX();
	y = slope_ * (x - pos.getX()) + pos.getY();
	behind = VecPosition(x,y);
	return behind;
}

/********************************
 *   	.		    .			*
 * 								*
 * 			 pos				*
 * 								*
 *		behind	   	.			*
 *								*
 *						goal	*
 ********************************/

VecPosition BasicAgent::Behind_ofPos_d(VecPosition pos , AngDeg slope , double distancce)
{
	VecPosition behind;
	double x,y;
	AngDeg slope_ = -1 / slope;
	x = (-1 * distancce) / sqrt(1 + slope_*slope_) + pos.getX();
	y = slope_ * (x - pos.getX()) + pos.getY();
	behind = VecPosition(x,y);
	return behind;
}


bool BasicAgent::Rneck(bool start) {
	bool a;
	if (start) {

		//state 1
		if(state ==1 ){
			a=Head_Yaw(50,10);
			if (a )
			{
				//								l_action = l_Rneck;
		cout<<"teeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeest   "<<state<<endl;
				state ++;
			}
			return false;
		}
		if(state ==2 ){
			a=Head_Yaw(20,10);
			if (a )
			{

				//										l_action = l_Rneck;
				state ++;
			}
			return false;
		}


		if(state ==3 ){///////////////////////
			a=Head_Yaw(-20,10);
			if (a )
			{
				//				l_action = l_Rneck;
				state ++;
			}
			return false;
		}

		else if(state == 4){
			a=Head_Yaw(-50,10);
			//			a=neck(   0,   -20,5 );
			if (a )
			{
				//								l_action = l_actDone;
				state=1;
				return true;
			}
			return false;
		}
	}
	return false;
}
bool BasicAgent::Behind (VecPosition goal , double error)
{
	out23<<WM->getCurrentCycle()<<"  behind1"<<endl;
	if(WM->getCurrentCycle() % 250 == 0 || isFirst)
	{
		out23<<WM->getCurrentCycle()<<"  behind2"<<endl;
		toop = WM->getBallPos();
		player = WM->getAgentPosition();
		isFirst=false;
	}
	VecPosition zero = VecPosition(0,0,0);
//	double DistanceToBall =  Agent.getDistanceTo2(Ball);
	AngDeg Direction_PosAndGoal = VecPosition::normalizeAngle((goal - toop).getDirection());
	VecPosition Behind_l = Behind_ofPos_l(toop , goal);
	double distance = (toop - Behind_l).getMagnitude2();
	VecPosition Behind_r = Behind_ofPos_r(toop , Behind_l ,Direction_PosAndGoal , distance);
	VecPosition Behind_u = Behind_ofPos_u(toop , Direction_PosAndGoal , distance);
	Behind_l.setX(Behind_l.getX()-0.10);
	VecPosition Behind_d = Behind_ofPos_d(toop , Direction_PosAndGoal , distance);
	Behind_d.setX(Behind_d.getX()-0.10);
	AngDeg ang = (Behind_ofPos_l(toop , goal) - toop).getDirection();
	Matrix m_t = Matrix::createMatrixFromVecPosition(
			VecPosition(1,0,0),
			VecPosition(-1*toop.getX(),1-toop.getY(),0),
			VecPosition(0,0,0)
	);
	Matrix m_r = Matrix::createRotationMatrix(-1 * ang);
	Matrix m_agent = Matrix::createMatrixFromVecPosition(player,zero,zero);
	Matrix m = m_t * m_agent;
	VecPosition agent_New = VecPosition(
			(m_r * m).getElement(1,1),
			(m_r * m).getElement(2,1),
			(m_r * m).getElement(3,1)
	);
//		cout<<
//				"\nBall = "<<Ball<<"\tagent = "<<Agent
//				<<"\tDirection_PosAndGoal = "<<Direction_PosAndGoal
//				<<"\tBehind_l = "<<Behind_l
//				<<"\tdistance = "<<distance
//				<<"\tBehind_r = "<<Behind_r
//				<<"\tBehind_u = "<<Behind_u
//				<<"\tBehind_d = "<<Behind_d
//				<<"\tang = "<<ang
//				<<"\tm_t = "<<m_t.str()
//				<<"\tm_r = "<<m_r.str()
//				<<"\tm = "<<m.str()
//				<<"\tm_agent = "<<m_agent.str()
//				<<"\tm_t*m_agent = "<<(m_t*m_agent).str()
//				<<"\t(m_t * m_agent).getElement(1,1) = "<<(m_t * m_agent).getElement(1,1)
//				<<"\t(m_t * m_agent).getElement(2,1) = "<<(m_t * m_agent).getElement(2,1)
//				<<"\t(m_t * m_agent).getElement(3,1) = "<<(m_t * m_agent).getElement(3,1)
//				<<"\tagent_New = "<<agent_New
//				<<endl;
	switch (Behind_state)
	{
	case 0:
		out23<<WM->getCurrentCycle()<<"  behind3"<<endl;
		if(agent_New.getDistanceTo2(Behind_u)<error || agent_New.getDistanceTo2(Behind_d)<error)
			Behind_state=1;//l;
		else
		{
			if(agent_New.getX() > 0)
			{
				out23<<WM->getCurrentCycle()<<"  behind4"<<endl;
				if(agent_New.getY() < 0)
				{
					out23<<WM->getCurrentCycle()<<"  behind5"<<endl;
					if(agent_New.getY() > Behind_d.getY())
					{
						out23<<WM->getCurrentCycle()<<"  behind6"<<endl;
						Behind_state = 2;//d;
					}
					else
					{
						out23<<WM->getCurrentCycle()<<"  behind7"<<endl;
						Behind_state = 1;//l;
					}
				}
				else
				{
					out23<<WM->getCurrentCycle()<<"  behind8"<<endl;
					if(agent_New.getY() < Behind_u.getY())
					{
						out23<<WM->getCurrentCycle()<<"  behind9"<<endl;
						Behind_state = 3;//u;
					}
					else
					{
						out23<<WM->getCurrentCycle()<<"  behind10"<<endl;
						Behind_state =1;// l;
					}
				}
			}
			else
			{
				out23<<WM->getCurrentCycle()<<"  behind11"<<endl;
				Behind_state = 1;//l;
			}

		}
		break;
	case 1:
		out23<<WM->getCurrentCycle()<<"  behind12"<<endl;
		if(agent_New.getDistanceTo2(Behind_l) > error)
		{
			out23<<WM->getCurrentCycle()<<"  behind13"<<endl;
			if(GotoPos(Behind_l,error,1,0.5))
				{
				out23<<WM->getCurrentCycle()<<"  behind14"<<endl;
					return true;
				}
		}
		else
			{
			out23<<WM->getCurrentCycle()<<"  behind15"<<endl;
				return true;
			}
		break;
	case 2:
		out23<<WM->getCurrentCycle()<<"  behind16"<<endl;
		if(agent_New.getDistanceTo2(Behind_d) > error)
		{
			out23<<WM->getCurrentCycle()<<"  behind17"<<endl;
			if(GotoPos(Behind_d,error,1,0.5))
				{
				out23<<WM->getCurrentCycle()<<"  behind18"<<endl;
					Behind_state=0;//s;
				}
		}
		else
			{
				out23<<WM->getCurrentCycle()<<"  behind19"<<endl;
				Behind_state=0;//s;
			}
		break;
	case 3:
		if(agent_New.getDistanceTo2(Behind_u) > error)
		{
			out23<<WM->getCurrentCycle()<<"  behind20"<<endl;
			if(GotoPos(Behind_u,error,1,0.5))
				{
					out23<<WM->getCurrentCycle()<<"  behind21"<<endl;
					Behind_state=0;//s;
				}
		}
		else
			{
				out23<<WM->getCurrentCycle()<<"  behind22"<<endl;
				Behind_state=0;//s;
			}
		break;
	default:
		{
			out23<<WM->getCurrentCycle()<<"  behind23"<<endl;
			Behind_state=0;//s;
		}
		break;
	}
	return false;
}

/*--------------------------------------------------------------------------*/
/*--------------------------  Turning Procedures  --------------------------*/
/*--------------------------------------------------------------------------*/


//////////////////////////////abc 13
bool BasicAgent::CheckLocalize ()
{
	if(!WM->get_r_localize ())
	{
		Log->log(14,"~~~~~~Basic Agent :: CheckLocalize");
		//***//cout<<"\n CheckLocalize "<<endl;
		//		Rneck(b1);
		if(WM->get_r_localize())
		{
			b11 = false;
			return true;
		}
		else
		{
			b11=true;
			return false;
		}

	}
	return true;
}
///////////////////////////////
bool BasicAgent::IsFallen()
{
	if(fabs(WM->fall[0])>9.8 || fabs(WM->fall[1])>9.8 ||fabs(WM->fall[2])>9.8 )
	{
		out23<<WM->getCurrentCycle()<<"  IsFallen1"<<endl;
		return false;
	}
	else
	{
		out23<<WM->getCurrentCycle()<<"  IsFallen2"<<endl;
		if(WM->fall[2]<1||l_action==l_standing_b||l_action==l_standing_f)
		{
			out23<<WM->getCurrentCycle()<<"  IsFallen3"<<endl;
			if(WM->fall[1]<-7.5)
			{
				out23<<WM->getCurrentCycle()<<"  IsFallen4"<<endl;
				Log->log(14,"**BasicAgent::IsFallen    FALL FRONT !");
				//***//cout<<"\nFALL FRONT "<<endl;
				fallFront= true;
				return true;
			}
			else if(WM->fall[1]>7.5)
			{
				out23<<WM->getCurrentCycle()<<"  IsFallen5"<<endl;
				Log->log(14,"**BasicAgent::IsFallen   FALL BACK !");
				//***//cout<<"\nFALL BACK "<<endl;
				fallBack=true;
				return true;
			}
			else if(WM->fall[0]<-7.5||WM->fall[0]>7.5)
			{
				out23<<WM->getCurrentCycle()<<"  IsFallen6"<<endl;
				Log->log(14,"**BasicAgent::IsFallen    FALL SIDES !");
				//***//cout<<"\n FALL BAGHAL "<<endl;
				fallGhalt=true;
				return true;
			}

		}
	}
	return false;
}
////////////////////////////////////////////SKILL////////////////////////////////////////////////
////////////////////////////////////////LOWLEVELACTION////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////MIDLEVELACTION///////////////////////////////////////

bool BasicAgent::StandUp()
{
	if(WM->Fback )
		{
			out23<<WM->getCurrentCycle()<<"  StandUp1"<<endl;
			StandBack(true);
		}
	else if(WM->Ffront )
		{
		out23<<WM->getCurrentCycle()<<"  StandUp2"<<endl;
			StandFront(true);
		}
	else if(WM->Fright||WM->Fleft )
		{
		out23<<WM->getCurrentCycle()<<"  StandUp3"<<endl;
		Ghalt(true);
		}
	else
		{
		out23<<WM->getCurrentCycle()<<"  StandUp4"<<endl;
		return false;
		}
}
bool BasicAgent::StandBack(bool  start) {
	static int t1 =0;
	static int t2 =0;
	cout<<"fallllllllllll3"<<endl;
	Log->log(14,"**BasicAgent::StandBack   Standing Back Start !");
	//***//cout<<"state 0 of standing_b "<<state<<endl;
	bool a, b, c, d, e, f, g, h, i, j , k , l, m , n ;
	if (start) {
		//state 1
		if (StandState == 1) {

			a = shoulder(SIDE_LEFT, -120, 0, 40);		b = shoulder(SIDE_RIGHT, -120, 0, 40);
			c = hip		(SIDE_LEFT, 0, 0, 10);		d = hip		(SIDE_RIGHT,0, 0, 10);
			e = ankle	(SIDE_LEFT, 0, 0, 10);		f = ankle	(SIDE_RIGHT,0, 0, 10);
			g = knee	(SIDE_LEFT, 0, 10);			h = knee	(SIDE_RIGHT,0, 10);
			i = elbow	(SIDE_LEFT, -90, 40);			j = elbow	(SIDE_RIGHT, 90,40);
			k =rotateArm(SIDE_LEFT, 90, 40);			l =rotateArm(SIDE_RIGHT,-90, 40);
			m =rotateLeg(SIDE_LEFT, 0, 10);			n =rotateLeg(SIDE_RIGHT, 0, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				//				t1++;
				//				if(t1==5){
				t1=0;
				StandState ++;
				Log->log(14,"**Basic Agent::StandBack   ~~~state= %d True~~~ " ,StandState);
				//				}
			}
			cout<<"StandBack="<<a<<b<<c<<d<<e<<f<<g<<h<<i<<j<<k<<l<<m<<n<<endl;
		}
		//state 2
		else if (StandState == 2) {
			a = shoulder(SIDE_LEFT, -40, 0, 30);		b = shoulder(SIDE_RIGHT, -40, 0, 30);
			c = hip		(SIDE_LEFT, 100, 0, 30);		d = hip		(SIDE_RIGHT,100, 0, 30);
			e = ankle	(SIDE_LEFT, -45, 0, 30);		f = ankle	(SIDE_RIGHT,-45, 0,30);
			g = knee	(SIDE_LEFT, -80, 30);			h = knee	(SIDE_RIGHT,-80, 30);
			i = elbow	(SIDE_LEFT, 0, 30);			j = elbow	(SIDE_RIGHT, 0, 30);
			k =rotateArm(SIDE_LEFT, 0, 30);			l =rotateArm(SIDE_RIGHT,0, 30);
			m =rotateLeg(SIDE_LEFT, -75, 30);			n =rotateLeg(SIDE_RIGHT, -75, 30);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				t2++;
				if(t2==25)
				{
					t2=0;
					StandState ++;

				}
				//				state ++;
				//				state=1;
				Log->log(14,"**Basic Agent::StandBack   ~~~state= %d True~~~ " ,StandState);
				//***//cout<<"state 2:true of standing_b "<<state<<endl;
			}
			Log->log(14,"**Basic Agent::StandBack    State # %d of  Standing FAILED!! ",StandState);
			//***//cout<<"state 2:false of standing_b "<<state<<endl;
		}
		//state 3
		else if (StandState == 3) {
			a = shoulder(SIDE_LEFT,-60, 0, 10);		b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip		(SIDE_LEFT, 70, 10, 10);		d = hip		(SIDE_RIGHT,70, -10, 10);
			e = ankle	(SIDE_LEFT, 45, -15, 10);		f = ankle	(SIDE_RIGHT,45, 15, 10);
			g = knee	(SIDE_LEFT, -115, 10);			h = knee	(SIDE_RIGHT,-115, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT,0, 10);
			m =rotateLeg(SIDE_LEFT, -35, 10);			n =rotateLeg(SIDE_RIGHT, -35, 10);
			if (a && b && c && d && e && f  && i && j && k && l && m && n )
			{
				l_action = l_standing_b;
				StandState++;
				Log->log(14,"**Basic Agent::StandBack   ~~~state= %d True~~~ " ,StandState);
				//***//cout<<"state 3:true of standing_b "<<state<<endl;
			}
			Log->log(14,"**Basic Agent::StandBack    State # %d of  Standing FAILED!! ",StandState);
			//***//cout<<"state 3:false of standing_b "<<state<<endl;
		}
		//state 4
		else if (StandState == 4) {
			a = shoulder(SIDE_LEFT,-60,0, 10);		b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip		(SIDE_LEFT, 70, 10, 10);		d = hip		(SIDE_RIGHT,70, -10, 10);
			e = ankle	(SIDE_LEFT, 35, -15, 10);		f = ankle	(SIDE_RIGHT,35, 15, 10);
			g = knee	(SIDE_LEFT,-80, 10);			h = knee	(SIDE_RIGHT,-80, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT,0, 10);			l =rotateArm(SIDE_RIGHT,0, 10);
			m =rotateLeg(SIDE_LEFT, -15, 10);			n =rotateLeg(SIDE_RIGHT, -15, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				l_action = l_standing_b;
				StandState ++;
				Log->log(14,"**Basic Agent::StandBack   ~~~state= %d True~~~ " ,StandState);
				//***//cout<<"state 4:true of standing_b "<<state<<endl;
			}
			Log->log(14,"**Basic Agent::StandBack    State # %d of  Standing FAILED!! ",StandState);
			//***//cout<<"state 4:false of standing_b "<<state<<endl;

		}
		//state 5
		else if (StandState == 5) {
			a = shoulder(SIDE_LEFT, -60, 0, 10);		b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip		(SIDE_LEFT, 70, 0, 10);		d = hip		(SIDE_RIGHT, 70, 0, 10);
			e = ankle	(SIDE_LEFT, 35, 0, 10);		f = ankle	(SIDE_RIGHT, 35, 0, 10);
			g = knee	(SIDE_LEFT, -80, 10);			h = knee	(SIDE_RIGHT, -80, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT, 0, 10);
			m =rotateLeg(SIDE_LEFT, -15, 10);			n =rotateLeg(SIDE_RIGHT, -15, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				//				t2++;
				l_action = l_standing_b;
				//				if(t2==2){
				StandState ++;
				Log->log(14,"**Basic Agent::StandBack   ~~~state= %d True~~~ " ,StandState);
				//***//cout<<"state 5:true of standing_b "<<state<<endl;
				//				}
			}
			Log->log(14,"**Basic Agent::StandBack    State # %d of  Standing FAILED!! ",StandState);
			//***//cout<<"state 5:false of standing_b "<<state<<endl;

		}
		//state 6
		else if (StandState == 6) {
			a = shoulder(SIDE_LEFT, -60, 0, 10);
			b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip(SIDE_LEFT, 35, 2, 10);
			d = hip(SIDE_RIGHT, 35, -2, 10);
			e = ankle(SIDE_LEFT, 55, -2, 10);
			f = ankle(SIDE_RIGHT, 55, 2, 10);
			g = knee(SIDE_LEFT, -90, 10);
			h = knee(SIDE_RIGHT, -90, 10);
			i = elbow(SIDE_LEFT, 0, 10);
			j = elbow(SIDE_RIGHT, 0, 10);
			if (a && b && c && d && e && f && g && h && i && j )
			{
				l_action=l_actDone;
				StandState =1;
				WM->Fback=false;
				WM->Fright=false;
				WM->Fleft=false;
				Log->log(14,"**Basic Agent::StandBack   ~~~state= %d True~~~ " ,StandState);
				//***//cout<<"state 10:true of standing_b "<<state<<endl;
				t1 = 0;
				Reset_Value();
				//				t2 = 0;
				//				WM->GYRMatrix.setElements(1,0,0,0,1,0,0,0,1);
				//****************//
				return true;
			}
		}
	}
	return false;
}
////////////////////////////////
bool BasicAgent::StandFront(bool  start) {
	static int t1 =0;
	static int t2 =0;
	static int t3 =0;
	static int t4 =0;

	bool a, b, c, d, e, f, g, h, i, j , k , l, m , n ;
	if (start) {

		//state 1
		if (StandState == 1) {
			a = shoulder(SIDE_LEFT, 0, 0, 10);		b = shoulder(SIDE_RIGHT, 0, 0, 10);
			c = hip		(SIDE_LEFT, 0, 0, 10);		d = hip		(SIDE_RIGHT, 0, 0, 10);
			e = ankle	(SIDE_LEFT, 0, 0, 10);		f = ankle	(SIDE_RIGHT, 0, 0, 10);
			g = knee	(SIDE_LEFT, 0, 10);			h = knee	(SIDE_RIGHT, 0, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT, 0, 10);
			m =rotateLeg(SIDE_LEFT, 0, 10);			n =rotateLeg(SIDE_RIGHT, 0, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{

				t4++;
				l_action = l_standing_f;
				if(t4==10)
				{
					t4=0;
					StandState ++;
				}
				cout<<"state 1:true of standing_f "<<StandState<<endl;
			}

			cout<<"state 1:false of standing_f "<<StandState<<endl;

		}
		//state 2
		else if (StandState == 2) {
			a = shoulder(SIDE_LEFT,-20, 0, 40);		b = shoulder(SIDE_RIGHT,-20, 0,40);
			c = hip		(SIDE_LEFT,90,10,40);		d = hip		(SIDE_RIGHT,90,-10, 40);
			e = ankle	(SIDE_LEFT,15,-30,40);		f = ankle	(SIDE_RIGHT,15,30, 40);
			g = knee	(SIDE_LEFT,-125,40);		h = knee	(SIDE_RIGHT,-125, 40);
			i = elbow	(SIDE_LEFT, 0, 40);			j = elbow	(SIDE_RIGHT, 0, 40);
			k =rotateArm(SIDE_LEFT, 0,40);			l =rotateArm(SIDE_RIGHT, 0, 40);
			m =rotateLeg(SIDE_LEFT,-80, 40);		n =rotateLeg(SIDE_RIGHT,-80, 40);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				t3++;
				l_action = l_standing_f;
				if(t3==10)
				{
					t3=0;
					StandState ++;
					cout<<"state 2:true of standing_f "<<StandState<<endl;
				}
			}
			cout<<"state 2:false of standing_f "<<StandState<<endl;
		}
		//state 3
		else if (StandState == 3) {
			a = shoulder(SIDE_LEFT,-25, 0, 10);		b = shoulder(SIDE_RIGHT,-25, 0, 10);
			c = hip		(SIDE_LEFT,70,10, 10);		d = hip		(SIDE_RIGHT,70,-10, 10);
			e = ankle	(SIDE_LEFT,45,-15, 10);		f = ankle	(SIDE_RIGHT,45,15, 10);
			g = knee	(SIDE_LEFT,-115, 10);		h = knee	(SIDE_RIGHT,-115, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT, 0, 10);
			m =rotateLeg(SIDE_LEFT,-35, 10);		n =rotateLeg(SIDE_RIGHT,-35, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				//		t1++;
				//		if(t1==10){
				StandState ++;
				t1=0;
				cout<<"state 3:true of standing_f "<<StandState<<endl;
				//		}
			}
			cout<<"state 3:false of standing_f "<<StandState<<endl;
		}
		//state 8
		else if (StandState == 4) {
			a = shoulder(SIDE_LEFT, 0, 0, 10);		b = shoulder(SIDE_RIGHT, 0, 0, 10);
			c = hip		(SIDE_LEFT,70,10, 10);		d = hip		(SIDE_RIGHT,70,-10, 10);
			e = ankle	(SIDE_LEFT,35,-15, 10);		f = ankle	(SIDE_RIGHT,35,15, 10);
			g = knee	(SIDE_LEFT,-80, 10);		h = knee	(SIDE_RIGHT,-80, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT, 0, 10);
			m =rotateLeg(SIDE_LEFT,-15, 10);		n =rotateLeg(SIDE_RIGHT,-15, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				l_action = l_standing_f;
				StandState ++;
				cout<<"state 4:true of standing_f "<<StandState<<endl;
			}
			cout<<"state 4:false of standing_f "<<StandState<<endl;
		}
		//state 9
		else if (StandState == 5) {
			a = shoulder(SIDE_LEFT, 0, 0, 10);		b = shoulder(SIDE_RIGHT, 0, 0, 10);
			c = hip		(SIDE_LEFT,70, 0, 10);		d = hip		(SIDE_RIGHT,70, 0, 10);
			e = ankle	(SIDE_LEFT,35, 0, 10);		f = ankle	(SIDE_RIGHT,35, 0, 10);
			g = knee	(SIDE_LEFT,-80, 10);		h = knee	(SIDE_RIGHT,-80, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT, 0, 10);
			m =rotateLeg(SIDE_LEFT,-15, 10);		n =rotateLeg(SIDE_RIGHT,-15, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				//		t2++;
				l_action = l_standing_f;
				//	if(t2==20){
				t2=0;
				StandState ++;
				cout<<"state 5:true of standing_f "<<StandState<<endl;
				//	}
			}
			cout<<"state 5:false of standing_f "<<StandState<<endl;
		}
		//state 10
		else if (StandState == 6) {
			a = shoulder(SIDE_LEFT, -60, 0, 10);
			b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip(SIDE_LEFT, 35, 2, 10);
			d = hip(SIDE_RIGHT, 35, -2, 10);
			e = ankle(SIDE_LEFT, 55, -2, 10);
			f = ankle(SIDE_RIGHT, 55, 2, 10);
			g = knee(SIDE_LEFT, -90, 10);
			h = knee(SIDE_RIGHT, -90, 10);
			i = elbow(SIDE_LEFT, 0, 10);
			j = elbow(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT, 0, 10);
			m =rotateLeg(SIDE_LEFT, 0, 10);			n =rotateLeg(SIDE_RIGHT, 0, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				cout<< "this_action="<<this_action<<endl;
				cout<< "last_actionInStand="<<last_action<<endl;
				l_action=l_actDone;
				StandState =1;
				WM->Ffront=false;
				WM->Fback=false;
				WM->Fright=false;
				WM->Fleft=false;
				cout<<"state 6:true of standing_f "<<StandState<<endl;
				//				t1 = 0;
				//				t2 = 0;
				//				t3=0;
				Reset_Value();
				return true;
			}
			cout<<"state 6:false of standing_f "<<StandState<<endl;
		}

	}
	return false;
}
/////////////////////////////////////

bool BasicAgent::Ghalt(bool  start) {
	static int t1 =0;
	static int t2 =0;
	cout<<"fallllllllllll3"<<endl;
	Log->log(14,"**BasicAgent::StandBack   Standing Back Start !");
	//***//cout<<"state 0 of standing_b "<<state<<endl;
	bool a, b, c, d, e, f, g, h, i, j , k , l, m , n ;
	if (start) {
		//state 1
		if(StandState==1){
			m =rotateLeg(SIDE_LEFT, -80, 40);			n =rotateLeg(SIDE_RIGHT, -80, 40);
			if ( m && n )
			{
				Log->log(14,"**BasicAgent::StandBack   Timer=%2.2f",t2);
				//			l_action = l_standing_b;
				StandState ++;
				Log->log(14,"**Basic Agent::StandBack   ~~~state= %d True~~~ " ,StandState);
			}
			Log->log(14,"**Basic Agent::StandBack    State # %d of  Standing FAILED!! ",StandState);
			//***//cout<<"state 1:false of standing_b "<<state<<endl;
		}
		else if (StandState == 2) {
			a = shoulder(SIDE_LEFT, -120, 0,40);		b = shoulder(SIDE_RIGHT, -120, 0, 40);
			c = hip		(SIDE_LEFT, 0, 0, 40);		d = hip		(SIDE_RIGHT, 0, 0, 40);
			e = ankle	(SIDE_LEFT, 0, 0, 40);		f = ankle	(SIDE_RIGHT, 0, 0, 40);
			g = knee	(SIDE_LEFT, 0, 40);			h = knee	(SIDE_RIGHT, 0, 40);
			i = elbow	(SIDE_LEFT, -90, 40);			j = elbow	(SIDE_RIGHT, 90, 40);
			k =rotateArm(SIDE_LEFT, 90, 40);			l =rotateArm(SIDE_RIGHT, -90, 40);
			m =rotateLeg(SIDE_LEFT, 0, 40);			n =rotateLeg(SIDE_RIGHT, 0, 40);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				t2++;
				Log->log(14,"**BasicAgent::StandBack   Timer=%2.2f",t2);
				//***//cout<<"t2ttttttttttttttttt="<<t2<<endl;
				//				l_action = l_standing_b;
				if(t2==20){
					t2=0;
					StandState ++;
					Log->log(14,"**Basic Agent::StandBack   ~~~state= %d True~~~ " ,StandState);
					//***///cout<<"state 1:true of standing_b "<<state<<endl;
				}
			}
			Log->log(14,"**Basic Agent::StandBack    State # %d of  Standing FAILED!! ",StandState);
			//***//cout<<"state 1:false of standing_b "<<state<<endl;
		}
		//state 2
		else if (StandState == 3) {
			a = shoulder(SIDE_LEFT, -40, 0, 30);		b = shoulder(SIDE_RIGHT, -40, 0, 30);
			c = hip		(SIDE_LEFT, 100, 0, 30);		d = hip		(SIDE_RIGHT,100, 0, 30);
			e = ankle	(SIDE_LEFT, -45, 0, 30);		f = ankle	(SIDE_RIGHT,-45, 0, 30);
			g = knee	(SIDE_LEFT, -80, 30);			h = knee	(SIDE_RIGHT,-80, 30);
			i = elbow	(SIDE_LEFT, 0, 30);			j = elbow	(SIDE_RIGHT, 0, 30);
			k =rotateArm(SIDE_LEFT, 0, 30);			l =rotateArm(SIDE_RIGHT,0, 30);
			m =rotateLeg(SIDE_LEFT, -70, 30);			n =rotateLeg(SIDE_RIGHT, -70, 30);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				l_action = l_standing_b;
				StandState ++;
				Log->log(14,"**Basic Agent::StandBack   ~~~state= %d True~~~ " ,StandState);
				//***//cout<<"state 2:true of standing_b "<<state<<endl;
			}
			Log->log(14,"**Basic Agent::StandBack    State # %d of  Standing FAILED!! ",StandState);
			//***//cout<<"state 2:false of standing_b "<<state<<endl;
		}
		//state 3
		else if (StandState == 4) {
			a = shoulder(SIDE_LEFT,-60, 0, 10);		b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip		(SIDE_LEFT, 70, 10, 10);		d = hip		(SIDE_RIGHT,70, -10, 10);
			e = ankle	(SIDE_LEFT, 45, -15, 10);		f = ankle	(SIDE_RIGHT,45, 15, 10);
			g = knee	(SIDE_LEFT, -115, 10);			h = knee	(SIDE_RIGHT,-115, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT,0, 10);
			m =rotateLeg(SIDE_LEFT, -35, 10);			n =rotateLeg(SIDE_RIGHT, -35, 10);
			if (a && b && c && d && e && f  && i && j && k && l && m && n )
			{
				l_action = l_standing_b;
				StandState ++;
				Log->log(14,"**Basic Agent::StandBack   ~~~StandState= %d True~~~ " ,StandState);
				//***//cout<<"state 3:true of standing_b "<<state<<endl;
			}
			Log->log(14,"**Basic Agent::StandBack    State # %d of  Standing FAILED!! ",StandState);
			//***//cout<<"state 3:false of standing_b "<<state<<endl;
		}
		//state 4
		else if (StandState == 5) {
			a = shoulder(SIDE_LEFT,-60,0, 10);		b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip		(SIDE_LEFT, 70, 10, 10);		d = hip		(SIDE_RIGHT,70, -10, 10);
			e = ankle	(SIDE_LEFT, 35, -15, 10);		f = ankle	(SIDE_RIGHT,35, 15, 10);
			g = knee	(SIDE_LEFT,-80, 10);			h = knee	(SIDE_RIGHT,-80, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT,0, 10);			l =rotateArm(SIDE_RIGHT,0, 10);
			m =rotateLeg(SIDE_LEFT, -15, 10);			n =rotateLeg(SIDE_RIGHT, -15, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				l_action = l_standing_b;
				StandState ++;
				Log->log(14,"**Basic Agent::StandBack   ~~~state= %d True~~~ " ,StandState);
				//***//cout<<"state 4:true of standing_b "<<state<<endl;
			}
			Log->log(14,"**Basic Agent::StandBack    State # %d of  Standing FAILED!! ",StandState);
			//***//cout<<"state 4:false of standing_b "<<state<<endl;

		}
		//state 5
		else if (StandState == 6) {
			a = shoulder(SIDE_LEFT, -60, 0, 10);		b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip		(SIDE_LEFT, 70, 0, 10);		d = hip		(SIDE_RIGHT, 70, 0, 10);
			e = ankle	(SIDE_LEFT, 35, 0, 10);		f = ankle	(SIDE_RIGHT, 35, 0, 10);
			g = knee	(SIDE_LEFT, -80, 10);			h = knee	(SIDE_RIGHT, -80, 10);
			i = elbow	(SIDE_LEFT, 0, 10);			j = elbow	(SIDE_RIGHT, 0, 10);
			k =rotateArm(SIDE_LEFT, 0, 10);			l =rotateArm(SIDE_RIGHT, 0, 10);
			m =rotateLeg(SIDE_LEFT, -15, 10);			n =rotateLeg(SIDE_RIGHT, -15, 10);
			if (a && b && c && d && e && f && g && h && i && j && k && l && m && n )
			{
				t1++;
				l_action = l_standing_b;
				if(t1==2){
					StandState ++;
					Log->log(14,"**Basic Agent::StandBack   ~~~state= %d True~~~ " ,StandState);
					//***//cout<<"state 5:true of standing_b "<<state<<endl;
				}
			}
			Log->log(14,"**Basic Agent::StandBack    State # %d of  Standing FAILED!! ",StandState);
			//***//cout<<"state 5:false of standing_b "<<state<<endl;

		}
		//state 6
		else if (StandState == 7) {
			a = shoulder(SIDE_LEFT, -60, 0, 10);
			b = shoulder(SIDE_RIGHT, -60, 0, 10);
			c = hip(SIDE_LEFT, 35, 2, 10);
			d = hip(SIDE_RIGHT, 35, -2, 10);
			e = ankle(SIDE_LEFT, 55, -2, 10);
			f = ankle(SIDE_RIGHT, 55, 2, 10);
			g = knee(SIDE_LEFT, -90, 10);
			h = knee(SIDE_RIGHT, -90, 10);
			i = elbow(SIDE_LEFT, 0, 10);
			j = elbow(SIDE_RIGHT, 0, 10);
			if (a && b && c && d && e && f && g && h && i && j )
			{
				l_action=l_actDone;
				StandState =1;
				WM->Fback=false;
				WM->Fright=false;
				WM->Fleft=false;
				Log->log(14,"**Basic Agent::StandBack   ~~~state= %d True~~~ " ,StandState);
				//***//cout<<"state 10:true of standing_b "<<state<<endl;
				t1 = 0;

				//				t2 = 0;
				return true;
			}

		}
		Log->log(14,"**Basic Agent::StandBack    State # %d of  Standing FAILED!! ",StandState);
		//***//cout<<"state 10:false of standing_b "<<state<<endl;

	}
	return false;
}


/**
 * This method is used to turn the agent to thee specified position
 * @param posTurn Position for the agent to turn to
 * @return bool When the agent has finished turning
 */
bool BasicAgent::turnToPoint(VecPosition posTurn,double e,bool start) {
	if(start){
		AngDeg angToTurn = VecPosition::normalizeAngle((posTurn - WM->getAgentPosition()).getDirection());
		return turnToAngle(angToTurn,e);
	}
	return false;
}

bool BasicAgent::turnToAngle(AngDeg angTurn,double e) {
	AngDeg Angle=VecPosition::normalizeAngle(angTurn - WM->get_Theta());
	Log->log(14,"**BasicAgent::turnToAngle { Angle = %2.2f   Angle Turn = %2.2f   WM->get Theta = %2.2f }",
			Angle , angTurn , WM->get_Theta());
	//***//cout<<"\nAnglem="<<Angle<<"\nangTurn : "<< angTurn <<"\n WM->get_Theta() : "<< WM->get_Theta()<<endl;
	bool b=turnToRelativeAngle(Angle,e);
	Log->log(14,"**BasicAgent::turnToAngle   Turn = %s   ",b);
	//***//cout<<"bturn"<<b<<endl;
	return b;
}
/////////////////////////////////////////////
bool BasicAgent::turnToRelativeAngle(AngDeg angTurn,double e) {
	Log->log(14,"**BasicAgent::turnToRelativeAngle   START !!!! **");
	//***//cout<<"ghesmate 0 = "<<angTurn<<endl;
	if(fabs(angTurn) <= e)
	{
		Log->log(14,"**BasicAgent::turnToRelativeAngle    Turning To Angle %2.2f FINISHED !!! ",angTurn);
		//***//cout<<"ghesmate 1 = "<<angTurn<<endl;
		return true;
	}
	else if(angTurn<-1*e)
	{
		Log->log(14,"**BasicAgent::turnToRelativeAngle    --Angle %2.2f IS NEGATIVE-- ",angTurn);
		//***//cout<<"ghesmate 2:1 = "<<angTurn<<endl;
		if(turn(SIDE_RIGHT))
		{
			Log->log(14,"**BasicAgent::turnToRelativeAngle   Turning To RIGHT--->ANGLE IS %2.2f ",angTurn);
			//***//cout<<"turn is ok."<<endl;
			//***//cout<<"ghesmate 2 = "<<angTurn<<endl;
		}
		else
			Log->log(14,"**BasicAgent::turnToRelativeAngle   Turning o RIGHT FAILED !!!!**");
		return false;
	}
	else if(angTurn>e)
	{
		Log->log(14,"**BasicAgent::turnToRelativeAngle  ++Angle %2.2f IS NOT NEGATIVE ++ ",angTurn);
		//***//cout<<"ghesmate 3:1 = "<<angTurn<<endl;
		if(turn(SIDE_LEFT))
		{
			Log->log(14,"**BasicAgent::turnToRelativeAngle   Turning To LEFT--->ANGLE IS %2.2f",angTurn);
			//***//cout<<"Turn is ok."<<endl;
			//***//cout<<"ghesmate 3 = "<<angTurn<<endl;
		}
		else
			Log->log(14,"**BasicAgent::turnToRelativeAngle   Turning o LEFT FAILED !!!!**");
		return false ;
	}
	return false;
}
//////////////////////////////////////////////////
bool BasicAgent::shoot()
{
	isFirst_shoot = false;
	out23<<WM->getCurrentCycle()<<"  shoot 1"<<endl;
	this_action=shooting;
	AngDeg Angle=WM->getRelativeAngle(OBJECT_BALL)+WM->getJointInstance(JID_HEAD_1)->getCurrentAngle()+2;
	if((Angle<60 && Angle>44)||(Angle>-60 && Angle <-44))
	{
		out23<<WM->getCurrentCycle()<<"  shoot 2"<<endl;
		shootState=1;
	}
	else if (fabs(Angle)<=44)
		{
		out23<<WM->getCurrentCycle()<<"  shoot 3"<<endl;
		shootState=2;
		}

	switch(shootState)
	{
	case 1:
		out23<<WM->getCurrentCycle()<<"  shoot 4"<<endl;
		if(Angle>-60 && Angle<-44)
		{
			out23<<WM->getCurrentCycle()<<"  shoot 5"<<endl;
			walk(0,1,0);
		}
		else if(Angle<60 && Angle>44)
		{
			out23<<WM->getCurrentCycle()<<"  shoot 6"<<endl;
			walk(0,-1,0);
		}
		break;
	case 2:
		out23<<WM->getCurrentCycle()<<"  shoot 7"<<endl;
		if(isFirst_kick)
		{
			out23<<WM->getCurrentCycle()<<"  shoot 8"<<endl;
			if(Angle<0 )
			{

				out23<<WM->getCurrentCycle()<<"  shoot 9"<<endl;
				side_kick = SIDE_RIGHT;
			}
			else
			{
				out23<<WM->getCurrentCycle()<<"  shoot 10"<<endl;
				side_kick = SIDE_LEFT;
			}
		}
		if(kick(side_kick))
		{
			out23<<WM->getCurrentCycle()<<"  shoot 11"<<endl;
			isFirst_shoot = true;
			return true;
		}
		out23<<WM->getCurrentCycle()<<"  shoot 12"<<endl;
		break;
	}
	return false;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*--------------------------  Learning Approcahes  -------------------------*/
/*--------------------------------------------------------------------------*/

/**
 * This method creates, loads and updates learning information and
 * starts the learning processes for agent
 */
bool BasicAgent::initializeLearning() {
	return true;
}

/**
 * This method saves all agent experiences and clears up variables on memory
 */
void BasicAgent::finalizeLearning() {
}

/**
 * This method uses our Genetic Algorithm systems to learn agent to walk
 */
void BasicAgent::walkStraight() {
}

/**
 * This method is used to turn the agent to the specified angle
 * @param angTurn Angle for the agent to turn towards
 * @return bool When the agent has finished turning
 */

//*****************

//***********************************

bool BasicAgent::CloseToBall(double e)
{
	VecPosition Agent,Ball;
	double dist;
	Agent = WM->getAgentPosition();
	Ball  =WM->getRelativePosition(OBJECT_BALL);
	Ball.setVecPosition(Ball[0],Ball[1],Ball[2],POLAR);
	VecPosition Ball1=VecPosition::getVecPositionFromPolar(Ball[0],Ball[1],Ball[2]);
	dist=Ball1.getMagnitude2();
	if(Ball==NULL){
		Ball=WM->getBallPos();
		dist=Agent.getDistanceTo2(Ball);
	}
	Log->log(14,"**BasicAgent::CloseToBall     Distance= %2.2f ", dist );
	//***//cout<<"dist1"<<dist<<endl;
	if(dist < e)
		return true;
	else
		return false;
}
//***********************************

ObjectT
BasicAgent::getNear_Number(VecPosition My)
{
	double AgentDist,OpponentDist;
	static int iNumber,INUM,MinDist;
	INUM=0;
	MinDist=20;
	for(iNumber=2;iNumber<7;iNumber++)
	{
		if(WM->getAgentNumber()==iNumber)
			continue;
		ObjectT obj = SoccerTypes::getTeammateObjectFromIndex (iNumber-1);
		VecPosition u=WM->getGlobalPosition(obj);
		AgentDist=u.getDistanceTo2(My);
		Log->log(14,"**BasicAgent::getNear_Number  DUP =(%2.2f,2.2f,%2.2f) , NUM= %d , AgentDist= %2.2f ",
				u.getX(),u.getY(),u.getZ() , iNumber , AgentDist );
		//***//cout<<"DUP"<<u<<" NUM == "<<iNumber<<"  AGENTDIST == "<<AgentDist<<endl;
		if(AgentDist>1000)
			continue;

		else if(AgentDist<1.5)
		{
			Log->log(14,"**BasicAgent::getNear_Number   AgentDist Is LOW !! ");
			//***//cout<<"LOW"<<endl;
			INUM=iNumber;
			break;
		}
		if(MinDist-AgentDist>0.25)
		{
			MinDist=AgentDist;
			INUM=iNumber;
		}
	}

	//***//cout<<"INUM"<<INUM<<endl;
	if(INUM==0)
	{
		//***//cout<<"NUMBER21"<<endl;
		iNumber=1;
		INUM=0;
		return  OBJECT_TEAMMATE_UNKNOWN;
	}
	if(INUM==2)
	{
		iNumber=1;
		INUM=0;
		//***//cout<<"NUMBER2"<<endl;
		return OBJECT_TEAMMATE_2;
	}
	else if(INUM==3)
	{
		iNumber=1;
		INUM=0;
		//***//cout<<"NUMBER3"<<endl;
		return OBJECT_TEAMMATE_3;
	}
	else if(INUM==4)
	{
		iNumber=1;
		INUM=0;
		//***//cout<<"NUMBER4"<<endl;
		return OBJECT_TEAMMATE_4;
	}
	else if(INUM==5)
	{
		iNumber=1;
		INUM=0;
		//***//cout<<"NUMBER5"<<endl;
		return OBJECT_TEAMMATE_5;
	}
	else if(INUM==6)
	{
		iNumber=0;
		INUM=0;
		//***//cout<<"NUMBER6"<<endl;
		return OBJECT_TEAMMATE_6;
	}
}

//***********************************



void BasicAgent::Say_Pass(ObjectT obj )
{

	if(obj == OBJECT_TEAMMATE_2)
		say("@");
	else if(obj == OBJECT_TEAMMATE_3)
		say("#");
	else if(obj == OBJECT_TEAMMATE_4)
		say("$");
	else if(obj == OBJECT_TEAMMATE_5)
		say("%");
	else if(obj == OBJECT_TEAMMATE_6)
		say("^");
}
//*****************************************/
bool BasicAgent::Reset_Value()
{
	out23<<WM->getCurrentCycle()<<"  Reset_value"<<endl;
	heap=VecPosition(0,0,0.21),ankleLeft=VecPosition(0,0,0),ankleRight=VecPosition(0,0,0);
	walkState = 1;shootState=1;walkPriod=0;kickState=1,TurnState=1;StandState=1;
	GotoPos_State =0;GP_gain=0;gain=0;
	Behind_state = 0;
	checkState=1;
	side_kick = SIDE_RIGHT;
	player = WM->getAgentPosition();
	toop = WM->getBallPos();
	isFirst = true; isFirst_kick = true; isFirst_shoot = true;
//	state=1;//neck////////////////////////////
	this_action = nothing ; 	//abc 14
    KState = 0;TState = 0;
	Speed=0.1;delta=0.1;
	Weight=0;Lenght=0;
	Turn=1;
	ang=0;
	b11 = true;
	result=false;
	rotate=0;
	Gain=0;
//	heap=VecPosition(0,0,0.21),ankleLeft=VecPosition(0,0,0),ankleRight=VecPosition(0,0,0);
//		walkState = 1;shootState=1;walkPriod=0;kickState=1,TurnState=1;
//		Speed=0.1;
//		delta=0.1;
//		Weight=0;
//		Lenght=0;
//		gain=0;
//	    KState = 0;
//		Turn=1;
//		ang=0;
//		TState = 0;
//		b11 = true;
//		GotoPos_State =0;
//		result=false;
//		GP_gain=0;
//		rotate=0;
//		Gain=0;
//		GotoPos_State =0;
//		state = 0;
//		walkSpeed=0.1;
//		deltaSpeed = 0.1;
//		isFirst=true;
//		isFirst_kick=true;
}
bool BasicAgent::Inverse(VecPosition Hip, VecPosition Ankle, SideT side,double speed,double error)
{
	double x=Hip.getX()-Ankle.getX();
	double z=Hip.getZ()-Ankle.getZ();
	double y=Hip.getY()-Ankle.getY();
	AngDeg Q=fabs(acosDeg((pow(x,2)+pow(z,2)+pow(ShankSizeZ,2)-pow(ThighSizeZ,2))/(2*ShankSizeZ*sqrt(pow(x,2)+pow(z,2)))));
	AngDeg E=acosDeg((pow(ShankSizeZ,2)+pow(ThighSizeZ,2)-(pow(x,2)+pow(z,2)))/(2*ShankSizeZ*ThighSizeZ));
	AngDeg S=fabs(atan2Deg(x,z));
	AngDeg Joint2=atan2Deg(y,z)+2;
	AngDeg Joint6=-(Joint2);
	AngDeg Joint3,Joint4,Joint5;
	static AngDeg LJ3,LJ4,LJ5,LJ6,LJ2,RJ3,RJ4,RJ5,RJ2,RJ6;

	if(x<=0)
	{
		out23<<WM->getCurrentCycle()<<"  Invers 1"<<endl;
		if(pow(z-ShankSizeZ,2)==pow(ThighSizeZ,2)-pow(x,2))
		{
			out23<<WM->getCurrentCycle()<<"  Invers 2"<<endl;
			Joint3=0;
			Joint4=180-E;
		}
		else{
			out23<<WM->getCurrentCycle()<<"  Invers 3"<<endl;
			Joint3=Q+S;
			Joint4=-(180-E);

		}
	}
	else if (x>0)
	{
		out23<<WM->getCurrentCycle()<<"  Invers 4"<<endl;
		if(pow(z-ShankSizeZ,2)==pow(ThighSizeZ,2)-pow(x,2))
		{
			out23<<WM->getCurrentCycle()<<"  Invers 5"<<endl;
			Joint3=0;
			Joint4=-(180-E);
		}else
		{
			out23<<WM->getCurrentCycle()<<"  Invers 6"<<endl;
			Joint3=-(S-Q);
			Joint4=-(180-E);
		}
	}
	Joint5=-Joint3-Joint4 +7;

	if(side==SIDE_RIGHT)
	{
		out23<<WM->getCurrentCycle()<<"  Invers 7"<<endl;
		cout<< "RJOINT:J3==\t"<<RJ3<< "\tjoint3==\t"<<WM->getJointInstance(JID_RLEG_3)->getCurrentAngle()<<"\tJ3==\t"<<Joint3<<endl;
		cout<< "RJOINT:J4==\t"<<RJ4<<"\tjoint4==\t"<<WM->getJointInstance(JID_RLEG_4)->getCurrentAngle()<<"\tJ4==\t"<<Joint4<<endl;
		cout<< "RJOINT:J5==\t"<<RJ5<<"\tjoint5==\t"<<WM->getJointInstance(JID_RLEG_5)->getCurrentAngle()<<"\tJ5==\t"<<Joint5<<endl;
		cout<<"RJOINT:J2=="<<RJ2<<"\tjoint2=="<<WM->getJointInstance(JID_RLEG_2)->getCurrentAngle()<<endl;
		cout<<"RJOINT:J6=="<<RJ6<<"\tjoint6=="<<WM->getJointInstance(JID_RLEG_6)->getCurrentAngle()<<endl;
		if(fabs(Joint3-RJ3)>7)
			cout<<"ERROR JOINT3=="<<Joint3-RJ3<<"  J3=="<<RJ3<<"  JOINT3"<<Joint3<<endl;
		if(fabs(Joint4-RJ4)>7)
			cout<<"ERROR JOINT4=="<<Joint4-RJ4<<"  J4=="<<RJ4<<"  JOINT4"<<Joint4<<endl;
		if(fabs(Joint5-RJ5)>7)
			cout<<"ERROR JOINT5=="<<Joint5-RJ5<<"  J5=="<<RJ5<<"  JOINT5"<<Joint5<<endl;
		if(fabs(Joint2-RJ2)>7)
			cout<<"ERROR JOINT2=="<<Joint2-RJ2<<"  J2=="<<RJ2<<"  JOINT2"<<Joint2<<endl;
		if(fabs(Joint6-RJ6)>7)
			cout<<"ERROR JOINT6=="<<Joint6-RJ6<<"  J6=="<<RJ6<<"  JOINT5"<<Joint6<<endl;
		RJ3=Joint3;RJ4=Joint4;RJ5=Joint5;RJ6=Joint6;RJ2=Joint2;
	}
	else
	{
		out23<<WM->getCurrentCycle()<<"  Invers 8"<<endl;
		cout<< "LJOINT:J3==\t"<<LJ3<< "\tjoint3==\t"<<WM->getJointInstance(JID_LLEG_3)->getCurrentAngle()<<"\tJ3==\t"<<Joint3<<endl;
		cout<< "LJOINT:J4==\t"<<LJ4<<"\tjoint4==\t"<<WM->getJointInstance(JID_LLEG_4)->getCurrentAngle()<<"\tJ4==\t"<<Joint4<<endl;
		cout<< "LJOINT:J5==\t"<<LJ5<<"\tjoint5==\t"<<WM->getJointInstance(JID_LLEG_5)->getCurrentAngle()<<"\tJ5==\t"<<Joint5<<endl;
		cout<<"LJOINT:J2=="<<LJ2<<"\tjoint2=="<<WM->getJointInstance(JID_LLEG_2)->getCurrentAngle()<<endl;
		cout<<"LJOINT:J6=="<<LJ6<<"\tjoint6=="<<WM->getJointInstance(JID_LLEG_6)->getCurrentAngle()<<endl;
		if(fabs(Joint3-LJ3)>7)
			cout<<"ERROR JOINT3=="<<Joint3-LJ3<<"  J3=="<<LJ3<<"  JOINT3"<<Joint3<<endl;
		if(fabs(Joint4-LJ4)>7)
			cout<<"ERROR JOINT4=="<<Joint4-LJ4<<"  J4=="<<LJ4<<"  JOINT4"<<Joint4<<endl;
		if(fabs(Joint5-LJ5)>7)
			cout<<"ERROR JOINT5=="<<Joint5-LJ5<<"  J5=="<<LJ5<<"  JOINT5"<<Joint5<<endl;
		if(fabs(Joint2-LJ2)>7)
			cout<<"ERROR JOINT2=="<<Joint2-LJ2<<"  J2=="<<LJ2<<"  JOINT2"<<Joint2<<endl;
		if(fabs(Joint6-LJ6)>7)
			cout<<"ERROR JOINT6=="<<Joint6-LJ6<<"  J6=="<<LJ6<<"  JOINT5"<<Joint6<<endl;
		LJ3=Joint3;LJ4=Joint4;LJ5=Joint5;LJ6=Joint6;LJ2=Joint2;
	}
	//	if(	hip(side,Joint3,Joint2,speed) & knee(side,Joint4,speed)  & ankle(side,Joint5,Joint6,speed))
	if(side==SIDE_LEFT){
		if(LHip_Pitch(Joint3,speed,error)&LHip_Roll(Joint2,speed,error)&LKnee_Pitch(Joint4,speed,error)&LAnkle_Picth(Joint5,speed,error)&LAnkle_Roll(Joint6,speed,error))
		{
			out23<<WM->getCurrentCycle()<<"  Invers 9"<<endl;
			return true;
		}
		else
		{
			out23<<WM->getCurrentCycle()<<"  Invers 10"<<endl;
			return false;
		}
	}
	if(side==SIDE_RIGHT)
	{
		if(RHip_Pitch(Joint3,speed,error)&RHip_Roll(Joint2,speed,error)&RKnee_Pitch(Joint4,speed,error)&RAnkle_Picth(Joint5,speed,error)&RAnkle_Roll(Joint6,speed,error))
		{
			out23<<WM->getCurrentCycle()<<"  Invers 11"<<endl;
			return true;
		}
		else
		{
			out23<<WM->getCurrentCycle()<<"  Invers 12"<<endl;
			return false;
		}
	}
}

bool BasicAgent::Inverse_2(VecPosition  heap,VecPosition ankleLeft, VecPosition ankleRight,double speed,double error)
{
	if(Inverse(heap,ankleLeft,SIDE_LEFT,speed,error)&
			Inverse(heap,ankleRight,SIDE_RIGHT,speed,error))
	{
		out23<<WM->getCurrentCycle()<<"  Invers_2 1"<<endl;

		return true;
	}
	out23<<WM->getCurrentCycle()<<"  Invers_2 2"<<endl;
	return false;
}
//**********************/

bool BasicAgent::reset()
{
	out23<<WM->getCurrentCycle()<<"  Reset1"<<endl;

	int g=30;
	LShoulder_Pitch	(-90 ,g);
	LShoulder_Roll(5,g);
	RShoulder_Pitch(-90,g);
	RShoulder_Roll(-5,g);
	LElbow_Roll(-90,g);
	LElbow_Pitch( -60, g);
	RElbow_Roll(90, g);
	RElbow_Pitch(60,g);
	heap=VecPosition(0,0,0.2);
	ankleLeft=VecPosition(0,0,0);
	ankleRight=VecPosition(0,0,0);
	if(Inverse_2(heap,ankleLeft,ankleRight,8))
	{
		out23<<WM->getCurrentCycle()<<"  Reset2"<<endl;

		return true;
	}

	return false;

}
bool BasicAgent::walk(double ForwardGain,double SideGain,double rotate)//********************//
{
	out23<<WM->getCurrentCycle()<<"  Walk1"<<walkState<<endl;

	cout<<"init:walkState: "<<walkState<< "Wp :"<<walkPriod<<endl;
	switch (walkState)
	{
	case 1:
		out23<<WM->getCurrentCycle()<<"  Walk2"<<walkState<<endl;
		if(reset())
		{
			out23<<WM->getCurrentCycle()<<"  Walk3"<<walkState<<endl;
			gain=ForwardGain/2;
			if(SideGain<0)
			{
				out23<<WM->getCurrentCycle()<<"  Walk4"<<walkState<<endl;
				walkState=2;
			}
			else
				{
				out23<<WM->getCurrentCycle()<<"  Walk5"<<walkState<<endl;
				walkState=3;
				}
			walkPriod = 0;
		}
		break;
	case 2://left
		if(walkPriod==10){
			out23<<WM->getCurrentCycle()<<"  Walk6"<<walkState<<endl;
			gain=ForwardGain;
			walkState= 3;
			walkPriod = 0;
		}
		else
			{
			out23<<WM->getCurrentCycle()<<"  Walk7"<<walkState<<endl;
			return step(gain,SideGain,rotate,SIDE_LEFT);
			}
		break;
	case 3://right
		if(walkPriod==10){
			out23<<WM->getCurrentCycle()<<"  Walk8"<<walkState<<endl;
			gain=ForwardGain;
			walkState= 2;
			walkPriod = 0;
		}
		else
			{
			out23<<WM->getCurrentCycle()<<"  Walk9"<<walkState<<endl;
			return step(gain,SideGain,rotate,SIDE_RIGHT);
			}
		break;
	}
	if(walkPriod==10)
		{
		out23<<WM->getCurrentCycle()<<"  Walk10"<<walkState<<endl;
		Walk_Result=true;
		}
	else

		{
		out23<<WM->getCurrentCycle()<<"  Walk11"<<walkState<<endl;
		Walk_Result=false;
		}
	return false;
}
bool BasicAgent::step(double forwardGain,double sideGain,double rotate,SideT side)
{
	AngDeg Angle;
	VecPosition vec;
	if(walkPriod<2)
	{
		out23<<WM->getCurrentCycle()<<"  Step"<<walkPriod<<endl;

		vec = VecPosition(0,0,Max_Hight);
	}
	else if(walkPriod<8)
	{
		out23<<WM->getCurrentCycle()<<"  Step"<<walkPriod<<endl;
		vec = VecPosition(Max_Lenght*forwardGain,Max_Weight*sideGain,0);//walkback 0.03
		cout<<"GOTOPOS: "<<Angle<<endl;
	}
	else if(walkPriod<10)
	{
		out23<<WM->getCurrentCycle()<<"  Step"<<walkPriod<<endl;
		vec = VecPosition(0,0,-Max_Hight);
	}
	walkPriod ++;
	if(side == SIDE_RIGHT){
		ankleRight += vec;
		if(walkPriod == 5)
		{
			out23<<WM->getCurrentCycle()<<"  Step1"<<walkPriod<<endl;
			ankleRight.setX(ankleLeft.getX());
			if(rotate<=0)
			{
				out23<<WM->getCurrentCycle()<<"  Step2"<<walkPriod<<endl;

				LHip_YawPitch(-7*fabs(rotate),50);
			RHip_YawPitch(-7*fabs(rotate),50);
			}

		}
		if(walkPriod>8 && sideGain<0)
		{
			out23<<WM->getCurrentCycle()<<"  Step3"<<walkPriod<<endl;

			ankleRight.setY(ankleLeft.getY());
		}
	}
	else{
		out23<<WM->getCurrentCycle()<<"  Step4"<<walkPriod<<endl;

		ankleLeft += vec;
		if(walkPriod == 5)
		{
			out23<<WM->getCurrentCycle()<<"  Step5"<<walkPriod<<endl;

			ankleLeft.setX(ankleRight.getX());
			if(rotate>=0)
			{
				out23<<WM->getCurrentCycle()<<"  Step6"<<walkPriod<<endl;

				LHip_YawPitch(-7*rotate,50);
			RHip_YawPitch(-7*rotate,50);
			}
		}
		if(walkPriod>8 && sideGain>0)
			{
			out23<<WM->getCurrentCycle()<<"  Step7"<<walkPriod<<endl;

			ankleLeft.setY(ankleRight.getY());
			}
	}

	heap.setX((ankleLeft+ankleRight).getX()/2);
	heap.setY((ankleLeft+ankleRight).getY()/2);
	heap.setZ(heap.getZ()+vec.getZ()/2);
	cout<<"ANKLE_R=="<<ankleRight<<endl;
	cout<<"ANKLE_L=="<<ankleLeft<<endl;
	cout<<"ANKLE_H=="<<heap<<endl;
	Inverse_2(heap,ankleLeft,ankleRight,50);
	//	if(walkPriod==10)
	//			return true;
	if(walkPriod==5)
	{
		out23<<WM->getCurrentCycle()<<"  Step8"<<walkPriod<<endl;

		return true;
	}
	return false;
}
//****************************************/
//****************************************/

bool BasicAgent::Shoot(SideT side)
{
	switch (shootState)
	{
	case 1:
		if(reset())
		{
			shootState=2;
		}
		break;
	case 2:
		if(kick(side))
		{
			shootState=3;
		}
		break;
	case 3:
		if(reset())
		{
			shootState=1;
			return true;
		}
		break;
	}
	return false;
}
bool BasicAgent::kick(SideT side)
{
	isFirst_kick = false;
	double speed=0;
	double error=-1;

	VecPosition vec_ankle,vec_hip;
	int a;
	if(side==SIDE_LEFT)
		a=1;
	else
		a=-1;
	switch(kickState)
	{
	case 1://
		vec_hip=VecPosition(0,0.01*a,0.005);
		ankleRight=VecPosition(0,0,0);
		ankleLeft=VecPosition(0,0,0);
		speed=10;
		if(KState>8)
		{
			kickState=2;
			KState=0;
		}
		break;
	case 2:
		vec_hip=VecPosition(0,0,0);
		vec_ankle=VecPosition(0,0,0.01);
		speed=10;
		if(KState>2)
		{
			kickState=3;
			KState=0;
		}
		break;
		//	case 3:
		//		vec_hip=VecPosition(0,0,0);
		//		vec_ankle=VecPosition(0,-a*0.01,0);
		//		speed=10;
		//		if(KState>2)
		//		{
		//			kickState=4;
		//			KState=0;
		//		}
		//		break;
	case 3:
		vec_hip=VecPosition(0,0,0);
		vec_ankle=VecPosition(-0.01,0,0);
		speed=10;
		if(KState>7)
		{
			kickState=4;
			KState=0;
		}
		break;
	case 4:
		vec_hip=VecPosition(0,0,0);
		vec_ankle=VecPosition(0,0,0);
		speed=10;
		if(KState>10)
		{
			kickState=5;
			KState=0;
		}
		break;
	case 5:
		vec_hip=VecPosition(0,0,0);
		vec_ankle=VecPosition(0.03,0,0);
		speed=80;
		error=10;
		if(KState>10)
		{
			kickState=6;
			KState=0;
		}
		break;
	case 6:
		vec_hip=VecPosition(0,0,0);
		vec_ankle=VecPosition(-0.025,0,0);
		speed=10;
		if(KState>9)
		{
			kickState=7;
			KState=0;
		}
		break;
		//	case 8:
		//			vec_hip=VecPosition(0,0,0);
		//			vec_ankle=VecPosition(0,0.01*a,0);
		//			speed=10;
		//			if(KState>2)
		//			{
		//				kickState=9;
		//				KState=0;
		//			}
		//			break;
	case 7:
		vec_hip=VecPosition(0,0,0);
		vec_ankle=VecPosition(0,0,-0.01);
		speed=10;
		if(KState>2)
		{
			kickState=8;
			KState=0;
		}
		break;
	case 8://
		vec_hip=VecPosition(0,-0.01*a,-0.005);
		vec_ankle=VecPosition(0,0,0);
		speed=10;
		if(KState>8)
		{
			isFirst_kick = true;
			kickState=1;
			KState=0;
			return true;
		}
		break;
	}
	if(side == SIDE_RIGHT){
		ankleRight += vec_ankle;
	}
	else{
		ankleLeft += vec_ankle;
	}
	heap += vec_hip;
	heap.setX((ankleLeft+ankleRight).getX()/2);
	KState ++;
	Inverse_2(heap,ankleLeft,ankleRight,speed,error);
	return false;
}
//***********************************************
bool BasicAgent::TurntoAngle(AngDeg Angle)
{
	SideT side;
	if(Angle<0)
		side=SIDE_RIGHT;
	else
		side=SIDE_LEFT;
	switch(Turn)
	{
	case 1:
	{
		Turn=2;
		ang=Angle;
	}
	break;
	case 2:
		if(fabs(ang)>5){
			if(turn(side))
			{
				ang=ang-5;
			}
		}
		else
		{
			Turn=1;
			ang=0;
			return true;
		}
	}
	return false;
}
//**********************************

bool BasicAgent::turn(SideT side)
{
	double speed=0;

	VecPosition vec_ankle1,vec_ankle2,vec_hip;//ankle1=Side
	cout<<"Turnstate"<<TurnState<<endl;
	int a;
	if(side==SIDE_LEFT)
		a=1;
	else
		a=-1;
	switch(TurnState)
	{
	case 1://
		//		vec_hip=VecPosition(0,0/*.003*a*/,0);
		//		vec_ankle1=VecPosition(0,0,0);
		//		vec_ankle2=VecPosition(0,0,0);
		heap=VecPosition(0,0,0.21);
		ankleLeft=VecPosition(0,0,0);
		ankleRight=VecPosition(0,0,0);
		speed=30;
		if(TState>2)
		{
			TurnState=2;
			TState=0;
		}
		break;
	case 2:
		vec_hip=VecPosition(0,0,0);
		vec_ankle1=VecPosition(0,0,0.004);
		vec_ankle2=VecPosition(0,0,0);
		speed=30;
		if(TState>3)
		{
			TurnState=3;
			TState=0;
		}
		break;
	case 3:
		vec_hip=VecPosition(0,0,0);
		vec_ankle1=VecPosition(0,0,0);
		vec_ankle2=VecPosition(0,0,0);
		speed=30;
		RHip_YawPitch(-8,50);LHip_YawPitch(-8,50);
		if(TState>3){
			TurnState=4;
			TState=0;
		}

		break;
	case 4 :
		vec_hip=VecPosition(0,0,0);
		vec_ankle1=VecPosition(0,0,-0.004);
		vec_ankle2=VecPosition(0,0,0.0);
		speed=30;
		if(TState>3)
		{
			TurnState=5;
			TState=0;
		}
		break;
		//	case 5 :
		//			vec_hip=VecPosition(0,/*-0.003*a*/0,0);
		//			vec_ankle1=VecPosition(0,0,0);
		//			vec_ankle2=VecPosition(0,0,0);
		//			speed=30;
		//			if(TState>4)
		//			{
		//				TurnState=6;
		//				TState=0;
		//			}
		//			break;
	case 5 :
		vec_hip=VecPosition(0,0,0);
		vec_ankle1=VecPosition(0,0,0);
		vec_ankle2=VecPosition(0,0,0.004);
		speed=30;
		if(TState>3)
		{
			TurnState=6;
			TState=0;
		}
		break;
	case 6:
		vec_hip=VecPosition(0,0,0);
		vec_ankle1=VecPosition(0,0,0);
		vec_ankle2=VecPosition(0,0,0);
		speed=30;
		RHip_YawPitch(0,50); LHip_YawPitch(0,50);
		if(TState>3){
			TurnState=7;
			TState=0;
		}
		break;
	case 7:
		vec_hip=VecPosition(0,0,0);
		vec_ankle1=VecPosition(0,0,0);
		vec_ankle2=VecPosition(0,0,-0.004);
		speed=30;
		if(TState>3)
		{
			TurnState=1;
			TState=0;
			return true;
		}
		break;
	}
	if(side == SIDE_RIGHT){
		ankleRight += vec_ankle1;
		ankleLeft += vec_ankle2;
	}
	else{
		ankleLeft += vec_ankle1;
		ankleRight += vec_ankle2;
	}
	heap += vec_hip;
	heap.setX((ankleLeft+ankleRight).getX()/2);
	heap.setZ(0.21);
	TState ++;
	speed=50;
	Inverse_2(heap,ankleLeft,ankleRight,speed);
	return false;
}
//**********************************
bool BasicAgent::GotoPos(VecPosition goal,double error,double MAX_Gain,double MIN_Gain)
{
//	if(WM->getCurrentCycle() % 250 == 0 || isFirst)
//	{
//		 isFirst=false;
//	}
	out23<<WM->getCurrentCycle()<<"  GotoPos1"<<endl;

//	cout<<"t1estttttt  "<<Agent<<"  "<<error<<endl;
//	player = WM->getAgentPosition();
	double distance= player.getDistanceTo2(goal);
	AngDeg Angle1 =VecPosition::normalizeAngle((goal-player).getDirection()-WM->get_Theta());
	AngDeg Angle2=VecPosition::normalizeAngle(Angle1-180);
	AngDeg MAX_Error;//15-45
	AngDeg Angle;
	cout<<"GOTOPOS=="<<gain<<"\t"<<Angle1<<"\t"<<distance<<endl;
//	if(fabs(Angle1)>120)//?????????
//	{
//		Angle=Angle2;
//		Gain=-gain;
//	}
//	else
//	{
		Angle=Angle1;
		Gain=gain;
//	}
	MAX_Error=35;
	if(distance<0.5)
	{
		out23<<WM->getCurrentCycle()<<"  GotoPos2"<<endl;
		MAX_Error=45;
	}
	if(distance<error)
	{
		out23<<WM->getCurrentCycle()<<"  GotoPos3"<<endl;
		GotoPos_State=0;
		return true;
	}
	switch(GotoPos_State)
	{
	case 0:
		out23<<WM->getCurrentCycle()<<"  GotoPos4"<<endl;
		if(turnToRelativeAngle(Angle,MAX_Error-15)){
			out23<<WM->getCurrentCycle()<<"  GotoPos5"<<endl;
			GotoPos_State=1;
			gain=MIN_Gain;
		}
		out23<<WM->getCurrentCycle()<<"  GotoPos6"<<endl;
		break;


	case 1:
		out23<<WM->getCurrentCycle()<<"  GotoPos7"<<endl;
		if(walk(Gain,0,rotate)){
			out23<<WM->getCurrentCycle()<<"  GotoPos8"<<endl;
			if(fabs(Angle) >= MAX_Error)
			{
				out23<<WM->getCurrentCycle()<<"  GotoPos9"<<endl;
				GotoPos_State=0;
			}
//			if(-Angle<-8)
//				rotate=-1;
//			else if(-Angle>8)
//				rotate=1;
//			else
				rotate=0;
				out23<<WM->getCurrentCycle()<<"  GotoPos10"<<endl;
			if(distance>1.5){
				out23<<WM->getCurrentCycle()<<"  GotoPos11"<<endl;
				if(gain<=MAX_Gain)
				{
					out23<<WM->getCurrentCycle()<<"  GotoPos12"<<endl;
					gain +=delta;
				}
			}
			else if(distance<=1.5)
			{
				out23<<WM->getCurrentCycle()<<"  GotoPos13"<<endl;
				if(gain>=MIN_Gain)
					{
					out23<<WM->getCurrentCycle()<<"  GotoPos14"<<endl;
					gain-=delta;
					}

			}
		}
		break;
	}
	out23<<WM->getCurrentCycle()<<"  GotoPos15"<<endl;
	return false;
}



//////////////////
bool BasicAgent::Rneck_f(bool start) {
	bool a;
	//	static int state=1;
	if (start) {
		//state 1gdfg
		if(state ==1 ){
			a = Head_Yaw(60,10);
			//			neck[0].setAngle(60,10);
			//			a=neck(    0, 0   , 2 );
			if (a )
			{
				Log->log(14,"**BasicAgent::Rneck_f      ~~~State 1 ~~~");
				//***//cout<<"rneck_f 1"<<endl;

				//				l_action = l_RneckF;
				state ++;
			}
			//			return false;
		}
		else if(state ==2 ){
			a= Head_Pitch(-10,30);
			//			a=neck(    0,  60   , 2);
			if (a )
			{
				Log->log(14,"**BasicAgent::Rneck_f    ~~~State 2 ~~~");
				//***//cout<<"rneck_f 2"<<endl;
				//				l_action = l_RneckF;
				state ++;
			}
			return false;
		}
		else if(state ==3 ){
			a=Head_Pitch(0,10);
			//			a=neck(    0, 0   , 2 );
			if (a )
			{
				Log->log(14,"**BasicAgent::Rneck_f    ~~~State 3 ~~~" );
				//***//cout<<"rneck_f 3"<<endl;
				//				l_action = l_RneckF;
				state ++;
			}
			return false;
		}

		if(state ==4){
			a=Head_Yaw(-60,30);
			//			a=neck(    0,   -60, 2 );
			if (a )
			{
				Log->log(14,"**BasicAgent::Rneck_f    ~~~State 4 ~~~");
				//***//cout<<"rneck_f 4"<<endl;
				//				l_action=l_RneckF;
				state=1;
				//				rneck_f = true;
				return true;
			}
			//			return false;
		}
	}
	return false;
}
//******************************************
bool BasicAgent::CheckForKick(VecPosition goal)
{
	if(WM->getCurrentCycle() % 250 == 0 || isFirst)
	{
		out23<<WM->getCurrentCycle()<<"  CheckForKick1"<<endl;
		player = WM->getAgentPosition();
		toop = WM->getBallPos();
		isFirst = false;
	}
	out23<<WM->getCurrentCycle()<<"  CheckForKick2"<<endl;
	double error_ang=25 ;
	double error_move=0.05;
	VecPosition behind = Behind_ofPos_l(toop,goal);
	double distance = toop.getDistanceTo2(player);
	AngDeg angle = VecPosition::normalizeAngle((-player + goal).getDirection()-WM->get_Theta());
	VecPosition ang_agent = player - toop;
	VecPosition ang_behind =behind - toop;

	AngDeg Angle =(ang_agent - ang_behind).getDirection();
	if(fabs(angle) > error_ang)
	{
		out23<<WM->getCurrentCycle()<<"  CheckForKick3"<<endl;
		if(turnToRelativeAngle(angle,error_ang))
			{
			out23<<WM->getCurrentCycle()<<"  CheckForKick4"<<endl;
			checkState= 1;
			}
	}
	switch (checkState)
	{
	case 1:
		out23<<WM->getCurrentCycle()<<"  CheckForKick5"<<endl;
		if(fabs(Angle)>15)
		{
			out23<<WM->getCurrentCycle()<<"  CheckForKick6"<<endl;
			if(Angle>0)
			{
				out23<<WM->getCurrentCycle()<<"  CheckForKick7"<<endl;
				walk(0,1,0);
			}
			else
			{
				out23<<WM->getCurrentCycle()<<"  CheckForKick8"<<endl;
				walk(0,-1,0);
			}
		}
		else
		{
			out23<<WM->getCurrentCycle()<<"  CheckForKick9"<<endl;
			checkState = 2;
		}
		break;
	case 2:
		out23<<WM->getCurrentCycle()<<"  CheckForKick10"<<endl;
		if(distance > error_move)
		{
			out23<<WM->getCurrentCycle()<<"  CheckForKick11"<<endl;
			if(Angle>6)
			{
				out23<<WM->getCurrentCycle()<<"  CheckForKick12"<<endl;
				walk(1,0,1);
			}
			else if(Angle<-6)
			{
				out23<<WM->getCurrentCycle()<<"  CheckForKick13"<<endl;
				walk(1,0,-1);
			}
			else
				{
				out23<<WM->getCurrentCycle()<<"  CheckForKick14"<<endl;
				walk(1,0,0);
				}
		}
		else
		{
			out23<<WM->getCurrentCycle()<<"  CheckForKick15"<<endl;
			return true;

		}
		break;
	default:
		out23<<WM->getCurrentCycle()<<"  CheckForKick16"<<endl;
		checkState=1;
		break;
	}
	return false;
}
//****************************************
//bool BasicAgent::Walk_Ball(VecPosition ball)
//{
//	VecPosition Me=WM->getAgentPosition();
//	VecPosition vec=ball-Me;
//	AngDeg Angle=VecPosition::normalizeAngle(-vec.getDirection()+WM->get_Theta());
//	double r1,r2,r;
//	if(walk((Lenght/Max_Lenght)*Speed,(Weight/Max_Weight)*Speed,0))
//	{
//		cout<<"init:WWWAngle:"<<Angle<<"\t Speed:"<<Speed<<endl;
//		Speed+=delta;
//		if(Speed>1)
//			Speed=1;
//		r1=fabs(Max_Lenght/cosDeg(Angle));
//		r2=fabs(Max_Weight/sinDeg(Angle));
//		if(r1<r2)
//			r=r1;
//		else
//			r=r2;
//		Lenght=r*cosDeg(Angle);
//		Weight=r*sinDeg(Angle);
//	}
//	return false;
//}
//*****************
bool BasicAgent::Front_Back_Walk(double Walk_gain)
{

	if(walk(walkSpeed*Walk_gain,0,0))
	{
		if(fabs(walkSpeed)>1)
		{
			deltaSpeed = -deltaSpeed;
		}
		walkSpeed += deltaSpeed;
		cout<<"front:delta11: "<<deltaSpeed<<"\t walkspeed11: "<<walkSpeed<<endl;
	}
	return false;
}

bool BasicAgent::Front_Back_SWalk(double Side_gain)
{
	if(walk(0,walkSpeed*Side_gain,0))
	{
			if(fabs(walkSpeed)>1)
			{
				delta=-delta;
			}
		walkSpeed += delta;
	}
	return false;
}
//***********************************


//bool BasicAgent::Walk_Target(VecPosition goal)
//{
//	VecPosition Agent=WM->getAgentPosition();
//	VecPosition Ball=WM->getBallPos();
//	VecPosition behind=Behind_ofPos_l(Ball,goal);
//	double distance=Agent.getDistanceTo2(behind);
//	VecPosition vec=(Ball-Agent).getDirection();
//	AngDeg Angle_AB=VecPosition::normalizeAngle(-vec.getDirection()+WM->get_Theta());
//	double r1,r2,r;
//	AngDeg Angle1,Angle2;
//	//		static int walk_target=0;
//	cout<<"target: ANG: "<<Angle<<endl;
//	if(WM->getCurrentCycle() % 250 == 0 || isFirst)
//	{
//		Angle1 =VecPosition::normalizeAngle(-(behind-Agent).getDirection()+WM->get_Theta());
//		Angle2=VecPosition::normalizeAngle(180-Angle1);
//		cout<<"target:Angle1: "<<Angle1<<"\tAngle2: "<<Angle2<<endl;
//		isFirst=false;
//	}
//	if(fabs(Angle1)>100)
//	{
//		Angle=Angle2;
//	}
//	else if(fabs(Angle1)<100)
//	{
//		Angle=Angle1;
//	}
//	if(fabs(Angle)>10)
//	{
//		if(walk((Lenght/Max_Lenght)*Speed,(Weight/Max_Weight)*Speed,0))
//		{
//			cout<<"target:Angle:"<<Angle<<"\t Speed:"<<Speed<<endl;
//			Speed=-delta;
//			if(Speed<0)
//				if(turnToRelativeAngle(Angle,15))
//				{
//					Speed=+delta;
//				}
//			r1=fabs(Max_Lenght/cosDeg(Angle));
//			r2=fabs(Max_Weight/sinDeg(Angle));
//			if(r1<r2)
//				r=r1;
//			else
//				r=r2;
//			cout<<"target:RRRRR"<<r<<endl;
//			Lenght=r*cosDeg(Angle);
//			Weight=r*sinDeg(Angle);
//		}
//	}
//	else
//	{
//		if(distance>1)
//			Walk_Ball();
//	}
//}

bool BasicAgent::Walk_Target(VecPosition goal)
{
	if(WM->getCurrentCycle() % 250 == 0 || isFirst)
	{
		player=WM->getAgentPosition();
		toop=VecPosition(0,-2);
//		toop=WM->getBallPos();
		isFirst=false;
	}
	if(WM->getCurrentCycle() % 50 == 0 || isFirst_theta)
	{
		player_theta = WM->get_Theta();
		isFirst_theta = false;
	}
	VecPosition behind=Behind_ofPos_l(toop,goal);
	double distance=player.getDistanceTo2(behind);
	VecPosition vec=(toop-player).getDirection();
	AngDeg Angle_AB=VecPosition::normalizeAngle(-vec.getDirection()+WM->get_Theta());
	double r1,r2,r;
	AngDeg Angle1,Angle2;
    static AngDeg Angle=0;
	cout<<"target: BALL: "<<WM->getBallPos()<<endl;
     static bool b=false;
		Angle1 =VecPosition::normalizeAngle(-(behind-player).getDirection()+WM->get_Theta());
		Angle2=VecPosition::normalizeAngle(180-Angle1);
			cout<<"target:Angle1: "<<Angle1<<"\tAngle2: "<<Angle2<<endl;
	if(fabs(Angle1)>100)
	{
		Angle=Angle2;
	}
	else if(fabs(Angle1)<100)
	{
		Angle=Angle1;
	}
	if(fabs(Angle)>10)
	{
		if(walk((Lenght/Max_Lenght)*Speed,(Weight/Max_Weight)*Speed,0))
		{
			cout<<"target:Angle:"<<Angle<<"\t Speed:"<<Speed<<endl;
			Speed=-delta;
			if(Speed<0)
			{
			    b=turnToRelativeAngle(Angle,15);
				if(b)
				{
					Speed=+delta;
				}
			}
			if(Speed>1)
				Speed=1;
			r1=fabs(Max_Lenght/cosDeg(Angle));
			r2=fabs(Max_Weight/sinDeg(Angle));
			if(r1<r2)
				r=r1;
			else
				r=r2;
			cout<<"target:RRRRR"<<r<<endl;
			Lenght=r*cosDeg(Angle);
			Weight=r*sinDeg(Angle);
		}
	}
	else
	{
		if(distance>1)
			Walk_Ball();
	}
}




//***************************************
bool BasicAgent::Walk_Ball()
	{
	out23<<WM->getCurrentCycle()<<"  Walk_Ball1"<<endl;
		VecPosition ball=WM->getBallPos();
		VecPosition Agent=WM->getAgentPosition();
		double r1,r2,r;
		AngDeg Angle =VecPosition::normalizeAngle(-(ball-Agent).getDirection()+WM->get_Theta());//ball-agent or agent-ball
		if(walk((Lenght/Max_Lenght)*Speed,(Weight/Max_Weight)*Speed,0))
		{
			out23<<WM->getCurrentCycle()<<"  Walk_Ball2"<<endl;

			Speed+=delta;
			if(Speed>1)
				Speed=1;
			r1=fabs(Max_Lenght/cosDeg(Angle));
			r2=fabs(Max_Weight/sinDeg(Angle));
			if(r1<r2)
				r=r1;
			else
				r=r2;
			Lenght=r*cosDeg(Angle);
			Weight=r*sinDeg(Angle);
		}
	}

//bool BasicAgent::Walk_Target(VecPosition goal)
//{
//	VecPosition Agent=WM->getAgentPosition();
//	VecPosition Ball=WM->getBallPos();
//	VecPosition behind=Behind_ofPos_l(Ball,goal);
//	double distance=Agent.getDistanceTo2(behind);
//	AngDeg Angle1 =VecPosition::normalizeAngle(-(behind-Agent).getDirection()+WM->get_Theta());
//	AngDeg Angle2=VecPosition::normalizeAngle(180-Angle1);
//
//	VecPosition vec=Agent-behind;
//	AngDeg Angle_AB=VecPosition::normalizeAngle(-vec.getDirection()+WM->get_Theta());
//	cout<<"init:Angle_AB: "<<Angle_AB<<"\t Angle: "<<Angle1<<endl;
//	static double Speed_sw=1;
//	static double Speed_w=0.1;
//	static double Speed_wb=1;
//	static int side=0;
//	static int step=2;
//	static bool a = true;
//	static AngDeg Angle=0;
//	double r1,r2,r;
//	cout<<"init:STEP "<<step<<"\t behind"<<behind<<endl;
//	if(WM->getCurrentCycle() % 100 == 0 || a)
//	{
//		cout<<"init:OOOOOOOOOOOO: "<<a<<endl;
//		if(fabs(Angle1)>100)
//		{
//			Angle=Angle2;
////			if(turnToRelativeAngle(Angle2,15))
////			cout<<"init:Angle22: "<<Angle2<<endl;
////			step=1;
//		}
//		else if(fabs(Angle1)>20 && fabs(Angle1)<=100)
//		{
//			cout<<"init:XXXXXXXXXXXXXXXXX: "<<endl;
//			Angle=Angle1;
////			step=1;
//		}
//		a = false;
//	}
//	cout<<"init:DISTANCE: "<<distance<<endl;
//	if(fabs(Angle)>20)
//	{
//		turnToRelativeAngle(Angle2,15);
//	}
//
//	else if(distance>0.5)
//	{
//		if(walk((Lenght/Max_Lenght)*Speed,(Weight/Max_Weight)*Speed,0))
//		{
//			cout<<"init:WWWAngle:"<<Angle<<"\t Speed:"<<Speed<<endl;
//			Speed+=delta;
//			if(Speed>1)
//				Speed=1;
//			r1=fabs(Max_Lenght/cosDeg(Angle));
//			r2=fabs(Max_Weight/sinDeg(Angle));
//			if(r1<r2)
//				r=r1;
//			else
//				r=r2;
//			Lenght=r*cosDeg(Angle);
//			Weight=r*sinDeg(Angle);
//		}
//	}
//	else
//	{
//		if(fabs(Angle_AB)>80)
//		{
//			if(walk(-Speed_wb,0,0))
//			{
//				Speed_wb-=delta;
//				if(Speed_wb<=0)
//				{
//					Speed_wb=1;
//				}
//			}
//			//				step=3;
//		}
//		else if(fabs(Angle_AB)>20 && fabs(Angle_AB)<80)
//		{
//			if(walk(-Speed_wb,0,0))
//			{
//				Speed_wb-=delta;
//				if(Speed_wb<=0)
//				{
//					Speed_wb=1;
//				}
//			}
//			//				step=4;
//		}
//		else if(Ball.getX()> Agent.getX())
//		{
//			if(walk(Speed_w,0,0))
//			{
//				Speed_w+=delta;
//				if(Speed_w>1)
//				{
//					Speed_w=1;
//				}
//			}
//			//				step=5;
//		}
//	}
//}
/*switch(step)
	{
	//	case 1:
	//
	//		if(turnToRelativeAngle(Angle,15))
	//			{
	//			cout<<"init:TTTTTTT: "<<Angle<<endl;
	////			if(reset())
	//			  step=2;
	//			}
	//		break;

	case 2:
			Walk_Ball(behind);
		break;

	case 3:
		cout<<"init: BALL.x: "<<Ball.getX()<<"\t Agent.x: "<<Agent.getX()<<endl;
		//		Speed=0.1;
			if(walk(-Speed_wb,0,0))
			{
				Speed_wb-=delta;
				if(Speed_wb<=0)
				{
					Speed_wb=1;
				}
			}
		break;

	case 4:
				cout<<"init:Speed_sw "<<endl;
				if(walk(0,Speed_sw*side,0))
				{
					if(Angle_AB>0)
						side=1;
					else
						side=-1;
					Speed_sw-=delta;
					if(Speed_sw<=0)
					{
						Speed_sw=1;
				}
			}
				break;

	case 5:
		if(walk(Speed_w,0,0))
		{
			Speed_w+=delta;
			if(Speed_w>1)
			{
				Speed_w=1;
			}
		}
		break;
	}
}*/
